pro readbuf,file,buf,tag=tag,nskip=nskip,point=point,ntag=ntag, $
            err=err,nc=nc,nl=nl,awk=awk
;+
; purpose:    read a multi-dimension data buffer from a named ASCII file
;
; USEAGE:     readbuf,file,buf
;             readbuf,file,buf,tag=tag,ntag=ntag,nskip=nskip,point=point,$
;                     err=err,nc=nc,nl=nl,awk=awk
; input:  
;
;   file      file name. If file name ends in a .gz or .Z suffix, the file
;             is first uncompressed by running it through zcat.
;
;             NOTE: make sure your shell knows where to find the zcat
;                   utility.  Also note that the generic unix version
;                   of zcat does not uncompress files created by the
;                   gnu compression algorithm.  These files usually
;                   have a .gz suffix.  The best bet is to set up a
;                   link in your bin directory pointing to the gnu
;                   version of zcat which uncompresses both .gz and .Z
;                   type compressed files.  (e.g.,
;                   /home/user13/frew/gnu/alpha-osf/bin/zcat)
;
;   buf       a named array or structure set up to match the 
;             data structure of FILE.
;             
;
; optional keyword input:
;
;   tag       string to search before reading begins.
;             READBUF will scan through FILE looking for a line
;             containing the string TAG. Data input starts on the
;             next record (or NSKIP records later, if NSKIP is set)
;
;   ntag      number of times TAG string is searched before reading input
;             (default=1 when TAG is set, otherwise zero)
;
;   nskip     number of records to skip before reading data.
;             If TAG is set, data input starts NSKIP records after the 
;             NTAG'th occurance of the TAG string in the file.
;             If TAG is not set, data input starts NSKIP records 
;             after the begining of the file. (default=0)
;
;   point     byte offset into file after which reading begins
;
;   nc        if set, buf is dimensioned fltarr(nc,nl) where nl is
;             either set with the NL keyword, or is set automatically
;             as the number of lines remaining in the file after tag,
;             ntag and nskip actions.  This option doesn't work when
;             POINT is specified.  Setting nc to zero causes both the
;             number of columns and the number of lines to be set
;             automatically by the file's line and word count (using the
;             unix wc utility). This specification is only useful if the
;             data is in the form of a rectangular array. The nc=0 option
;             does not work with the nskip, point, ntag, and tag options.
;             
;
;   nl        if set buf is dimensioned fltarr(nc,nl) where nl is set
;             with the NL keyword.
;
;   awk       an awk command string used to preprocess the file.  if
;             the input file is compressed (indicated by a .gz or .Z
;             suffix) the results of 'zcat file' is piped into awk.
;             Otherwise, awk reads the input file directly.  In either
;             case the awk command string is used by awk to extract
;             information from file and write the results to
;             readbuf_tmp~.  This temporary file is subsequently read
;             by readbuf, and deleted after processing.
;             
; output:
;
;  buf        data array 
;
; keyword output:
;
;  point      byte offset into file at which reading stopped,
;             can be used as input to next call of READBUF
;
;  err        returns 1 if an io read error is detected.
;             returns 0 if read is successful.
;
; EXAMPLE:
;;       following a bunch of uninteresting output a fortran code 
;;       writes to file junk.dat as follows
;;
;;        write(*,'(a)') 'bufout:'
;;        do 20 k=1,3
;;         do 10 j=1,6
;;           write(*,'(1p11e11.3)') (buf(i,j,k), i=1,11) 
;;    10   continue
;;    20 continue
;;
;;      the buf array could be read with
; 
;        buf=fltarr(11,6,3)
;        readbuf,'junk.dat',buf,tag='bufout:'
; 
;;      if the fortran code listed above were executed twice
;;      it is possible to read the second occurance of buf as follows
; 
;        point=0                              
;        buf0=fltarr(1)
;        readbuf,'junk.dat',buf0,tag='bufout:',point=point 
;        buf=fltarr(11,6,3)
;        readbuf,'junk.dat',buf,tag='bufout:',point=point
; 
;;      In this case the first call to READBUF is used to find the
;;      point (byte offset) in the file just past the first instance
;;      of the string 'bufout:'.  The second call to READBUF starts 
;;      where the previous READBUF left off, scanning the file until
;;      just after the second  occurance of 'bufout:'. At this point
;;      the second buf is read.
; 
;;      another way of doing the same thing is with the NTAG parameter,
; 
;        buf=fltarr(11,6,3)
;        readbuf,'junk.dat',buf,ntag=2,tag='bufout:',
;       
;       NOTE: care must be taken that the TAG string is unique.
;
;; Use awk to preprocess a file (which happens to be compressed)
;; The awk command does the following:
;;
;;  1. change the file seperator character to a comma
;;  2. select records for which column three = 272,273,276 or 277
;;  3. print columns 3,6,7,8 and 9
;
;        awk='BEGIN {FS=","};int($3) ~ /27[2367]/ {print $3 $6 $7 $8 $9}'
;        readbuf,'a_D97.sgp.gz',buf,nc=0,awk=awk
;        print,buf
;
;  author:  Paul Ricchiazzi                            8JAN93
;           Institute for Computational Earth System Science
;           University of California, Santa Barbara
;-


if n_params() eq 0 then begin
  xhelp,'readbuf'
  return
endif  

compressed=strmatch(file,'*.gz') or strmatch(file,'*.Z')

tmpfile='readbuf_tmp~'
if keyword_set(awk) then begin
  if compressed then begin
    cmd='zcat '+file+' | awk ''' + awk + ''' > ' + tmpfile
  endif else begin
    cmd='awk ''' + awk + ''' ' + file +' > ' + tmpfile
  endelse
  delete=1
  ;print,f='(a,$)',cmd
  spawn,cmd
  ;print,f='(2a)',string(13b),string(replicate(32b,strlen(cmd)))
  ifile=tmpfile
endif else begin
  if compressed gt 0 then begin
    cmd='zcat '+file+' > ' + tmpfile
    spawn,cmd
    ifile=tmpfile    
    delete=1
  endif else begin
    ifile=file
    delete=0
  endelse
endelse

line=''

openr,lun,/get_lun,ifile,delete=delete

on_ioerror,bail_out

if keyword_set(ntag) eq 0 then ntag=1
if keyword_set(point) then point_lun,lun,point
;
iline=0
if keyword_set(tag) then begin
  for ntg=1,ntag do begin
    repeat begin
      readf,lun,line 
      iline=iline+1
    endrep until strpos(line,tag) ge 0
  endfor
endif
;
if keyword_set(nskip) then begin
  for i=1,nskip do begin
    readf,lun,line
    iline=iline+1
  endfor
endif
;
if n_elements(nc) ne 0 then begin
  if not keyword_set(nl) then nl=n_lines(ifile)-iline
  case nc of
   0:begin
       nl=n_lines(ifile,nw)
       nc=nw/nl
       buf=fltarr(nc,nl)
     end
   1:buf=fltarr(nl)
   else:buf=fltarr(nc,nl)
  endcase
endif

readf,lun,buf
point_lun,-lun,point
free_lun,lun
on_ioerror,null
;
err=0
return

bail_out:

err=1
free_lun,lun
return
end







