;-------------------------------------------------------------
;+
; NAME:
;       MAGCRS
; PURPOSE:
;       Cursor with a magnified window.
; CATEGORY:
; CALLING SEQUENCE:
;       magcrs, x, y
; INPUTS:
; KEYWORD PARAMETERS:
;       Keywords:
;         /INITIALIZE  Must be done before using or after a
;           color table change.  Must be done for each different
;           window that uses magcrs.
;         STATE=state  Structure with state information.
;           Returned for /INIT.  Must give for each call.
;           Allows multiple mag cursors for multiple windows.
;         MAG=m    Mag factor (def=10).  On /INIT only.
;         SIZE=s   Mag window size (def=200).  On /INIT only.
;         /DATA    Work in data coordinates (default).
;         /DEVICE  Work in device coordinates.
;         /NORMAL  Work in normalized coordinates.
; OUTPUTS:
; COMMON BLOCKS:
; NOTES:
;       Notes: When finished with mag window remove it:
;         wdelete,st.win
;         Can set starting point using tvcrs,x,y.
; MODIFICATION HISTORY:
;       R. Sterner, 1996 Feb 26
;
; Copyright (C) 1996, Johns Hopkins University/Applied Physics Laboratory
; This software may be used, copied, or redistributed as long as it is not
; sold and this copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied warranties
; whatsoever.  Other limitations apply as described in the file disclaimer.txt.
;-
;-------------------------------------------------------------
	pro magcrs_up, x, y, st
 
	;---------  Read cursor and display magnified view  ----------
	t = tvrd2(x-st.rdsz2,y-st.rdsz2,st.rdsz,st.rdsz)
	t = rebin(t,st.wsz,st.wsz,/samp)	; Magnify.
	it = t(st.xmid+1,st.ymid+1)             ; Mid pixel.
        if st.lum(it) lt 128 then cc=st.br $
           else cc=st.dr                        ; Outline color.
        t(st.xmid:st.xmid+st.mag,st.ymid)=cc    ; Draw mid pixel outline.
        t(st.xmid:st.xmid+st.mag,st.ymid+st.mag)=cc
        t(st.xmid,st.ymid:st.ymid+st.mag)=cc
        t(st.xmid+st.mag,st.ymid:st.ymid+st.mag)=cc
	;------  Display magnified view  ------
	wset,st.win
	device,get_graphics=gmode	; Need Graphics mode 3.  Get current.
	device,set_graphics=3		; Set mode 3.
	tv,t
	device,set_graphics=gmode	; Restore old mode.
	wset,st.win0
 
	return
	end
 
;====================================================
;	magcrs.pro = Mag window cursor
;	R. Sterner, 1996 Feb 23
;====================================================
 
	pro magcrs, x0, y0, initialize=init, state=st, mag=mag0, $
	  size=msize0, data=dat0, device=dev0, normal=nrm0, help=hlp
 
	if keyword_set(hlp) then begin
	  print,' Cursor with a magnified window.'
	  print,' magcrs, x, y'
	  print,'   x,y = cursor position.    in,out'
	  print,' Keywords:'
	  print,'   /INITIALIZE  Must be done before using or after a'
	  print,'     color table change.  Must be done for each different'
	  print,'     window that uses magcrs.'
	  print,'   STATE=state  Structure with state information.'
	  print,'     Returned for /INIT.  Must give for each call.'
	  print,'     Allows multiple mag cursors for multiple windows.'
	  print,'   MAG=m    Mag factor (def=10).  On /INIT only.'
	  print,'   SIZE=s   Mag window size (def=200).  On /INIT only.'
	  print,'   /DATA    Work in data coordinates (default).'
	  print,'   /DEVICE  Work in device coordinates.'
	  print,'   /NORMAL  Work in normalized coordinates.'
	  print,' Notes: When finished with mag window remove it:'
	  print,'   wdelete,st.win'
	  print,'   Can set starting point using tvcrs,x,y.'
	  return
	endif
 
	;-------  Initialize  -------------------
	if keyword_set(init) then begin
	  tvlct,r,g,b,/get
	  lum = .3 * r + .59 * g + .11 * b
	  br = (where(lum eq max(lum)))(0)
	  dr = (where(lum eq min(lum)))(0)
 
          if n_elements(mag0) eq 0 then mag0=1          ; Force defined.
          if n_elements(msize0) eq 0 then msize0=200    ; Def mag win size.
          msize = round(msize0)                         ; Rounded size.
          mag = round(mag0)                             ; Rounded mag.
          if mag eq 1 then mag=10                       ; Def is 10.
          rdsz = round(float(msize)/mag)                ; Read size.
          rdsz2 = rdsz/2                                ; Offset.
          xmid = rdsz2*mag                              ; Mag win midpoint.
          ymid = rdsz2*mag
          wsz = rdsz*mag                                ; True mag win size.
 
	  st = {lum:lum, br:br, dr:dr, mag:mag, rdsz:rdsz, rdsz2:rdsz2, $
	    wsz:wsz, xmid:xmid, ymid:ymid, win:29, win0:!d.window}
 
	  if n_params(0) eq 0 then return		; Just init.
	endif
	;--------------------------------------
 
	;-------  Check that state structure is given  ---------
	if n_elements(st) eq 0 then begin
	  print,' Error in magcrs: Need to give state structure.  This'
	  print,'   structure is returned on /INIT and must be sent for'
	  print,'   each call to magcrs afterward: magcrs,x,y,state=st.'
	  return
	endif
 
	;--------  Take care of some entry items  ---------
	;-----  Coordinates  -------------
        dat = keyword_set(dat0)
        dev = keyword_set(dev0)
        nrm = keyword_set(nrm0)
        if dat+dev+nrm gt 1 then begin
          print,' Error in lini: set only one of /DATA, /DEVICE, or /NORMAL.'
          return
        endif
        if dat eq 0 then dat=1-(dev>nrm)        ; Def is /data.
	if dat eq 1 then begin
	  if !x.s(1) eq 0 then begin
	    print,' Error in magcrs: data coordinates not yet established.' 
            print,'  Must make a plot before calling magcrs or use /DEVICE
            print,'   or /NORMAL keyword.' 
            stop 
	  endif
	endif
	;-----  Cursor position (from entry coordinates to device) ---------
        if dat then begin
          tmp = round(convert_coord(x0,y0,/data,/to_dev))
          x=tmp(0,0) & y=tmp(1,0)
        endif
        if dev then begin
          x=round(x0) & y=round(y0)
        endif
        if nrm then begin
          tmp = round(convert_coord(x0,y0,/norm,/to_dev))
          x=tmp(0,0) & y=tmp(1,0)
        endif
	;----  Mag window  --------
	device,window_state=wstate			; Any mag window?
	if wstate(st.win) eq 1 then begin		; Yes.
	  mwin=st.win					; Get its number.
	  st.win = mwin					; Update state.
	endif else begin 				; No.
	  window, /free, xs=st.wsz, ys=st.wsz, $	; Make one.
	  title='Mag: '+strtrim(st.mag,2)
	  mwin = !d.window				; Get its number.
	  st.win = mwin					; Update state.
	  wset, st.win0
	  magcrs_up, x, y, st
	endelse
 
	;---------  Read cursor and display magnified view  ----------
	wset, st.win0
	cursor,x,y,2,/device
	magcrs_up, x, y, st
 
	;-------  Return last coordinates in entry coord system  ----------
        if dat then begin
          tmp = convert_coord(x,y,/dev,/to_data)
          x0=tmp(0,0) & y0=tmp(1,0)
        endif
        if dev then begin
          x0=x & y0=y
        endif
        if nrm then begin
          tmp = convert_coord(x,y,/dev,/to_norm)
          x0=tmp(0,0) & y0=tmp(1,0)
        endif
 
	return
	end
