;+NAME/ONE LINE DESCRIPTIONG OF ROUTINE:
;    LONLATTIME2ES calculates the theoretical downwelling irradiance.
; 
;==========================================================================
; function lonlatime2es
;
; Inputs:
;   lon  - scalar or vector of longitude at each observation (deg)
;   lat  - scalar or vector of latitude at each observation  (deg)
;   year - scalar or vector of year at each observation
;   day  - scalar or vector of day at each observation
;   msec - scalar or vector of millisecs of day at each observation
;
; Optional Keyword Inputs:
;   F0=F0       - solar irradiance at each band (mW/cm^2/um/sr),
;                 default is for the 8 SeaWiFS band passes.
;   k_oz=koz    - ozone absorption coefficients at each band (cm^-1),
;                 default is for the 8 SeaWiFS band passes.
;   oz=oz       - ozone concentration at each observation (cm),
;                 default is 0.25.
;   Tau_r=Tau_r - Rayleigh optical thickness at each band.
;                 default is for the 8 SeaWiFS band passes.
;   band=band   - scalar or vector of band numbers to calculate for,
;                 default is [0,1,2,3,4,5,6,7].
;   octs=octs   - switch default to 8 OCTS bands if octs=1.
;
; Outputs:
;   Es - Theoretical surface irradiance at each observation, for each band
;        (mW/cm^2/um/sr) is returned by the function.
;
; Written By:
;   B. A. Franz, SAIC GSC, NASA/SIMBIOS Project, December 1998.
;
;==========================================================================

function lonlattime2es, lon, lat, year, day, msec, $
                        F0=F0,oz=oz,k_oz=k_oz,Tau_r=Tau_r,band=band, $
                        octs=octs

    if (n_elements(k_oz) eq 0) then $
      if (keyword_set(octs)) then $
        k_oz   = [0.00153,0.00436,0.02497,0.04698,0.11117,0.04735,0.00827,0.00373] $
      else $
        k_oz   = [0.00103,0.00400,0.02536,0.04200,0.09338,0.04685,0.00837,0.00485]

    if (n_elements(Tau_r) eq 0) then $
      if (keyword_set(octs)) then $
        Tau_r  = [0.3131,0.23335,0.1557,0.1266,0.0859,0.0440,0.0256,0.0158] $
      else $
        Tau_r  = [0.3132,0.2336,0.1547,0.1330,0.09475,0.0446,0.0256,0.0169]

    if (n_elements(F0) eq 0) then $
      if (keyword_set(octs)) then $
        F0     = [171.06,188.48,194.56,185.59,183.33,152.43,122.39,98.48] $
      else $
        F0     = [170.79,189.45,193.66,188.35,185.33,153.41,122.24,98.82]

    if (n_elements(oz) eq 0) then $
        oz = 0.25


    if (n_elements(band) eq 0) then begin
        nb   = n_elements(F0)
        band = indgen(nb)
    endif else begin
        nb   = n_elements(band)
    endelse

    ns = n_elements(day)
    Es = fltarr(nb,ns)

    lonlat2solz, lon, lat, year, day, msec, solz

    Tau_oz = k_oz * oz
    mu0    = cos(!pi/180.0*solz)
    t0     = exp(-(0.5*Tau_r + 0.008 + Tau_oz)/mu0)
    rday   = day + double(msec)/86400.D3
    fday   = fsol(rday)

    for i=0,nb-1 do begin
        ib = band(i)
        Es(ib,*) = F0(ib) * t0(ib) * mu0 * fday
    endfor

    return,Es
end
