;+NAME/ONE LINE DESCRIPTIONG OF ROUTINE:
;    NLW_OUTBAND converts band-weighted Lwn to Lwn at the nominal band center.
;
;FUNCTION NLW_Outband
;
; This function implements the SeaWiFS out-of-band correction.  The function
; converts band-weighted Lwn to Lwn at the nominal band center.
;
; Inputs:
;    nlw        Array of band-weighted Lwn
; Outputs:
;    lwn        Array of band-centered Lwn
;
; The out-of-band correction has the form of:
;   f = A0 + A1*ratio + A2*ratio^2
; where raio is the ratio of Lwn in either of bands 1-3 to the Lwn in band 5.
; The correction factors can be computed for the Band 1 to Band 5 ratio, the 
; Band 2 to Band 5 ratio, or the Band 3 to Band 5 ratio.  Currently, the 
; function uses the Band 3 to Band 5 ratio.
;
;
; This function has been adapted from the MSL12 routine nlw_outband.f.
;
; Author:  Gene Eplee
;          SAIC General Sciences Corporation
;          301-296-0953
;          5 March 2001
;
; Modification history:
;
;------------------------------------------------------------------------------

FUNCTION NLW_Outband,nlw

;
; Define the correction coefficients.
;
   A0 = DBLARR(6,3)
   A0(*,0) = [1.0019d, 0.99506d, 1.0015d, 1.0242d, 1.0228d, 0.95896d]
   A0(*,1) = [1.0019d, 0.99521d, 1.0022d, 1.0238d, 1.0222d, 0.95763d]
   A0(*,2) = [1.0019d, 0.99394d, 0.9891d, 1.0189d, 1.0206d, 0.95780d]
 
   A1 = DBLARR(6,3)
   A1(*,0) = [8.037d-05, 0.0021183d, 0.0070173d, -0.0082834d, -0.011199d, -0.025845d]
   A1(*,1) = [5.2879d-05, 0.0019534d, 0.0062889d, -0.0079055d,-0.01067d, -0.02444d]
   A1(*,2) = [0.00018485d, 0.002471d, 0.017407d, 0.0010734d,-0.003063d, -0.011993d]
 
   A2 = DBLARR(6,3)
   A2(*,0) = [-2.6d-05, -0.00011572d, -0.00065452d, 8.2237d-05, 0.00015839d, 0.00059993d]
   A2(*,1) = [-1.9865d-05, -0.00010031d, -0.00054018d, 0.00010971d, 0.00018511d, 0.00060593d]
   A2(*,2) = [-8.8191d-05, -3.3504d-05, -0.0025598d, -0.0029009d, -0.0030067d, -0.0053342d]

;
; Set the band ratio to use in the correction
;
   iratio = 2

;
; Compute the band 3 to band 5 Lwn ratio:
;
   IF (nlw(2) LT 0.0) THEN BEGIN
      PRINT,''
      PRINT,'NLW_Outband:  Negative Lwn in Band 3.'
      PRINT,''
      RETURN,-2
   ENDIF
   IF (nlw(4) LT 0.0) THEN BEGIN
      PRINT,''
      PRINT,'NLW_Outband:  Negative Lwn in Band 5.'
      PRINT,''
      RETURN,-2
   ENDIF
   ratio = nlw(2)/nlw(4)

;
; Compute the out-of-band correction
;
   F = A0(*,iratio) + A1(*,iratio)*ratio + A2(*,iratio)*ratio*ratio
   lwn = nlw*F


RETURN,lwn
END
