;+NAME/ONE LINE DESCRIPTION OF ROUTINE
;    RAY_RAD computes Rayleigh radiances for SeaWiFS using Rayleigh tables.

;*******************************************************************************
;   NAME:
;       RAY_RAD
;
;   PURPOSE:
;       Compute Rayleigh radiances for SeaWiFS using the operational Rayleigh
;       tables, with inputs of viewing and sun geometries and atmo. pressure
;
;   CALLING SEQUENCE:
;       RESULT = RAY_RAD(Pres,THETA0,THETA,PHI,Wavelength,pix,line)
;           Pres = array of atmospheric pressures
;           THETA0 = array of solar zenith angle
;           THETA = array of satellite zentih angle
;           PHI = array of relative azimuth angle between sun and sensor
;           wavelength = wavelength for which Rayleigh radiances are desired
;           pix = number of pixels for SeaWiFS file of interest
;           line = number of lines for SeaWiFS file of interest
;           day = day of year, (added for earth-sun distance correction of F0)
;
;   OUTPUT:
;       an array of (pix x line) with Rayleigh radiances corresponding to the 
;       input geometry
;
;       THIS INTERPOLATES USING THE TABLES OF RAYLEIGH SCATTERING
;       FOR SUN AND VIEWING ANGLES WHICH ARE NOT COVERED IN THE
;       TABLES. IT USES 2-D LINEAR INTERPOLATION.  
;
;       ALL ANGLES ARE IN DEGREES
;   NOTE:  This is an IDL version of a FORTRAN routine written by 
;          Menghua Wang.  For the most part, I've tried to get the routine
;          to use IDL's array-based processing, except for the ray_ang indexing
;          there may be a way to do it, I just gave up...for now.
;   ALSO NOTE: There is a hard-coded directory location,this will need to be 
;           modified for your SeaDAS location
;       
;   Translated by:  Sean Bailey, Futuretech Corportaion, April 1999 
;
;   Modifications:
;       19990504    Added earth-sun distance correction to F0BAR value
;
;*******************************************************************************
FUNCTION RAY_RAD, Pres, THETA0, THETA, PHI, wavelength, pix, line,day


        
NSUN_RAY = 45
NRAD_RAY = 41
NORDER_RAY = 3
NLAMBDA = 8
LAMBDAS= [412,443,490,510,555,670,765,865]

RAY_ANG = FLTARR(NRAD_RAY)
RAY_SUN = FLTARR(NSUN_RAY)
RAY_FOR_I= FLTARR(NRAD_RAY, NORDER_RAY, NSUN_RAY)
RAY_I = FLTARR(pix,line)
;Solar Constants for SeaWiFS wavelengths
F0 = fltarr(NLAMBDA)
F0 = [170.79, 189.45, 193.66, 188.35, 185.33, 153.41, 122.24, 98.82]
koz = [1.03,4.0,25.36,42.,93.38,46.85,8.37,4.85]
PRES0 = 1013.25

;Correction for relative azimuth angle, code is picky about the angles...
;This is known to work for the 'relative azimuth' field for a SeaWiFS L1B file
PHI = PHI + 180.
phi_idx = where(PHI lt -180.)
if (phi_idx[0] ge 0.) THEN PHI(phi_idx) = PHI(phi_idx) + 360.
phi_idx = where(PHI gt 180.)
if (phi_idx[0] ge 0.) THEN PHI(phi_idx) = PHI(phi_idx) - 360.
        
        
;Directory for rayliegh tables...
;NEEDS TO BE POINTED AT THE RIGHT PLACE!
DirName ='/disk2/seadas/data/dsp/cal/'
    INF = strarr(NLAMBDA)
    INF(0) = 'sea412pol.new'
    INF(1) = 'sea443pol.new'
    INF(2) = 'sea490pol.new'
    INF(3) = 'sea510pol.new'
    INF(4) = 'sea555pol.new'
    INF(5) = 'sea670pol.new'
    INF(6) = 'sea765pol.new'
    INF(7) = 'sea865pol.new'
    
    INFIL = strarr(NLAMBDA)
    FOR i = 0, NLAMBDA -1 DO INFIL(i) = DirName + INF(i)
    
MRAD    = NRAD_RAY
MORDER  = NORDER_RAY
MSUN    = NSUN_RAY
ray_for = fltarr(MRAD)
MLAMBDA = NLAMBDA

;Here's where we read in the information from the approriate table
i = where(lambdas eq wavelength)

OPENR, LUN, INFIL(i), /GET_LUN
READF, LUN, RAY_ANG

FOR j = 0, MSUN-1 DO BEGIN
    READF, LUN, sun_j,tau_i,dep_i
    RAY_SUN(j) = sun_j
    RAY_TAU = tau_i
    RAY_DEP = dep_i
    
    READF, LUN, RAY_FOR
    RAY_FOR_I(*,0,j)  = ray_for
    
    READF, LUN, RAY_FOR
    RAY_FOR_I(*,1,j)  = ray_for
    
    READF, LUN, RAY_FOR
    RAY_FOR_I(*,2,j)  = ray_for
ENDFOR  
CLOSE, LUN
           

;       FIGURE OUT THE INDEX TO LOCATE PROPER PAIR OF SUN ANGLES
    ISUN_LOW = intarr(pix,line)
        ISUN_HIGH = intarr(pix,line)
        ISUN_LOW  = FIX(THETA0/2) ;+ 1
        ISUN_HIGH = ISUN_LOW + 1
 
;    FIND THE VIEWING ANGLES FOR INTERPOLATION
    ;J = intarr(pix,line,NRAD_RAY)
    J1 = intarr(pix,line)
    J2 = intarr(pix,line)
    FOR k = 0, pix-1 DO BEGIN
        FOR n = 0, line-1 DO BEGIN
            J = where(ray_ang ge theta(k,n))
            J2(k,n) = J[0]
            J1(k,n) = J2(k,n) -1
        ENDFOR
    ENDFOR 
    ;J(*,*,*) = where(ray_ang ge theta)
    ;J2 = FIX(theta/2)
    ;J1 = J2 -1 

;    THETA IS BETWEEN RAY_ANG(J1) AND RAY_ANG(J2)

;    SET UP THE VARIABLES REQUIRED FOR INTERPOLATION

HSUN = fltarr(pix,line)
HVIEW = fltarr(pix,line)
P = fltarr(pix,line)
Q = fltarr(pix,line)
RAJ1 = RAY_ANG(J1)
RAJ2 = RAY_ANG(J2)
RSH = RAY_SUN(ISUN_HIGH)
RSL = RAY_SUN(ISUN_LOW)

HSUN  = RSH - RSL
HVIEW = RAJ2 - RAJ1
P     = ( THETA0 - RSL )/HSUN
Q     = ( THETA - RAJ1 )/HVIEW
        
F00 = fltarr(pix,line)
F10 = fltarr(pix,line)
F01 = fltarr(pix,line)
F11 = fltarr(pix,line)
VALUE = fltarr(pix,line)
RAY_I(*) = 0.
; OK, now interpolate
FOR m = 0, 2 DO BEGIN 
    marr = intarr(pix,line) + m  
    F00 = RAY_FOR_I( J1, marr, ISUN_LOW) ; SEE ABROMOWITZ AND STEGUN
    F10 = RAY_FOR_I( J1, marr, ISUN_HIGH) ; HANDBOOK OF MATHEMATICAL
    F01 = RAY_FOR_I( J2, marr, ISUN_LOW) ; FUNCTIONS. PAGE 882        
    F11 = RAY_FOR_I( J2, marr, ISUN_HIGH) ; FORMULA 25.2.67
    VALUE = (1.-P)*(1.-Q)*F00 + P*Q*F11 + P*(1.-Q)*F10 + Q*(1.-P)*F01
    RAY_I = (VALUE * COS((2*!PI*(PHI * m)/360)) + RAY_I)
    
ENDFOR
   
; Correction of Rayleigh radiances for atmospheric pressure
pp0 = fltarr(pix,line)
fac = fltarr(pix,line)
tauray = fltarr(pix,line)
pp0 = pres / pres0 
pplt0 = where(pp0 le 0.0)          
if (pplt0[0] gt 0.0) THEN pp0(pplt0) = 1.0
tauray(*,*) = pp0(*,*)*ray_tau
fac(*,*) = ((1.0-exp(-tauray(*,*)/cos(!PI*theta/180))) / (1.0-exp(-ray_tau/cos(!PI*theta/180))))
ray_i = ray_i*fac
     
; Include the SeaWiFS solar constants 
i = where(lambdas eq wavelength)
FoBAR = fltarr(pix,line)
A=1.00014
B=0.01671
C=0.9856002831
D=3.4532868
E=360.0
F1 = 1/((A - B * cos(2.*!PI*(C*day-D)/E) - 0.00014*cos(4.*!PI*(C*day-D)/E))^2)
FoBAR(*,*) = F0(i)*F1
RAY_I = RAY_I * FoBAR
;Oz = fltarr(pix,line)
;Oz(*,*) = koz(i)
;tauOz = Oz*Ozone
;RAY_I = RAY_I * EXP(tauOz*((1/cos(180/!PI*theta))+(1/cos(180/!PI*theta0))))
CLOSE,/ALL
RETURN, RAY_I
    
END


