;
;+NAME/ONE LINE DESCRIPTION OF ROUTINE:    
;     PSTEK2SD sets paper size and displays (tvscl) image in postscript 
;     with margins for the tek2sd printer.
;
;DESCRIPTION: 
;     Sets up an optimised display size for post_script output, 
;     proportional to the ratio of the x/y axes of the image; then 
;     displays the input array in this window, using the IDL display 
;     procedure 'tvscl'. The 'optimised' window selects the largest 
;     possible display size for letter size post-script printout, 
;     keeping the x/y ratio of the image the same as the original. The 
;     image is rotated if y is larger than x, before display. 'tvscl' 
;     which is used to display the image,defaults to the maxima and 
;     the minima of the image, unless otherwise specified by the user. 
;     Optional output file name can be put in if desired filename is 
;     other than 'idl.ps'. An optional scaling parameter can be used 
;     if the output should be less than full letter size. The output 
;     file can be sent directly to a postscript printer.
;
;CALLING SEQUENCE: 
;     PSTEK2SD, image [,min] [,max] [,OUTPUT=output] [,SCALE=scale] 
;              [,COLOR=color] [,CTABLE=ctable] [,/HSV] [,/HLS]
;
;ARGUMENTS (I = input, O = output, [] = optional): 
;     IMAGE     I   flt [arr]  Name of input array to be displayed.    
;     MIN      [I]  flt        User specified minima for tvscl 
;                              display; Default is minima of array.  
;     MAX      [I]  flt        User specified maxima for tvscl 
;                              display; Default is maxima of array.   
;     OUTPUT   [O]  str        If specified, 'output' is the name for  
;                              postscript format output. Default is 
;                              'idl.ps'. The postscript format output
;                              file can be sent to a postscript 
;                              printer directly.
;     SCALE    [I]  flt        If specified (has to be less than 1.0), 
;                              the image covers less than the full size 
;                              letter page. Default is SCALE = 1.0.
;     COLOR    [I]  int        Number of IDL color table to use for
;                              color output. PS file must be sent to
;                              a color printer. Set to -1 to use pre-
;                              defined PS color table.
;     R(G,B)COLOR [I]  intarr  RGB colors to be used for defining output
;                              color table in TVLCT. PS file must be sent to 
;                              a color printer. Not compatible with COLOR
;                              keyword.
;     HSV      [I]  int        Set this keyword if using HSV color system
;                              for input to TVLCT.
;     HLS      [I]  int        Set this keyword if using HLS color system
;                              for input to TVLCT.
;
;WARNINGS:
;
;     1.   Scaling Factor has to be less than 1.0, as this routine is
;          set for maximum display size. 
;     2.   The default procedure with no optional keywords will produce
;          a black and white plot.
;     3.   The procedure sets the device type back to the previous one
;          if the device type was other than post-script, ex. 'X', 
;          'tek' or 'regis'. But if the previous device was also 'ps',
;          then the changed x/y sizes and offsets are retained. The 
;          user should remember to reset 'ps' device parameters to 
;          those desired, unless psimage is used again to optimize
;          the display scale.          
;     4.   The output file is created in the default or current 
;          directory. As postscript files may be quite large, the 
;          user should make sure there is enough available space.
;     5.   The postscript output uses 8 bits_per_pixel resolution.
;
;EXAMPLE: 
;     1. To produce a balck and white display for image image1, letting 
;        the default maxima and minima of the array be the display scale, 
;        and let the default 'idl.ps' file be the output :
;
;        pstek2sd, image1			     
;
;     2. To produce a  display for image array image1, scaled to 0.8 
;        times the maximum possible letter-size output display, using 
;        0 and 20 as minima and maxima of the display scale, and 
;        output the post-script file to a file called 'out.ps':
;     
;        pstek2sd,image1,0,20, output = 'out.ps', scale= 0.8
;
;     3. To produce a display for image image1, letting the default 
;        maxima and minima of the array be the display scale, using
;        IDL color table 3 (Red Temperature), and
;        let the default 'idl.ps' file be the output :
;
;        pstek2sd, image1, color=3
;
;     4. To produce a display for image image1, letting the default
;        maxima and minima of the array be the display scale, using
;        user defined colors (hsv) in the HSV color system, and send 
;        output to the file image1.ps:
;
;        pstek2sd, image1, rcolor=h, gcolor=s, bcolor=v, /hsv
;
;#
;COMMON BLOCKS:
;     Colors
;
;PROCEDURE (AND OTHER PROGRAMMING NOTES)
;     Uses IDL intrinsic routine 'tvscl' to display the input array in
;     a post-script format file. The post-script file is created in 
;     the default directory.
;
;PERTINENT ALGORITHMS, LIBRARY CALLS, ETC.:
;     none
;     
;MODIFICATION HISTORY:
;     Written by Mila Mitra, General Sciences Corp. 17  March 1992
;     SPR 11250  26-Aug-1993  Enable color output. J. Newmark
;     Modified for TEK2SD 7/14/94 RGA
;
;.TITLE
;Routine PSTEK2SD
;-
pro pstek2sd,image,min,max,output=output,scale=scale,color=color,$
    rcolor=rcolor,gcolor=gcolor,bcolor=bcolor,hsv=hsv,hls=hls

COMMON colors,r_orig,g_orig,b_orig,r_curr,g_curr,b_curr
s=size(image)
old_dev=!d.name
set_plot,'ps'
;SET UP OUTPUT FILE, OPTIONAL COLORS AND OPTIONAL MIN, MAX AND SCALING 
; PARAMETERS:
;
if keyword_set(output) eq 0 then output='idl.ps'
IF (keyword_set(rcolor) and keyword_set(gcolor) and keyword_set(bcolor)) $
 THEN BEGIN
  device,/color,filename=output,bits_per_pixel=8 
  tvlct,rcolor,gcolor,bcolor
  IF keyword_set(hsv) then tvlct,rcolor,gcolor,bcolor,/hsv
  IF keyword_set(hls) then tvlct,rcolor,gcolor,bcolor,/hls
ENDIF ELSE IF (keyword_set(color)) THEN BEGIN
  device,/color,filename=output,bits_per_pixel=8 
  IF (color NE -1) THEN loadct,color
ENDIF ELSE device,filename=output,color=0,bits_per_pixel=8
if n_elements(scale) eq 0 then scale=1
if n_elements(min) eq 0 then min=min(image)
if n_elements(max) eq 0 then max=max(image)
;
;IF YSIZE OF IMAGE IS GREATER THAN X SIZE, ROTATES IMAGE
;
r=float(s(1))/float(s(2))
if (r gt 1.0) then begin
imrot=rotate(image,1)
end
;
;CASE 1: iF X LESS THAN Y, AND X/Y < 7/8.5 :
;
case 1 of
r le 7/8.5 : begin
n=8.5
m=8.5*r
device,/inches,xoff=0.75,yoff=1.25,scale=scale,xsize=m,ysize=n
tvscl,image <max >min
end
;
;CASE 2: IF X LESS THAN Y, AND 7/8.5 < (X/Y) < 1.0 :
;
(r gt 7./8.5) and (r le 1.0):  begin
m=7.0
n=m/r
device,/inches,xoff=0.75,yoff=1.25,scale=scale,xsize=m,ysize=n
tvscl,image <max >min
end
;
;CASE 3: iF Y LESS THAN X, AND 1.0 < (X/Y) < 1.21, ROTATES IMAGE :
;
(r gt 1.0) and (r le 8.5/7.): begin
m=7.0
n=m*r
device,/inches,xoff=0.75,yoff=1.25,scale=scale,xsize=m,ysize=n
tvscl,imrot <max >min
end
;
;CASE 4: iF Y LESS THAN X, AND (X/Y) > 1.21, ROTATES IMAGE :
;
(r gt 8.5/7.) : begin
n=8.5
m=n/r
device,/inches,xoff=0.75,yoff=1.25,scale=scale,xsize=m,ysize=n
tvscl,imrot <max >min
end
;
else:print,'ratio of x/y out of range'
endcase
;
device,/close
;
;SETS DEVICE BACK TO PREVIOUS DEVICE
;
set_plot,old_dev
return
end

