;+NAME/ONE LINE DESCRIPTIN OF ROUTINE:
;    FFT_SPLINE calculates the coefficients of the fast Fourier transform
;    of an unevenly sampled real function.
;
;-------------------------------------------------------------
; Procedure FFT_SPLINE
;
; IDL procedure to calculate the coefficients of the fast Fourier
; transform of a real function which was unevenly sampled in space
; or time.  The function is first interpolated to a user provided
; sampling increment using a cubic spline interpolation function. 
; 
; Written By:    BA Franz
; Date:          14 April 1992
;
; Inputs:
;     xin    - sampling points
;     yin    - function as sampled
;     delx   - sampling increment desired
;
; Outputs:
;     power(0,*) - real part of fft
;     power(1,*) - imaginary part of fft
;     freq       - frequencies associated with each element of power
;     xout       - evenly spaced sampling points
;     yout       - function interpolated to xout
;
;-------------------------------------------------------------

pro fft_spline,xin,yin,delx,power,freq,xout,yout

if (n_params(0) eq 0) then begin
    print,' '
    print,'Usage: fft_spline,xin,yin,delx,power,freq,xout,yout'
    print,' '
    return
endif

;
; Calculate the new sampling points and interpolate
;
nin  = n_elements(xin)
nout = long((xin(nin-1)-xin(0))/delx)
xout = xin(0) + findgen(nout)*delx
yout = spline(xin,yin,xout)

;
; For real functions, only want positive frequencies
;
n = long(nout/2)
freq   = findgen(n)/(2.0*n*delx)
power  = fltarr(2,n)


;
; Calculate FFT and separate complex result into parts
;
pow  = fft(yout,1)
temp = float(pow)        ; Real part
power(0,*) = temp(0:n-1)
temp = imaginary(pow)    ; Imaginary Part
power(1,*) = temp(0:n-1)

return
end
