;+NAME/ONE LINE DESCRIPTION OF FUNCTION:
;    POLYSMOOTH performs polynomial boxcar smoothing.
;
;----------------------------------------------------------------------
; Function POLYSMOOTH
;
; IDL function to perform polynomial boxcar smoothing
;
; Inputs:
;    x          - independent variable
;    y          - dependent variable (to be smoothed)
;    width      - total width of window used for each fit
;    ndim       - order of polynomial used for each window fit
;    [wts]      - weights to apply to each element of y
;    [bad_data] - data less than this value will be ignored
;    [plot]     - will generate a plot for each window fit if set to 1
;
;----------------------------------------------------------------------
function polysmooth,x,y,width,ndim,wts=wts,bad_data=bad_data,plot=plot

width = fix(width)
ndim  = fix(ndim)

if (n_elements(wts) eq 0) then wts = y*0.+1.
if (n_elements(bad_data) eq 0) then bad_data=0.0

if (not width) then begin
    message,'Odd width desired. Adding 1 to width.',/cont
    width = width+1
endif

if (width lt ndim+1) then begin
    message,'Minimum width is ndim+1. Resetting width.',/cont
    width = ndim+1
endif

ysmooth = y           ; Return vector
hwidth  = width/2L    ; Half width
n = n_elements(y)     ; Number of values

for i=0L,n-1 do begin
    ;
    ; Calculate indices to use in fitting near point i. Also calculate
    ; position of point i within the set of indices to fit.  Finally
    ; calculate the order of the polynomial to use in fitting;  the 
    ; order is reduced near the end points.
    ;
    if (i lt hwidth) then begin
        ;
        ; Left end
        ;
        indx = [0,i+hwidth]
        ipt  = i
        idim = min([ipt+1,ndim])
    endif else if (n-i-1 lt hwidth) then begin
        ;
        ; Right end
        ;
        indx = [i-hwidth,n-1]
        ipt  = hwidth
        idim = min([n-i,ndim])
    endif else begin
        ;
        ; Central region
        ;
        indx = [i-hwidth,i+hwidth]
        ipt  = hwidth
        idim = ndim
    endelse

    ;
    ; Select data within fitting region
    ;
    xx = double(x(indx(0):indx(1)))
    yy = double(y(indx(0):indx(1)))
    ww = double(wts(indx(0):indx(1)))

    ;
    ; Translate to center of gravity to reduce precission losses
    ;
    x0 = med(xx)
    xx = xx - x0

    ;
    ; Select only good data values
    ;
    s = where(yy gt bad_data)

    if (s(0) ne -1) then begin
        
        ngood = n_elements(s)

        if (ngood lt idim+1) then begin
            ;
            ; Not enough points for requested order, reduce order
            ;
            idim = ngood
            message,'Too few points at point '+strtrim(string(i),2),/cont
            message,'Reducing order of fit to '+strtrim(string(idim),2),/cont
        endif

        ;
        ; Calculate weighted polynomial fit of order idim
        ;
;        c = polyfitw(xx(s),yy(s),ww(s),idim)
;        c = robust_poly_fit(xx(s),yy(s),idim)
        c = lmpolyfit(xx(s),yy(s),idim,wts=ww(s))

        ;
        ; Evaluate fit for all points in fitting region
        ;
        yfit = c(0) & for iterm=1,idim do yfit = yfit+c(iterm)*xx^iterm

        ;
        ; Replace point i of output vector with value of fit at point i
        ;
        ysmooth(i) = yfit(ipt)

        ;
        ; Make plot if requested
        ;
        if (keyword_set(plot)) then begin
            plot,xx(s)+x0,yy(s),xr=[min(xx(s))-.1,max(xx(s))+.1],/yst,psym=3
;            eplot,xx(s)+x0,yy(s),sqrt(1./ww(s))
            oplot,xx+x0,yfit
            opplot,xx(ipt)+x0,yfit(ipt),psym=2
        endif

    endif else begin
        ;
        ; All points in fitting region were bad
        ;
        message,'No good data in window at point '+strtrim(string(i),2),/cont
        ysmooth(i) = bad_data
    endelse

endfor

return,ysmooth
end
