;----------------------------------------------------------------------
; Function MODELFIT
;
; Levenburg-Marquart optimization.
;
; Modified version of IDL user's library routine CURVEFIT.
;
; Modified by: BA Franz
;    - parameter fixing.
;    - rescaling of curvature and gradient matrices to reduce chances 
;      of over/underflow.
;    - accumulation of iteration statistics
;    - calculation of parameter uncertainties
;    - nfree specifiable
;    - parameter file output
;
; Calling Sequence:
;     yfit = modelfit(x,y,w,a, $
;                     function_name=function_name,pfix=pfix,nfree=nfree,
;                     sigma=sigmaa,stats=stats,covar=covar,chisqr=chisqr
;
; Inputs:
;     x  - n-element vector containing the independent variable.  
;          Note: x can be a structured array for multi-dimensional input.
;     y  - n-element vector containing the dependent variable.
;     w  - n-element vextor containing the weights (e.g., w=1/sigma_y^2)
;     a  - m-element vector of initial parameters.
;
; Optional Input:
;     function_name=function_name - name of the fitting function (def='funct')
;     pfix=pfix   - k-element vector containing indices of parameters which 
;                   are to to be held at there input values (def=none).
;     nfree=nfree - number of degrees of freedom (def=n - m - k)
;     parfile=parfile - if specified, the updated parameters will be written
;                   to this file as an IDL saveset at each iteration.
;     pchi=pchi - percent change in chisquare used to end iteration.
;     ppar=ppar - percent change in parameter values used to end iteration.
;     nmax=nmax - max number of iterations
;
; Optional Output:
;     sigma  - estimated parameter uncertainties (-1 = fixed)
;     stats  - structure containing statistics at each iteration step
;     covar  - covariance matrix
;     chisqr - final chisqr
;
; Fitting Function:
;
;     The fitting function must have the calling sequence:
;
;     function_name(x,a,f,dfda,indxpar=indxpar)
;
;     where:  x and a are as described above
;             f is an n-element vector containing the value of the 
;                 function evaluated at each x
;             indxpar is a v-element vector containing the indices of "a" 
;                 for which the parameters are varying
;             dfda is a (v x n) array of partial derivative of the function 
;                 with respect to the varying parameters at each x
;
;----------------------------------------------------------------------
function modelfit,x,y,w,a,                                    $ ; Inputs
         function_name=function_name,pfix=pfix,nfree=nfree,   $ ; Inputs
         parfile=parfile,                                     $ ; Inputs
         pchi=percent,ppar=percpar,nmax=iter_max,             $ ; Inputs
	 sigma=sigmaa,stats=stats,covar=covar,chisqr=chisqr     ; Outputs

if (n_elements(function_name) eq 0) then function_name = "FUNCT"

if (n_elements(parfile) eq 0) then $
    savpar=0                       $
else if (parfile eq '') then       $
    savpar=0                       $
else                               $ 
    savpar=1

lun=-1
 
if (n_elements(percpar) eq 0) then $
  percpar  = 0.1       ; Percent change in params used to end iteration
if (n_elements(percent) eq 0) then $
  percent  = 0.01      ; Percent change in chisquare used to end iteration
if (n_elements(iter_max) eq 0) then $
  iter_max = 25        ; Maximum iterations allowed

deltachi = 1.e-7       ; Absolute change in chisquare used to end iteration
flambda  = 0.0001      ; Initial value for lambda

nfix     = n_elements(pfix)                  ; Number of fixed parameters
nterms   = n_elements(a) - nfix              ; Number of variable parameters
diag     = indgen(nterms)*(nterms+1)         ; Subscripts of diagonal elements

if (n_elements(nfree) eq 0) then $
    nfree = n_elements(x)-nterms             ; Number degrees of freedom

if (nfree le 0) then begin
    message,'Not enough data points.',/cont
    return,-1
endif

;
; Set-up for accumulation of run statistics
;
statrec = {params:fltarr(nfix+nterms),chisq:0.,iter:0L,lambda:0.}
stats   = statrec

;
; Set-up for parameter fixing
;
indxpar = bindgen(nterms+nfix)
if (nfix ne 0) then $
    for i=0,nfix-1 do indxpar = indxpar(where(indxpar ne pfix(i)))
anew = a            ; Full parameter list
a    = a(indxpar)   ; Variable parameter list
printf,lun,''
printf,lun,a
printf,lun,''

;
; Primary iteration loop
;
iter = 0
done = 0
while ( (iter lt iter_max) and (not done) ) do begin

    iter = iter+1

    ;
    ; Get fit with partials for this parameter set
    ;    
    anew(indxpar) = a
    call_procedure,function_name,x,anew,yfit,pder,indxpar=indxpar
    chisqr = total(w*(y-yfit)^2)/nfree

    if (iter eq 1) then begin
        printf,lun,0,flambda,chisqr
        ;
        ; Accumulate stats
        ;
        statrec.params = anew
        statrec.chisq  = chisqr
        statrec.iter   = 0
        statrec.lambda = flambda
        stats = statrec
    endif 

    ;
    ; Evaluate alpha (curvature) and beta (gradient) matricies.
    ;
    alpha = transpose(pder) # (w # (dblarr(nterms)+1)*pder)
    beta  = (y-yfit)*w # pder

    ;
    ; Rescale to minimize overflow, underflow
    ;
    amax  = max(abs(alpha(diag)))
    amin  = min(abs(alpha(diag)))
    scale = sqrt(amax*amin)

    if (scale eq 0.0) then begin
        message,'Hessian contains zeroes on the diagonal',/info
        for i=0,nterms-1 do print,i,alpha(diag(i))
        stop
    endif

    alpha = temporary(alpha)/scale
    beta  = temporary(beta)/scale

    ;
    ; Form normalized curvature matrix
    ;
    c = sqrt(alpha(diag) # alpha(diag))
    alpha_c = alpha/c

    ;
    ; Secondary iteration loop 
    ;
    repeat begin

        ;
        ; Invert modified curvature matrix to find new parameters.
        ;
        array = alpha_c 
        array(diag) = 1.0 + flambda
        array = invert(array,status)
        case (status) of
         0: 
         1: message,'Inversion of hessian matrix failed',/info
         2: message,'Inversion of hessian matrix invalid',/info
        endcase
        b = a + array/c # transpose(beta)
 
        ;
        ; Get new model fit
        ;
        anew(indxpar) = b
        printf,lun,b
        call_procedure,function_name,x,anew,yfit
        newchisqr = total(w*(y-yfit)^2)/nfree
        printf,lun,iter,flambda,newchisqr

        ;
        ; Accumulate stats
        ;
        statrec.params = anew
        statrec.chisq  = newchisqr
        statrec.iter   = iter
        statrec.lambda = flambda
        stats = [temporary(stats),statrec]
        if (savpar) then save,file=parfile,stats,anew

        ;
        ; Update lambda (assumes chi square increased)
        ;
        flambda = flambda*10.
   
    endrep until (newchisqr le chisqr)

    ;
    ; New minimum found, save new params and decrease lambda
    ; Note: decrease must account for false assumption above
    ; that chi square increased.
    ; 
    da      = abs(b-a)
    dchisqr = chisqr - newchisqr
    a       = b
    flambda = flambda/100.  

    ;
    ; Test for convergence
    ;
    if (max(da/a)*100. le percpar) then begin

        printf,lun,'Convergence Reached: Relative Change in Parameters'

        if (dchisqr/newchisqr*100. le percent) then begin
            printf,lun,'Convergence Reached: Relative Change in Chisquare'
            done = 1
        endif
        if (newchisqr eq 0.0) then begin
            printf,lun,'Convergence Reached: Chisquare=0.0'
            done = 1
        endif
        if (dchisqr le deltachi) then begin
            printf,lun,'Convergence Reached: Absolute Change in Chisquare'
            done = 1
        endif

        if (done) then begin
            chisqr = newchisqr
            printf,lun,'Good fit at iteration',iter
            printf,lun,'Chisquare = ',chisqr
        endif

    endif

endwhile

if (not done) then message, 'Failed to converge',/informational

;
; Save new params 
;
anew(indxpar) = a  &  a = anew

;
; Calculate uncertainty in parameters.
;
call_procedure,function_name,x,a,yfit,pder,indxpar=indxpar
chisqr  = total(w*(y-yfit)^2)/nfree
hess    = 2.0 * transpose(pder) # (w # (dblarr(nterms)+1)*pder)
covar   = invert(hess,status)
case (status) of
  0: 
  1: message,'Inversion of hessian matrix failed',/info
  2: message,'Inversion of hessian matrix invalid',/info
endcase
sigmaa  = a*0-1 & sigmaa(indxpar) = confint(hess,/hess,nfree,chisqr,conf=68.0)

return,yfit
end
