;+NAME/ONE LINE DECSRIPTION OF ROUTINE:
;    POLFIT2D determines a two-dimensional polynomial fit.
;
; NAME:
;	POLFIT2D
;
; PURPOSE:
;	This function determines a 2-dimensional polynomial fit.
;
; CATEGORY:
;	Curve and surface fitting.
;
; CALLING SEQUENCE:
;	Result = POLFIT2D(z [,x,y,Degree=Degree,Bad_Data=Bad_Data,Coef=Coef)
;
; INPUTS:
; 	z:	The data to fit. Can be vector or 2D array.
;   x,y:  Location of z values.  If z is a 2D array of dimension n x m, then x 
;       and y are optional but must be vectors of length n and m respectively.
;       if z is a vector, x and y are required and must be vectors of equal length
;       to z.
;
;	Degree = Degree: The maximum degree of fit (in one dimension).
;	Bad_Data = Bad_Data: values of z equal to Bad_Data will be ignored.
;
; OUTPUT:
;	This function returns a fitted array of equal dimension to z.
;
; OUTPUT KEYWORDS:
;	coef:	The array of coefficients for a polynomial function
;		of x and y to fit data.
;		This parameter is returned as a (Degree+1) by (Degree+1) 
;		element array.
;
; PROCEDURE:
; 	Fit a 2D polynomial. The function fitted is:
;  	    F(x,y) = Sum over i and j of Coef(j,i) * x^i * y^j
;
;   Note: SFIT is a faster but less flexible alternative to this routine.
;
; MODIFICATION HISTORY:
;	May 1995, B. A. Franz, based on User Library routine SFIT
;
;-
function polfun2d, coef, x, y, ut=ut

    n2     = n_elements(coef)
    degree = fix(sqrt(n2)-1)

    if (n_elements(ut) eq 0) then begin

        nx = n_elements(x)
        ny = n_elements(y)
        m  = nx

        ut = dblarr(n2, m, /nozero)
        for i=0, degree do for j=0,degree do $
            ut(i*(degree+1) + j, *) = x^i * y^j
    endif

    return,reform(reform(coef,n2) # ut)

end

; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------

function polfit2d, zz, xx, yy, degree=degree, coef=kx, bad_data=bad_data

   on_error, 2

   if (n_elements(degree) eq 0) then degree=2
   if (n_elements(bad_data) eq 0) then bad_data=-16000

   n2=(degree+1)^2

   ;
   ; Convert input to x,y,z vectors of equal length
   ;
   s = size(zz)
   case s(0) of

     2 : begin
         m  = s(1) * s(2)
         nx = n_elements(xx)
         ny = n_elements(yy)
         if (nx eq 0 or ny eq 0) then begin
             nx = s(1) 
             ny = s(2) 
             xx = findgen(nx)
             yy = findgen(ny)
             x  = reform(xx # replicate(1.,ny), m)
             y  = reform(replicate(1.,nx) # yy, m)
         endif else if (nx ne s(1) or ny ne s(2)) then begin
             message,'x must be vector of length as z(*,0)',/info
             message,'y must be vector of length as z(*,1)',/info
             return,zz
         endif else begin
             x = reform(xx # replicate(1.,ny), m)
             y = reform(replicate(1.,nx) # yy, m)
         endelse
         z = reform(zz, m)
         end

     1 : begin
         m  = s(1)
         nx = n_elements(xx)
         ny = n_elements(yy)
         if (nx ne m or ny ne m or nx ne ny) then begin
             message,'x and y must be vectors of same length as z',/info
             return,zz
         endif
         x  = xx
         y  = yy
         z  = zz
         end

      else : begin
             message,'z must be a 1D or 2D array',/info
             return,zz
         end
 
    endcase

    g = where(z ne bad_data, m)
    if (m eq 0) then begin
        message,'No good data to fit',/info
        return,zz
    endif

    ut = dblarr(n2, m, /nozero)
    for i=0L, degree do for j=0L,degree do $
    ut(i*(degree+1) + j, *) = x(g)^i * y(g)^j

    kk = invert(ut # transpose(ut)) # ut
    kx = dblarr(degree+1, degree+1) + float(kk # reform(z(g), m, 1))

    fit = polfun2d(kx,x,y)

    if (s(0) eq 2) then fit = reform(fit, nx, ny)

    return, fit

end


