;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    ROBUST_POLY_FIT performs an outlier-resistant polynomial fit.
;
;NAME:
; ROBUST_POLY_FIT
;
;PURPOSE:
; An outlier-resistant polynomial fit.
;
;CALLING SEQUENCE:
; COEFF = ROBUST_POLY_FIT(X,Y,NDEGREE  ,YFIT,CHISQUARE)
;
;INPUTS:
; X = Independent variable vector, floating-point or double-precision
; Y = Dependent variable vector
;
;OUTPUTS:
; Function result = coefficient vector, length NDEGREE+1.
; Either floating point or double precision.
;
;OPTIONAL OUTPUT PARAMETERS:
; YFIT = Vector of calculated y's
; CHI  = the reduced chi-square
;
;RESTRICTIONS:
; Large values of NDEGREE should be avoided. This routine works best
; when the number of points >> NDEGREE.
;
;PROCEDURE:
; For the initial estimate, the data is sorted by X and broken into NDEGREE+2 
; sets. The X,Y medians of each set are fitted to a polynomial via POLY_FIT.
; Bisquare ("Tukey's Biweight") weights are then calculated, using the 
; interquartile range as the measure of dispersion and a limit of 6
; standard deviations.
; The fit is repeated iteratively until the chi-square, also calculated using 
; biweights, begins to grow or changes by less than CLOSE_ENOUGH, now set
; .0002.
;
;AUTHOR: H. Freudenreich, STX, 8/90. Revised 4/91.
;
;-
FUNCTION          ROBUST_POLY_FIT,X,Y,NDEG,YFIT,CHI

;  ON_ERROR,2

  CLOSE_ENOUGH = .0002

  EPSILON = 1.0E-20

  NPTS = N_ELEMENTS(X)
  IF (NPTS/4*4) EQ NPTS THEN NEED2 = 1 ELSE NEED2 = 0
  N3 = 3*NPTS/4  &  N1 = NPTS/4

; If convenient, move X and Y to their centers of gravity:
  IF NDEG LT 4 THEN BEGIN
     X0=TOTAL(X)/NPTS
     Y0=TOTAL(Y)/NPTS
     U=X-X0     
     V=Y-Y0
  ENDIF ELSE BEGIN
     U = X
     V = Y
  ENDELSE

; The initial estimate.

; Choose an odd number of segments:
  NUM_SEG = NDEG+2
  IF (NUM_SEG/2*2) EQ NUM_SEG THEN NUM_SEG =NUM_SEG+1
  MIN_PTS = NUM_SEG*3
  IF NPTS LT MIN_PTS THEN BEGIN
;    Settle for least-squares:
     CC = POLY_FIT( U, V, NDEG, YFIT )
  ENDIF ELSE BEGIN
;    Break up the data into segments:
     Q = SORT(U)
     U = U(Q)  &  V = V(Q)
     N_PER_SEG = INTARR(NUM_SEG)
     N_PER_SEG(*) = NPTS/NUM_SEG

;    Put the leftover points in the middle segment:
     N_LEFT = NPTS - N_PER_SEG(0)*NUM_SEG
     N_PER_SEG(NUM_SEG/2) = N_PER_SEG(NUM_SEG/2) + N_LEFT
     R = FLTARR(NUM_SEG)  &  S = FLTARR(NUM_SEG)
     R(0)=MED( U(0:N_PER_SEG(0)-1) ) & S(0)=MED( V(0:N_PER_SEG(0)-1) )
     I2 = N_PER_SEG(0)-1L
     FOR I=1L,NUM_SEG-1 DO BEGIN
       I1 = I2 + 1L
       I2 = I1 + N_PER_SEG(I) - 1
       R(I) = MED( U(I1:I2) )     &  S(I) = MED( V(I1:I2) )
     ENDFOR
;    Now fit:
     CC = POLY_FIT( R,S, NDEG )
     YFIT = CC(0) & FOR I=1,NDEG DO YFIT=YFIT+CC(I)*U^I
  ENDELSE

; Calculate the interquartile range and scale it to the std. deviation:
  DEV   = ABS(V-YFIT) 
  DEVS = DEV(SORT(DEV))
  IF NEED2 THEN SIG=(DEVS(N3-1)+DEVS(N3)-DEVS(N1-1)-DEVS(N1))*.5 $
           ELSE SIG  = DEVS(N3)-DEVS(N1)
  SIG = SIG/1.35 ; (a good estimate for N > 40, otherwise an OK estimate)

; Calculate the (bi)weights:
  Z = ABS(DEV)/(6.*SIG)
  S = WHERE( Z GT 1.0 )  &  IF (S(0) GE 0) THEN Z(S) = 1.
  W = (1.-Z*Z)^2
  W = NPTS*W/TOTAL(W)

; Loop until we converge, or give up:
  NFREE = FLOAT(NPTS-NDEG-1)
  NUM_IT = 0
  CHI = 1.0E30
  DIFF = CHI
  WHILE( (DIFF GT CLOSE_ENOUGH) AND (NUM_IT LT 50) ) DO BEGIN
     NUM_IT = NUM_IT + 1
   ; Re-fit:
     OLDCC = CC
     CC= POLYFITW( U, V, W, NDEG, YFIT )
     DEV   = ABS(V-YFIT)
     DEVS = DEV(SORT(DEV))
     IF NEED2 THEN SIG=(DEVS(N3-1)+DEVS(N3)-DEVS(N1-1)-DEVS(N1))*.5 $
              ELSE SIG  = DEVS(N3)-DEVS(N1)
     SIG = SIG/1.35
     IF SIG LT EPSILON THEN BEGIN
        CHI = 0.
        PRINT,' ROBUST_POLY_FIT: Zero Residuals.'
        GOTO,AFTERFIT
     ENDIF
     Z=ABS(DEV)/(6.*SIG)
     Q=WHERE(Z GT 1.) & IF Q(0) GE 0 THEN Z(Q)=1.0
     W = (1.-Z*Z)^2
     W = NPTS*W/TOTAL(W)
;      The chi-square of the residuals:
     OLDCHI = CHI
     CHI = TOTAL(W*DEV*DEV)/FLOAT(NPTS-NDEG-2)
     DIFF = OLDCHI - CHI
     IF DIFF LT 0. THEN CC=OLDCC
  ENDWHILE
  IF NUM_IT GT 50 THEN PRINT,$
    '   ROBUST_POLY_FIT did not converge in 50 iterations!'

  AFTERFIT:
; Shift the curve back from its center of gravity:
  IF NDEG EQ 1 THEN BEGIN
     CC(0) = CC(0)-CC(1)*X0 + Y0
  ENDIF ELSE IF NDEG EQ 2 THEN BEGIN
     CC(0) = CC(0)-CC(1)*X0+CC(2)*X0^2 + Y0
     CC(1) = CC(1)-2.*CC(2)*X0
  ENDIF ELSE IF NDEG EQ 3 THEN BEGIN
     CC(0) = CC(0)-CC(1)*X0+CC(2)*X0^2-CC(3)*X0^3 + Y0
     CC(1) = CC(1)-2.*CC(2)*X0+3.*CC(3)*X0^2
     CC(2) = CC(2)-3.*CC(3)*X0
  ENDIF

; Calculate the fit at points X:
  IF( N_PARAMS(0) GT 3 )THEN BEGIN
     YFIT = CC(0)
     FOR I=1,NDEG DO YFIT=YFIT+CC(I)*X^I
  ENDIF

  RETURN,CC(*)
END
