;+
; NAME:
;   NCDF_PARSE
;
; PURPOSE:
;   Parses a NetCDF4 file and returns a nested structure containing all
;   of the groups, datasets, and attributes.
;
; CALLING SEQUENCE:
;   Result = NCDF_PARSE(File [, /NODATA])
;
; RETURN VALUE:
;   Result: A nested structure.
;
; INPUTS:
;   File: A scalar string giving the file to parse.
;
; KEYWORD PARAMETERS:
;   NODATA = Set this keyword to disable loading of data values while
;            parsing the file. The default is to read all datasets.
;
; EXAMPLE:
;     file = FILEPATH('test.nc', SUBDIR=['examples','data'])
;     struc = NCDF_PARSE(file)
;     array = struct.z._data
;
; MODIFICATION HISTORY:
;   Written by:  Gwyn Fireman, June 2013
;   Modified by:
;
;-

;-------------------------------------------------------------------------
function ncdf_readatts, fileid, varid, global=global

; global attributes
if keyword_set(global) then begin
  natts = (ncdf_inquire(fileid)).ngatts
  for i=0, natts-1 do begin
    attname = ncdf_attname(fileid, i, /global)
    ncdf_attget, fileid, attname, attvalue, /global
    if (ncdf_attinq(fileid,attname,/global)).datatype eq 'CHAR' $
    then attvalue=string(attvalue)
    struct = (n_elements(struct) eq 0) $
             ? create_struct(        attname, attvalue) $
             : create_struct(struct, attname, attvalue)
  endfor                        ; global atts

; variable attributes
endif else begin
  natts = (ncdf_varinq(fileid, varid)).natts
  for i=0, natts-1 do begin
    attname = ncdf_attname(fileid, varid, i)
    ncdf_attget, fileid, varid, attname, attvalue
    if (ncdf_attinq(fileid,varid,attname)).datatype eq 'CHAR' $
    then attvalue=string(attvalue)
    struct = (n_elements(struct) eq 0) $
             ? create_struct(        attname, attvalue) $
             : create_struct(struct, attname, attvalue)
  endfor                        ; var atts
endelse

return, struct
end

;-------------------------------------------------------------------------
function ncdf_parse, filename, nodata=nodata
; load NetCDF contents into a structure

fileid   = ncdf_open(filename, /NOWRITE)
fileinfo = ncdf_inquire(fileid)

; variables
for i=0, fileinfo.nvars-1 do begin
  varname = (ncdf_varinq(fileid, i)).name

; read variable attributes
  atts = ncdf_readatts(fileid, i)
  varstruct = atts

; read variable values as needed
  if not keyword_set(nodata) then begin
    varid = ncdf_varid(fileid, varname)
    ncdf_varget, fileid, varid, value
    varstruct = create_struct(varstruct, '_DATA', value)
  endif

; add to structure definition
  struct = (n_elements(struct) eq 0) $
           ? create_struct(        varname, varstruct) $
           : create_struct(struct, varname, varstruct)
  
endfor                          ; variables

; global attributes
atts = ncdf_readatts(fileid, /global)
struct = (n_elements(struct) eq 0) ? $
         atts : create_struct(struct, atts)

ncdf_close, fileid
return, struct
end
