;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    READCC reads an sds from a cost color netCDF file, applies scaling
;
;  NAME:
;    readcc
;
;  PURPOSE:                                   
;    Simplifies the reading of Coast Colour files and applies
;    slope/intercept scaling. Can read multiple band-specific 
;    products into a single array.
;
;  CALLING SEQUENCE:
;    data = readcc(filename,prodname)
;
;  INPUT:
;    filename - L2 filename string.
;    prodname - L2 SDS product name string. If this ends in '_', all SDS
;               products of name prodname_nnn (where nnn is wavelength)
;               will be returned.
;
;  OUTPUT:
;    data - 1, 2, or 3-dimensional array containing the requested product.
;
;  SUBROUTINES CALLED:
;
;  WRITTEN BY:
;    B.A. Franz, NASA/GSFC, July 2011
;
;-
function readcc,file,field,lon=lon,lat=lat,solz=solz,senz=senz,sola=sola,sena=sena,geom=geom

    fid = NCDF_OPEN(file, /NOWRITE)

    did = NCDF_DIMID(fid,'x')
    NCDF_DIMINQ,fid,did,name,nx
    did = NCDF_DIMID(fid,'y')
    NCDF_DIMINQ,fid,did,name,ny

    print,nx,ny

    gain    = 1
    offset  = 0
    data    = -1

    last_char = strmid(field,strlen(field)-1,1)

    if (last_char eq '_') then begin

      wl = strtrim(string(indgen(15)+1),2)
      nb = n_elements(wl)

      first =  1

      for i=0,nb-1 do begin

        sdsname = field+wl(i)
        vid = NCDF_VARID(fid,sdsname)
        NCDF_VARGET, fid, vid, val

        if (n_elements(val) ne 1 or val(0) ne -1) then begin
            NCDF_ATTGET, fid, vid, 'scale_factor', gain
            if (first eq 1) then begin
                first = 0
                if (gain ne 0.0 and gain ne 1.0) then begin
                    data  = val*gain 
                endif else begin 
                    data  = val
                endelse
            endif else begin
                if (gain ne 0.0 and gain ne 1.0) then begin
                    data  = [[[data]],[[val*gain]]] 
                endif else begin 
                    data  = [[[data]],[[val]]]
                endelse
            endelse
        endif

      endfor

    endif else begin

      sdsname = field
      vid = NCDF_VARID(fid,sdsname)
      NCDF_VARGET, fid, vid, val

      if (val(0) ne -1 or n_elements(val) gt 1) then begin
          NCDF_ATTGET, fid, vid, 'scale_factor', gain
          if (gain ne 0.0 and gain ne 1.0) then begin
              data  = val*gain + offset 
          endif else begin
              data  = val
          endelse
      endif

    endelse

    if (keyword_set(geom)) then begin

      sdsname = 'longitude'
      vid = NCDF_VARID(fid,sdsname)
      NCDF_VARGET, fid, vid, lon1

      NCDF_ATTGET,fid,vid,'offset_x', off_x
      NCDF_ATTGET,fid,vid,'offset_y', off_y
      NCDF_ATTGET,fid,vid,'subsampling_x', sub_x
      NCDF_ATTGET,fid,vid,'subsampling_y', sub_y

      sdsname = 'latitude'
      vid = NCDF_VARID(fid,sdsname)
      NCDF_VARGET, fid, vid, lat1

      sdsname = 'sun_zenith'
      vid = NCDF_VARID(fid,sdsname)
      NCDF_VARGET, fid, vid, solz1

      sdsname = 'sun_azimuth'
      vid = NCDF_VARID(fid,sdsname)
      NCDF_VARGET, fid, vid, sola1

      sdsname = 'view_zenith'
      vid = NCDF_VARID(fid,sdsname)
      NCDF_VARGET, fid, vid, senz1

      sdsname = 'view_azimuth'
      vid = NCDF_VARID(fid,sdsname)
      NCDF_VARGET, fid, vid, sena1

      nxctl = n_elements(lon1[*,0])
      nyctl = n_elements(lon1[0,*])

      xctl = findgen(nxctl)*sub_x + off_x
      yctl = findgen(nyctl)*sub_y + off_y

      x = indgen(nx)
      y = indgen(ny)
      ;
      ; We now have all the information at each control point, so we can
      ; interpolate to all pixels, all lines.
      ;
      lon   = fltarr(nx,ny)
      lat   = fltarr(nx,ny)
      solz  = fltarr(nx,ny)
      sola  = fltarr(nx,ny)
      senz  = fltarr(nx,ny)
      sena  = fltarr(nx,ny)

      tlon   = fltarr(nx,nyctl)
      tlat   = fltarr(nx,nyctl)
      tsolz  = fltarr(nx,nyctl)
      tsola  = fltarr(nx,nyctl)
      tsenz  = fltarr(nx,nyctl)
      tsena  = fltarr(nx,nyctl)
  
      ;
      ; Interpolate angles across each control-point line
      ;
      print,'Interpolating rows'
      for j=0L,nyctl-1 do begin
          tlon (*,j) = spline(xctl,reform(lon1 (*,j)),x)
          tlat (*,j) = spline(xctl,reform(lat1 (*,j)),x)
          tsolz(*,j) = spline(xctl,reform(solz1(*,j)),x)
          tsola(*,j) = spline(xctl,reform(sola1(*,j)),x)
          tsenz(*,j) = spline(xctl,reform(senz1(*,j)),x)
          tsena(*,j) = spline(xctl,reform(sena1(*,j)),x)
      endfor
  
      ;
      ; Fill missing lines by interpolating columns
      ;
      print,'Interpolating columns'
      for i=0L,nx-1 do begin
          lat (i,*) = spline(yctl,tlat (i,*),y)
          lon (i,*) = spline(yctl,tlon (i,*),y)
          solz(i,*) = spline(yctl,tsolz(i,*),y)
          sola(i,*) = spline(yctl,tsola(i,*),y)
          senz(i,*) = spline(yctl,tsenz(i,*),y)
          sena(i,*) = spline(yctl,tsena(i,*),y)
      endfor
  
    endif
  
    NCDF_CLOSE, fid

    return, data
end
