;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    READL2HDF reads an sds from an MSl12 hdf file, applies scaling
;
;  NAME:
;    readl2hdf
;
;  PURPOSE:                                   
;    Simplifies the reading of SeaWiFS/SIMBIOS standard L2 hdf file. Auto
;    applies slope/intercept scaling. Can read multiple band-specific 
;    SDS products into a single array.
;
;  CALLING SEQUENCE:
;    data = readl2hdf(filename,prodname)
;
;  INPUT:
;    filename - L2 filename string.
;    prodname - L2 SDS product name string. If this ends in '_', all SDS
;               products of name prodname_nnn (where nnn is wavelength)
;               will be returned.
;
;  OUTPUT:
;    data - 1, 2, or 3-dimensional array containing the requested product.
;
;  SUBROUTINES CALLED:
;
;  WRITTEN BY:
;    B. A. Franz, SAIC General Sciences Corp., 1999.
;
;-
function readl2hdf,file,field

    fid = HDF_SD_START(file, /READ)

    gain    = 1
    offset  = 0
    data    = -1

    last_char = strmid(field,strlen(field)-1,1)

    if (last_char eq '_') then begin

      wl = strtrim(string(rd_sds(fid,'wavelength')),2)
      nb = n_elements(wl)

      first =  1

      for i=0,nb-1 do begin

        sdsname = field+wl(i)
        val     = rd_sds(fid, sdsname)

        if (n_elements(val) ne 1 or val(0) ne -1) then begin
            iret    = HDF_Get_ATTR(fid, 'slope',     SDS=sdsname, gain)
            iret    = HDF_Get_ATTR(fid, 'intercept', SDS=sdsname, offset)
            if (first eq 1) then begin
                first = 0
                if (gain ne 0.0 and gain ne 1.0) then $
                    data  = val*gain + offset $
                else $
                    data  = val
            endif else begin
                if (gain ne 0.0 and gain ne 1.0) then $
                    data  = [[[data]],[[val*gain + offset]]] $
                else $
                    data  = [[[data]],[[val]]]
            endelse
        endif

      endfor

    endif else begin

      sdsname = field
      val     = rd_sds(fid, sdsname)

      if (val(0) ne -1 or n_elements(val) gt 1) then begin
          iret    = HDF_Get_ATTR(fid, 'slope',     SDS=sdsname, gain)
          if (iret eq -1) then $
              iret    = HDF_Get_ATTR(fid, 'Slope',     SDS=sdsname, gain)
          iret    = HDF_Get_ATTR(fid, 'intercept', SDS=sdsname, offset)
          if (iret eq -1) then $
              iret    = HDF_Get_ATTR(fid, 'Intercept', SDS=sdsname, offset)
          if (gain ne 0.0 and gain ne 1.0) then $
              data  = val*gain + offset $
          else $
              data  = val
      endif

    endelse

    HDF_SD_END, fid

    return, data
end


