;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    READL3BIN reads a specific product from a seawifs bin file
;
;  NAME:
;    readl3bin
;
;  PURPOSE:                                   
;    Simplifies the reading of SeaWiFS standard L3 bin file. Returns mean
;    bin values for a specified product, and corresponding lon/lat.
;
;  CALLING SEQUENCE:
;    data = readl3hdf(filename,prodname,lon,lat)
;
;  INPUT:
;    filename - L2 filename string.
;    prodname - L2 SDS product name string. If this ends in '_', all SDS
;               products of name prodname_nnn (where nnn is wavelength)
;               will be returned.
;
;  OUTPUT:
;    data - 1-dimensional array containing the requested product.
;    lon  - 1-demensional array of bin longitude positions
;    lat  - 1-demensional array of bin latitude positions
;
;  SUBROUTINES CALLED:
;    get_vds_in_vg
;    bin2ll
;
;  WRITTEN BY:
;    J. Gales, Futuretech Corp..
;    B. A. Franz, SAIC General Sciences Corp..
;    February 2000.
;
; HISTORY:
;
;       Tue Oct 29 15:47:19 2002, Joel Gales <joel@sherpa.domain.sdps>
;       Tue Apr 03 10:37:00 2009, Joel Gales <joel@sherpa.domain.sdps>
;         - Add support for quality field
;
;		
;
;-
function readl3bin_nc,file,prod,lon,lat,bins,nobs,wts,nscenes,sum,sum2, $
                      qual=qual,beg=beg,extent=extent,nrows=nrows, $
                      quiet=quiet,time_rec=time_rec

  binindex = read_hdf5(file, 'BinIndex', group='level-3_binned_data')
  beg = binindex._begin
  extent = binindex.extent
  start = binindex.start_num
  mx = binindex.max
  nrows = n_elements(binindex)

  binlist = read_hdf5(file, 'BinList', group='level-3_binned_data')

  bins = binlist.bin_num
  nobs = binlist.nobs
  wts = binlist.weights
  nscenes = binlist.nscenes
  time_rec = binlist.time_rec

  n = LONG(total(double(nobs ne 0)))
  if (not keyword_set(quiet)) then print,'# of non-zero bins:', n

  bins = bins[0:n-1]
  nobs = nobs[0:n-1]
  wts = wts[0:n-1]
  nscenes = nscenes[0:n-1]
  time_rec = time_rec[0:n-1]

  if (prod eq '') then begin
     data = -1
     goto, skip
  endif

  prod = read_hdf5(file, prod, group='level-3_binned_data')

  sum = prod.sum
  sum = sum[0:n-1]
  sum2 = prod.sum_squared
  sum2 = sum2[0:n-1]

  data = sum/wts

skip:

  print,'nrows: ', nrows
  bin2ll,nrows,bins,lat,lon

  return, data
end
