;-------------------------------------------------------------
;+
; NAME:
;       nrl_hico_h5_to_flat.pro
; PURPOSE:
;       
; CALLING SEQUENCE:
;       
; INPUTS:
;      infile: required scalar string with the name of an HDF5 file. 
;              The path needs to work with IDL. In general, use a fully 
;              qualified path. 
;
;      outdir: required scalar string with the name of the output directory
;
; OUTPUTS:
;
; MODIFICATIONS:
; By Marcos Montes, marcos.montes@nrl.navy.mil
; Original version 2013/10/21
; 
; 2013/11/05 Made the two arguments optional. If not provided, code will
; prompt user with a dialog box. 
;
; This source code was written at the the U.S. Naval Research Laboratory
; and is of US Government origin. If you did not receive this file from the
; Naval Research Laboratory, the caliber/quality of the code cannot be
; vouched for. 
;
;“THIS SOFTWARE IS PROVIDED “AS IS” AND WITHOUT ANY REPRESENTATION OR
;WARRANTY, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
;WARRANTY OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR ANY
;WARRANTIES OF ACCURACY OR COMPLETENESS OR THAT THE USE OF THE SOFTWARE
;WILL NOT INFRINGE OR VIOLATE ANY PATENT OR OTHER PROPRIETARY RIGHTS OF ANY
;THIRD PARTY (WHETHER DIRECTLY OR INDIRECTLY).  NRL shall not liable for
;any damage or loss except to the extent that such damage or loss were the
;direct result of the acts of NRL employees AND to the extent such
;liability may be established and pursued against NRL under the Federal
;Tort Claims Act and other applicable law and regulation that specifically
;waive the sovereign immunity of the United States.” 
;
; 
;
pro nrl_hico_h5_to_flat,infile,outdir

   compile_opt idl2

   program_name = 'nrl_hico_h5_to_flat'
   program_version = '1.1'
   program_date = '2013/11/05'
   program_author = 'Marcos Montes, marcos.montes@nrl.navy.mil'

   if (n_params() ne 2) then begin
      if (n_elements(infile) ne 1) then begin
         infile=!null
         infile=dialog_pickfile(/must_exist,filter='*.h5',title='Select a HICO HDF5 file')
         if (infile eq '') then begin
            print,'==============================================='
            print,'nrl_hico_h5_to_flat: input HDF5 file not selected; returning to caller'
            print,'==============================================='
            return
         endif
      endif

      if (n_elements(outdir) ne 1) then begin
         outdir=!null
         path=file_dirname(infile,/mark_directory)
         outdir=dialog_pickfile(path=path,/directory,/must_exist,title='Select an output directory')
         if (outdir eq '') then begin
            print,'==============================================='
            print,'nrl_hico_h5_to_flat: output directory not selected; returning to caller'
            print,'==============================================='
            return
         endif
      endif

;      print,'==============================================='
;      print,'In : ',program_name
;      print,'Returning because number of provided arguments NE 2'
;      print,''
;      print,'Usage:'
;      print,'IDL> nrl_hico_h5_to_flat,infile,outdir'
;      print,'2 parameters required. First is input h5 file'
;      print,'second is an output directory'
;      print,'==============================================='
;      return
   endif

   if (~h5f_is_hdf5(infile)) then begin
      print,'==============================================='
      print,'In : ',program_name
      print,'Returning; infile is NOT an HDF5 file, infile = ',infile,$
            format='(2a)'
      print,'Please ensure that the directory path and name are correct'
      print,'==============================================='
      return
   endif

   if (~file_test(outdir,/directory,/write)) then begin
      print,'===============================================' 
      print,'Returning; outfir is not a writeable directory, outdir = ',outdir,$
            format='(2a)'
      print,'==============================================='      
      return
   endif

; This IS overkill. Basically, I'm going to read the entire file at
; once, and import all the data into structures. Hey, it works on my
; MacBook Pro with my version of ENVI, so it will work for everyone, right?
; The benefit is that everything is done at once. The (possible)
; disadvantage is that everything is done at once. I will endeavor to test
; this on a few different OS/IDL combinations to see if it is robust
; enough. Thankfully, HICO files are a standard size, which puts a limit on
; the amount of data we expect.    

   hparse=h5_parse(infile,/read_data)

   base=file_basename(hparse._name,'.h5')

   this_bo=byte_ordering()

; BEGIN RADIANCE FILE
; First, the calibrated HICO data and associated header file
   dims=hparse.products._lt._Dimensions
   oname=string(outdir,path_sep(),base,'.bil',format='(4a)')
   openw,lun,oname,/get_lun
   writeu,lun,transpose(hparse.products._lt._data,[1,0,2]) ; re-ordered to BIL from BIP
   close,lun & free_lun,lun

   isf=1.d/reform(hparse.products._lt.slope._data)
   isf_array=make_array(dims[0],type=4,value=floor(isf)*1.0) ; scale factors
   runits=reform(hparse.products._lt.units._data)            ;units
   zplot_y=string(isf,'* radiance (',runits,')',format='(f5.2,3a)') ; zplot
   scene_id=reform(hparse.metadata.hico.target.id._data) 
   target_name=reform(hparse.metadata.hico.target.name._data)
   target_type=reform(hparse.metadata.hico.target.type._data)
   target_aos=reform(hparse.metadata.hico.target.aos._data)
   target_angle=reform(hparse.metadata.hico.target.angle._data)

   description=['Calibrated HICO data',$
               string(' target_name: ',target_name),$
               string(' target_type: ',target_type),$
               string(' target_angle: ',target_angle),$
               string(' target_aos: ',target_aos)]
; construction of history
   history=['']
   tnames=tag_names(hparse.metadata.hico.calibration)
   lens=strlen(tnames)
   for i=6,68 do begin ; normal radiance history part
      key=strlowcase(strmid(reform(tnames[i]),0,lens[i]-1))
      value=strtrim(reform(hparse.metadata.hico.calibration.(i).(3)),2)
      history=[history,string(key,' = ',value,format='(3a)')]
   endfor
   history=[history,'']
   tnames=tag_names(hparse.metadata.hico.target)
   lens=strlen(tnames)
   for i=6,10 do begin ;  target_* part
      key=string('target_',strlowcase(strmid(reform(tnames[i]),0,lens[i])),format='(2a)')
      value=strtrim(reform(hparse.metadata.hico.target.(i).(3)),2)
      history=[history,string(key,' = ',value,format='(3a)')]
   endfor
   history=[history,'']
   tnames=tag_names(hparse.metadata.fgdc.identification_information.time_period_of_content)
   lens=strlen(tnames) 
   for i=6,9 do begin ; "date/time" part
      key=string(strlowcase(strmid(reform(tnames[i]),0,lens[i])),format='(a)')
      value=strtrim(reform(hparse.metadata.fgdc.identification_information.time_period_of_content.(i).(3)),2)
      history=[history,string(key,' = ',value,format='(3a)')]      
   endfor
   history=[history,'']
   tnames=tag_names(hparse.metadata.fgdc.identification_information.spatial_domain)
   lens=strlen(tnames)
   for i=6,9 do begin ; "sort of like bounding box part"
      key=string(strlowcase(strmid(reform(tnames[i]),0,lens[i])),format='(a)')
      value=strtrim(reform(hparse.metadata.fgdc.identification_information.spatial_domain.(i).(3)),2)
      history=[history,string(key,' = ',value,format='(3a)')]      
   endfor   
   history=[history,'']
   history=[history,string('hdf_extraction_program_name_and_version = ',program_name,' ',program_version,$
            format='(4a)')]
   history=[history,string('hdf_extraction_program_author = ',program_author,format='(2a)')]
   history=[history,string('input_hdf_file = ',infile,format='(2a)')]
   history=[history,'']
   
   oname=string(outdir,path_sep(),base,'.hdr',format='(4a)')
   openw,lun,oname,/get_lun
   write_nrl_header,lun,ns=dims[1],nl=dims[2],nb=dims[0],offset=0,inter='bil',$
                    data_type=2,file_type='ENVI Standard',sensor_type='HICO',$
                    wl=0.001d*hparse.products._lt.wavelengths._data,$
                    wavelength_unit='micrometers',$
                    fwhm=0.001d*hparse.products._lt.fwhm._data,$
                    units=runits,xstart=1,ystart=1,$
                    scale=isf_array,byte_order=this_bo,$
                    z_titles=['wavelength (micrometer)',zplot_y],$
                    scene_id=scene_id,descrip=description,$
                    history=history
   close,lun & free_lun,lun
; Note: currently the header file is missing some of the values. It seems
; to me that the L0 Hico Header data isn't present; but it is more
; likely that I haven't yet figured out exactly where the data is. I
; see the keywords; I don't see the values. 
; END RADIANCE FILE

; BEGIN GEOMETRY FILE
; Now a geometry file; convert from 6 BSQ planes to BIL
   oname=strtrim(reform(hparse.metadata.hico.inputs.geometry_file.(3)),2)
   oname=string(outdir,path_sep(),oname,format='(3a)')
   nlen=strlen(oname)
   odat=transpose([[[hparse.navigation.(7).(5)]],[[hparse.navigation.(6).(5)]],$
                   [[hparse.navigation.(9).(5)]],[[hparse.navigation.(8).(5)]],$
                   [[hparse.navigation.(11).(5)]],[[hparse.navigation.(10).(5)]]],[0,2,1])
   openw,lun,oname,/get_lun
   writeu,lun,odat
   close,lun & free_lun,lun
   odat=!null ; free the space
; order is such to get the bands listed below:
   band_names=['longitude (degrees)', 'latitude(degrees)', $
               'view zenith (degrees)','view azimuth (degrees clockwise from N)', $
               'solar zenith (degrees)', 'solar azimuth (degrees clockwise from N)']

   oname=string(strmid(oname,0,nlen-3),'hdr',format='(2a)')
   openw,lun,oname,/get_lun
   write_nrl_header,lun,ns=dims[1],nl=dims[2],nb=6,offset=0,inter='bil',$
                    data_type=4,file_type='ENVI Standard',xstart=1,ystart=1,$
                    byte_order=this_bo,bnames=band_names,sensor_type='HICO',$
                    history=history
   close,lun & free_lun,lun
; END GEOMETRY FILE
   

end
