;-------------------------------------------------------------
;+
; NAME:
;	write_nrl_header.pro       
; PURPOSE:
;	used to write variables to an ENVI style "extended" header.  
;	The variables are passed to the procedure in a 'key_word = data' form.
;	Variables not passed, are not written.
;       
; CALLING SEQUENCE:
;	WRITE_nrl_HEADER, out_unit
;       
; INPUTS:
;	out_unit: The file ID for the open header file that information
;       is to be written to.
;          
; KEYWORD PARAMETERS:
;       Keywords:
;	    NS=ns use this keyword to specify an integer variable that contains
;              the number of samples in the image file.

;	    NL=nl  use this keyword to specify an integer variable that contains
;	    the number of lines in the image file.
;	    NB=nb  use this keyword to specify an integer variable that contains
;	    the number of bands in the image file.
;	    OFFSET=offset use this keyword to specify an integer variable that 
;           contains the offset in bytes to the start of the data in the image 
;           file.
;	    INTER=inter use this keyword to specify a text variable that 
;           contains the image files interleave type.  Current valid values are:;		 'bil', 'bip' and 'bsq'.
;	    DATA_TYPE=data_type use this keyword to specify an integer variable
;           that contains the IDL data type of the image file, using the 
;           convention:
;                1 = byte 
;                2 = integer 
;                3 = long integer
;                4 = floating point 
;                5 = double precision 
;                6 = complex
;	    BYTE_ORDER=byte_order use this keyword to specify an integer 
;           variable that describes the order of the bytes in the image file.  
;           Current valid values are:
;	        0 for files created on DEC and MS-DOS systems 
;               1 for files created on other machines (e.g., SUN, SGI, IBM, 
;                 HP and DG).
;	    BBL=bbl use this keyword to specify an integer array containing the
;           bad bands list.  the size of the array should equal NB.  a value of
;	    one represents a good band and a vale of zero represents a
;	    bad band.
;	    FILE_TYPE=file_type use this keyword to specify a text variable 
;           used by ENVI to define  specific file types.
;	    SENSOR_TYPE=sensor_type use this keyword to specify a text variable
;            used by ENVI to define specific sensor names.
;	    BNAMES=bnames use this keyword to specify a text array that contains
;           the band names associated with each band.
;	    DESCRIP=descrip use this keyword to specify a text array that 
;           contains descriptive information about the image file.
;	    WL=wl use this keyword to specify an floating point array 
;           containing the wavelength for each band.  the size of the array 
;           should equal NB.  The units are in microns for nrl standard files.
;	    FWHM=fwhm use this keyword to specify an floating point array 
;           containing the fwhm for each band.  The size of the array should 
;           equal NB.  The units are in microns for nrl standard files.
;	    XSTART=xstart use this keyword to specify an integer variable that 
;           contains the X starting sample for the first pixel in the image file
;	    YSTART=ystart use this keyword to specify an integer variable that
;           contains the Y starting row for the first pixel in the image file.
;	    MAP_INFO=map_info use this keyword to specify a text array 
;           containing ENVI map information for the image file.
;	    DEF_BANDS=def_bands use this keyword to specify an integer array 
;           that contains the default bands to automatically load into the 
;           ENVI Available Bands List's RGB (3 element array) or Grayscale 
;           (1 element array) text boxes.
;	    Z_RANGE=zrange use this keyword to specify an two element floating 
;           point array that specifies the default minimum and maximum values 
;           for Z plots.
;	    Z_AVERAGE=zavg use this keyword to specify an two element integer 
;           array that specifies the number of pixels in the X and Y directions
;           to average for Z plots.
;	    Z_TITLES=ztitles use this keyword to specify an two element text 
;           array that specifies the default X and Y axis values for Z plots.
;	    DEF_STRETCH=def_stretch use this keywoord to specify a text variable
;           that defines the default ENVI stretch applied when image is 
;           displayed.
;	    PIXEL_SIZE=pixel_size use this keyword to specify a three element 
;           text array that  specifies the X and Y pixel size in the first two 
;           elements, and the units (e.g., units=meters) for the third element.
;           If only the first two elements are present, ENVI assumes the units 
;           are in meters.
;           SCALE=scale use this keyword to specify an floating point array that;           defines the scale factor for each image band.  The scale factor is
;           the number used to convert the band physical units to integer values
;	    for storage.  The number of elements in this array should equal NB.
;	    UNITS=units use this keyword to specify a text variable that defines
;           the units of the image file before scaling.
;	    DATE=date use this keyword to specify a three element integer array
;	    that defines the GMT date of the center image pixel.  array elements
;	    are YYYY, MM, DD.
;	    TIME=time use this keyword to specify a three element floating point
;           array that defines the GMT time of the center image pixel.  
;           The array elements are HH, MM, SS.SSS.
;	    LONGITUDE=longitude use this keyword to specify a three element 
;           floating point array that defines the longitude of the center image
;           pixel.  The array elements are DDD, MM, SS.SSS.
;	    LONG_HEM=long_hem use this keyword to specify a text variable that 
;           defines the E or W longitude hemisphere.
;	    LATITUDE=latitude use this keyword to specify a three element 
;           floating point array that defines the latitude of the center image 
;           pixel.  The array elements are DD, MM, SS.SSS.
;	    LAT_HEM=lat_hem use this keyword to specify a text variable that 
;           defines the N or S latitude hemisphere.
;	    ZENITH_ANG=zenith_ang use this keyword to specify a three element 
;           floating point array that defines the view direction zenith angle 
;           of the center image pixel.  The array elements are DD, MM, SS.SSS 
;           with nadir view defined as (00, 00, 00.000).
;	    AZIMUTH_ANG=azimuth_ang use this keyword to specify a three element
;           floating point array that defines the view direction azimuth angle 
;           of the center image pixel. The array elements are DDD, MM, SS.SSS 
;           with angles defined clockwise of local North.
;	    ALTITUDE=altitude use this keyword to specify the sensor altitude 
;           in kilometers.  The variable is in 'real' format.
;	    HISTORY=history use this keyword to specify a text array that 
;           contains textual history information related to processing of the 
;           image.
;	    image_data_quantity=image_data_quality set the image data quantity,
;           i.e., is the input image rrs, reflectance, radiance at the sensor,
;           radiance at the ground, etc.
;
; This source code was written at the the U.S. Naval Research Laboratory
; and is of US Government origin. If you did not receive this file from the
; Naval Research Laboratory, the caliber/quality of the code cannot be
; vouched for. 
;
; Distribution Statement:
; Distribution authorized to U.S. Government Agencies only (premature
; dissemination) (2012 November 8). Other requests for this document shall
; be referred to Marcos Montes (Naval Research Laboratory, Code 7232,
; Washington, DC, 20375; 202-767-7308; marcos.montes@nrl.navy.mil).
;
;“THIS SOFTWARE IS PROVIDED “AS IS” AND WITHOUT ANY REPRESENTATION OR
;WARRANTY, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
;WARRANTY OF MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR ANY
;WARRANTIES OF ACCURACY OR COMPLETENESS OR THAT THE USE OF THE SOFTWARE
;WILL NOT INFRINGE OR VIOLATE ANY PATENT OR OTHER PROPRIETARY RIGHTS OF ANY
;THIRD PARTY (WHETHER DIRECTLY OR INDIRECTLY).  NRL shall not liable for
;any damage or loss except to the extent that such damage or loss were the
;direct result of the acts of NRL employees AND to the extent such
;liability may be established and pursued against NRL under the Federal
;Tort Claims Act and other applicable law and regulation that specifically
;waive the sovereign immunity of the United States.” 
;
; MODIFICATIONS:
; 2010 Septmber 22 MJM Added missing_lines,num_missing_lines to keep up
; with nrl_file_query (those were added earlier in 2010 by W SNyder to
; nrl_file_query); spectra_names, reflectance_scale_factor (added to
; nrl_file_query by MJM for spectral library, standard ENVI keyword/value
; pairs); major_frame_offsets, minor_frame_offsets (added by MJM to
; nrl_file_query, also standard envi keywords).   
;
; 2009/2010? At some point someone added version and scene_id
;
; 2008 Septemebr 2 - MJM - added image_data_quantity keyword. This is input
;                    to this routine. 
;
; 2008 February 13 MJM tafkaa_ground_elevation added. Changed angle format
; to accept reals. Added /noenvi. If set, then the first "ENVI" at the top
; of the file will not be written. This allows incremental addition of kwds
; to header file.
;
; 2007 October 30 MJM
; Changed output of image_scale_factor to a real since there is no
; guarantee that it will be integer. 
; Added wavelength unit support.
;
; Version 1.0
; Written by W.A.Snyder NRL-DC, Oct 19, 1999.
;
; MJM - Marcos Montes, NRL - marcos.montes@nrl.navy.mil 
; 2008 February 8, Bug Fix for case when descrip has only one element
; 2008 Feb 26 - shortened the wl, fwhm so fewer entries appear per line
;===================================================================
;
Pro write_nrl_header, out_unit, ns=ns, nl=nl, nb=nb, offset=offset, $
                      inter=inter, data_type=data_type, $
                      byte_order=byte_order, bbl=bad_bands, $
                      file_type=file_type, sensor_type=sensor_type, $
                      bnames=band_names, descrip=descrip, wl=wave_len, $
                      fwhm=wave_fwhm, $
                      wavelength_unit=wavelength_unit,$
                      xstart=xstart, ystart=ystart, map_info=map_info, $
                      def_bands=def_bands, $
                      z_range=z_range, z_average=z_average, $
                      z_titles=z_titles, $
                      def_stretch=def_stretch, $
                      pixel_size=pixel_size, $
                      scale=image_scale_factor, $
                      units=image_unscaled_units, $
                      date=image_center_date, $
                      time=image_center_time, $
                      longitude=image_center_long, $
                      long_hem=image_center_long_hem, $
                      latitude=image_center_lat, $
                      lat_hem=image_center_lat_hem, $
                      zenith_ang=image_center_zenith_ang, $
                      azimuth_ang=image_center_azimuth_ang, $
                      altitude=sensor_altitude, $
                      tafkaa_ground_elevation=ground_elevation,$
                      image_data_quantity=image_data_quantity,$
                      history=history,noenvi=noenvi,$
                      version=version,scene_id=scene_id,$
                      missing_lines=missing_lines,$
                      num_missing_lines=num_missing_lines,$
                      spectra_names=spectra_names,$
                      reflectance_scale_factor=reflectance_scale_factor,$
                      major_frame_offsets=major_frame_offsets,$
                      minor_frame_offsets=minor_frame_offsets

   if (keyword_set(noenvi)) then begin
   endif else printf, out_unit, format='("ENVI")'

   if  n_elements(descrip) ne 0 then begin
       printf,out_unit,'description = { ',descrip,format='(a,(a," "),$)'
       printf,out_unit,'}'
   endif

   if n_elements(ns) ne 0 then printf, out_unit, format='("samples = ", i0)', ns
   if n_elements(nl) ne 0 then printf, out_unit, format='("lines = ", i0)', nl
   if n_elements(nb) ne 0 then printf, out_unit, format='("bands = ", i0)', nb
   if n_elements(offset) ne 0 then printf, out_unit, format='("header offset = ", i0)', offset
   if n_elements(inter) ne 0 then printf, out_unit, format='("interleave = ", a)', inter
   if n_elements(data_type) ne 0 then printf, out_unit, format='("data type = ", i0)', $
     data_type
   if n_elements(byte_order) ne 0 then printf, out_unit, format='("byte order = ", i0)', $
     byte_order
   if n_elements(file_type) ne 0 then printf, out_unit, format='("file type = ", a)', $
     file_type
   if n_elements(sensor_type) ne 0 then printf, out_unit, format='("sensor type = ", a)', $
     sensor_type
   if n_elements(version) ne 0 then printf, out_unit, format='("version = ", i0)', $
     version
   if n_elements(scene_id) ne 0 then printf, out_unit, format='("scene_id = ", i0)', $
     scene_id
   if n_elements(xstart) ne 0 then printf, out_unit, format='("x start = ", i0)', xstart
   if n_elements(ystart) ne 0 then printf, out_unit, format='("y start = ", i0)', ystart

   if (n_elements(reflectance_scale_factor) ne 0) then begin ; MJM 2010/09/22
       printf,out_unit,format='("reflectance scale factor = ", f0)',$
         reflectance_scale_factor
   endif

   if (n_elements(minor_frame_offsets) ne 0) then begin ; MJM 2010/09/22
       printf,out_unit,format='(a,i0,",",i0,a)','minor frame offsets = { ',$
         minor_frame_offsets,'}'
   endif

   if (n_elements(major_frame_offsets) ne 0) then begin ; MJM 2010/09/22
       printf,out_unit,format='(a,i0,",",i0,a)','major frame offsets = { ',$
         major_frame_offsets,'}'
   endif

   if (n_elements(num_missing_lines) ne 0) then begin ; MJM 2010/09/22
       printf,out_unit,format='("num_missing_lines = ",i0)',num_missing_lines
   endif

   if (n_elements(missing_lines) ne 0) then begin ; MJM 2010/Sep/22, modeled on bbl
       printf, out_unit, format='(a)','missing_lines = {'
       printf, out_unit, format='(10(" ",i0,:,","),$)', missing_lines
       printf, out_unit, format='("}")'       
   endif

   if n_elements(map_info) ne 0 then begin
       num = n_elements(map_info)
       text = '{'
       for i = 0, num-2 do begin
           text = text + map_info[i] + ', '
       endfor
       text = text + map_info[num-1] + '}'
       printf, out_unit, format='("map info = ",a)', text
   endif

   if n_elements(def_bands) ne 0 then begin
       printf, out_unit, format='("default bands = {",3(i0,:,", "),$)', def_bands
       printf, out_unit, format='("}")'
   endif

   if n_elements(z_range) ne 0 then $
     printf, out_unit, format='("z plot range = {",f0.2,", ",f0.2,"}")', z_range
   if n_elements(z_average) ne 0 then $
     printf, out_unit, format='("z plot average = {",i0,", ",i0,"}")', z_average
   if n_elements(z_titles) ne 0 then $
     printf, out_unit, format='("z plot titles = {",a,", ",a,"}")', z_titles
   if n_elements(def_stretch) ne 0 then $
     printf, out_unit, format='("default stretch = ", a)', $
     def_stretch

   if n_elements(pixel_size) ne 0 then begin
       num = n_elements(pixel_size)
       text = '{'
       for i = 0, num-2 do begin
           text = text + pixel_size[i] + ', '
       endfor
       text = text + pixel_size[num-1] + '}'
       printf, out_unit, format='("pixel size = ",a)', text
   endif

   if n_elements(image_unscaled_units) ne 0 then begin
       num = n_elements(image_unscaled_units)
       text = '{'
       for i = 0, num-2 do begin
           text = text + image_unscaled_units[i] + ', '
       endfor
       text = text + image_unscaled_units[num-1] + '}'
       printf, out_unit, format='("image_unscaled_units = ",a)', text
   endif

   if (n_elements(wavelength_unit) ne 0) then $
     printf,out_unit, format='("wavelength units = ",a)',wavelength_unit

   if (n_elements(image_data_quantity) ne 0) then $
     printf, out_unit, 'image_data_quantity = ',image_data_quantity,format='(2a)'

   if n_elements(image_center_date) ne 0 then $
     printf, out_unit, format='("image_center_date = {",i4.4,", ",i2.2,", ",i2.2,"}")', $
     image_center_date

   if n_elements(image_center_time) ne 0 then $
     printf, out_unit, format='("image_center_time = {",i2.2,", ",i2.2,", ",f8.3,"}")', $
     image_center_time

   if n_elements(image_center_long) ne 0 then $
     printf, out_unit, format='("image_center_long = {",f0,", ",f0,", ",f8.3,"}")', $
     image_center_long

   if n_elements(image_center_long_hem) ne 0 then $
     printf, out_unit, format='("image_center_long_hem = ", a)', image_center_long_hem

   if n_elements(image_center_lat) ne 0 then $
     printf, out_unit, format='("image_center_lat = {",f0,", ",f0,", ",f8.3,"}")', $
     image_center_lat

   if n_elements(image_center_lat_hem) ne 0 then $
     printf, out_unit, format='("image_center_lat_hem = ", a)', image_center_lat_hem

   if n_elements(image_center_zenith_ang) ne 0 then begin
       printf, out_unit, format='("image_center_zenith_ang = {",f0,", ",f0,", ",f8.3,"}")', $
         image_center_zenith_ang
   endif

   if n_elements(image_center_azimuth_ang) ne 0 then begin
       printf, out_unit, format='("image_center_azimuth_ang = {",f0,", ",f0,", ",f8.3,"}")', $
         image_center_azimuth_ang
   endif

   if n_elements(ground_elevation) ne 0 then begin
       printf,out_unit,format='("tafkaa_ground_elevation = ",f0)',ground_elevation
   endif

   if n_elements(sensor_altitude) ne 0 then begin
       printf, out_unit, format='("sensor_altitude = ",f9.2)', sensor_altitude
   endif

   if n_elements(image_scale_factor) ne 0 then begin
       printf, out_unit, format='("image_scale_factor = {")'
       printf, out_unit, format='(7(" ",f0,:,","),$)', image_scale_factor
       printf, out_unit, format='("}")'
   endif

   if n_elements(band_names) ne 0 then begin
       printf, out_unit, format='("band names = {")'
       printf, out_unit, format='(7(" ",a,:,","),$)', band_names
       printf, out_unit, format='("}")'
   endif

   if (n_elements(spectra_names) ne 0) then begin ; MJM 2010/Sep/22, modeled on band names, but 3 per line
       printf, out_unit, format='("spectra names = {")'
       printf, out_unit, format='(3(" ",a,:,","),$)', spectra_names
       printf, out_unit, format='("}")'
   endif

   if n_elements(wave_len) ne 0 then begin
       printf, out_unit, format='("wavelength = {")'
       printf, out_unit, format='(7(" ",f0,:,","),$)', wave_len
       printf, out_unit, format='("}")'
   endif

   if n_elements(wave_fwhm) ne 0 then begin
       printf, out_unit, format='("fwhm = {")'
       printf, out_unit, format='(7(" ",f0,:,","),$)', wave_fwhm
       printf, out_unit, format='("}")'
   endif
   if n_elements(bad_bands) ne 0 then begin
       printf, out_unit, format='("bbl = {")'
       printf, out_unit, format='(10(" ",i0,:,","),$)', bad_bands
       printf, out_unit, format='("}")'
   endif

   if n_elements(history) ne 0 then begin
       printf, out_unit, format='(a)',''
       printf, out_unit, format='("history = begins")'
       printf, out_unit, format='(a)', history
       printf, out_unit, format='("history = ends")'
   endif
   
end
