pro plt_hst_lin, base, test, prm_name, ptitle, base_str, test_str, $
  EXCL_HIGH_BIN = excl_high_bin, PLT_RNG = plt_rng, BIN_SZ = bin_sz, $
  TLUN = tlun
;
;  plt_hst_lin will plot the two histograms of a non-chlor quantity in 
;  a linear axis plot for a base (x) and a test (y)
;
;  call seq:
;    plt_hst_lin, base, test, ptitle, base_str, test_str
;
;  base - x data (y vs x)
;  test - y data (y vs x)
;  prm_name - parameter name
;  ptitle - plot title
;  base_str, test_str - strings to identify the base and test histograms
;  KEYWORDS:
;    EXCL_HIGH_BIN - if set, exclude highest bin count when scaling plot
;          to y axis - this will eliminate pile-ups from messing up the 
;          display
;    PLT_RNG - [low, high] range to plot for non-chlor plots, 
;          default = data min, max
;    BIN_SZ - bin size for non-chlor plots, default = .01 
;    TLUN - logical unit to write out to
;
;  get the histogram arrays of the test and base, use .001 bins
write_file = 0
IF  KEYWORD_SET( tlun )  THEN  write_file = 1
;
IF  KEYWORD_SET( plt_rng )  THEN BEGIN
  hmin = plt_rng[0]
  hmax = plt_rng[1]
  END ELSE  BEGIN
  hmin = MIN( [ base, test ] )
  hmax = MAX( [ base, test ] )
  END
;
IF  KEYWORD_SET( bin_sz ) THEN binsiz = bin_sz  ELSE binsiz = .01
;
PRINT, 'Histogram plots;'
IF  write_file EQ 1  THEN  PRINTF, tlun, 'Histogram plots;'
;
npts = (hmax - hmin ) / binsiz + 1
xax=findgen( npts ) * binsiz + hmin
yaxb = HISTOGRAM( base, binsize = binsiz, min = hmin, max = hmax )
yaxt = HISTOGRAM( test, binsize = binsiz, min = hmin, max = hmax )
;
;  for some / all linear plots, maybe don't scale max to highest, but
;  exclude the very highest point
IF  KEYWORD_SET( excl_high_bin )  THEN $
  BEGIN
  mx = MAX( yaxb )
  ix = WHERE( yaxb LT mx )
  mx1 = MAX( yaxb[ ix ] )
  mx = MAX( yaxt )
  ix = WHERE( yaxt LT mx )
  mx2 = MAX( yaxt[ ix ] )
  ymax = MAX( [mx1, mx2] )
  END $
ELSE $
  ymax = MAX( [ yaxb, yaxt ] )
plot,xax,yaxb,linestyle=1, psym = 10, title=ptitle, $
xtitle = prm_name, ytitle = 'Number', XRANGE=[hmin,hmax], $
YRANGE = [ 0, ymax / .65 ]
oplot, xax,yaxt, psym = 10
;
;  get and add the info on each distribution
npt_b = N_ELEMENTS( base )
mean_b = TOTAL( DOUBLE( base ) ) / npt_b
sd_b = TOTAL( DOUBLE( base ) * DOUBLE( base ) ) / npt_b - $
       mean_b * mean_b
IF  sd_b GT 0  THEN  sd_b = SQRT( sd_b )  ELSE  sd_b = 0
;
res = CONVERT_COORD( [hmin, hmax], [0, ymax / .65], /DATA, /TO_NORMAL )
print,'CONVERT_COORD, res ', res
b_x1 = res[0,0]
b_x2 = res[0,1]
b_y1 = res[1,0]
b_y2 = res[1,1]
ann_x = b_x1 + ( b_x2 - b_x1 ) * .02
ann_y = b_y2 - ( b_y2 - b_y1 ) *.05
ann_dy = ( b_y2 - b_y1 ) *.06
;
str = base_str + ' (dotted line):'
PRINT, str
IF  write_file EQ 1  THEN  PRINTF, tlun, str
XYOUTS, ann_x, ann_y, str, /NORMAL, CHARSIZE = .7
;
str = STRING( npt_b, FORMAT = '( "# points: ", i12 )' )
PRINT, str
IF  write_file EQ 1  THEN  PRINTF, tlun, str
XYOUTS, ann_x, ann_y-ann_dy, str, /NORMAL, CHARSIZE = .7
;
str = STRING( mean_b, FORMAT = '( "Mean: ", f9.4 )' )
PRINT, str
IF  write_file EQ 1  THEN  PRINTF, tlun, str
XYOUTS, ann_x, ann_y-2*ann_dy, str, /NORMAL, CHARSIZE = .7
;
str = STRING( sd_b, FORMAT = '( "Std dev: ", f9.4 )' )
PRINT, str
IF  write_file EQ 1  THEN  PRINTF, tlun, str
XYOUTS, ann_x, ann_y-3*ann_dy, str, /NORMAL, CHARSIZE = .7
;
str = STRING( MIN( base ), MAX( base ), $
   FORMAT = '( "Min: ", f9.4, " Max: ", f9.4 )' )
PRINT, str
IF  write_file EQ 1  THEN  PRINTF, tlun, str
XYOUTS, ann_x, ann_y-4*ann_dy, str, /NORMAL, CHARSIZE = .7
;
;  test...
npt = N_ELEMENTS( test )
mean = TOTAL( DOUBLE( test ) ) / npt
sd = TOTAL( DOUBLE( test ) * DOUBLE( test ) ) / npt - $
       mean * mean
IF  sd GT 0  THEN  sd = SQRT( sd )  ELSE  sd = 0
;
ann_x = b_x1 + ( b_x2 - b_x1 ) * .45
;
str = test_str + ' (solid line):'
PRINT, str
IF  write_file EQ 1  THEN  PRINTF, tlun, str
XYOUTS, ann_x, ann_y, str, /NORMAL, CHARSIZE = .7
;
str = STRING( npt, FORMAT = '( "# points: ", i12 )' )
PRINT, str
IF  write_file EQ 1  THEN  PRINTF, tlun, str
XYOUTS, ann_x, ann_y-ann_dy, str, /NORMAL, CHARSIZE = .7
;
str = STRING( mean, FORMAT = '( "Mean: ", f9.4 )' )
PRINT, str
IF  write_file EQ 1  THEN  PRINTF, tlun, str
XYOUTS, ann_x, ann_y-2*ann_dy, str, /NORMAL, CHARSIZE = .7
;
str = STRING( sd, FORMAT = '( "Std dev: ", f9.4 )' )
PRINT, str
IF  write_file EQ 1  THEN  PRINTF, tlun, str
XYOUTS, ann_x, ann_y-3*ann_dy, str, /NORMAL, CHARSIZE = .7
;
str = STRING( MIN( test ), MAX( test ), $
   FORMAT = '( "Min: ", f9.4, " Max: ", f9.4 )' )
PRINT, str
IF  write_file EQ 1  THEN  PRINTF, tlun, str
XYOUTS, ann_x, ann_y-4*ann_dy, str, /NORMAL, CHARSIZE = .7
;
return
end
