;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    AUTOHIST draws a histogram using automatic bin-sizing.
;
;NAME:
; AUTOHIST
;
;PURPOSE:
;
; Draws a histogram using automatic bin-sizing. It chooses a number
; of bins (initially, 100). If this leads to a histogram in which
; > 1/3 of the central 50% of the bins are empty, it decreases the
; number of bins and tries again. The minimum # bins is 20.
;
;CALLING SEQUENCE:
; AUTOHIST, SAMPLE, XLINES,YLINES, XCENTERS, YCENTERS
;
;INPUT ARGUMENT:
; V = the vector to be histogrammed
;
;OUTPUT ARGUMENT:
; ZX = the x coordinates of the points that trace the rectangular 
;      histogram bins
; ZY = the y coordinates. The histogram is simply ZY vs ZX.
; XX = the x values of the bin centers
; YY = the corresponding y values
;
;SUBROUTINE CALLS:
; MED, which calculates a median
;
;AUTHOR: H. Freudenreich, STX, 1/91
;-
PRO AUTOHIST,V,ZX,ZY,XX,YY  ;    IDL Version 2

WLIM  = 10.

; Use the median and median abs. deviation to establish bounds:
X0=MED(V)
DEV=ABS(V-X0)
Q=WHERE(V LT X0)
IF Q(0) GE 0 THEN MAD1=MED(DEV(Q)) ELSE MAD1=0.
Q=WHERE(V GT X0)
IF Q(0) GE 0 THEN MAD2=MED(DEV(Q)) ELSE MAD2=0.

HI=X0+WLIM*MAD1
LO=X0-WLIM*MAD2

IF (MIN(V) GT 0.) AND (LO LT 0.) THEN LO=1.0E-8

; Initialize the number of bins and begin to iterate:
NUM = N_ELEMENTS(V)
IF NUM GT 125 THEN NBINS=125 ELSE NBINS=FIX(1.25*NUM)

lupe:
NBINS = fix(NBINS*.75)
DEL = (HI-LO)/NBINS
IND=(V-LO)/DEL
Q=WHERE( IND GT (NBINS-1) )
IF Q(0) GE 0 THEN IND(Q)=NBINS-1
Q=WHERE( IND LT 0 )
IF Q(0) GE 0 THEN IND(Q)=0
N=N_ELEMENTS(V)
YY=FLTARR(NBINS)
FOR I=LONG(0),N-1 DO BEGIN
  YY(IND(I))=YY(IND(I))+1.
ENDFOR
XX=FLTARR(NBINS)
XX(0)=LO+DEL/2.
FOR J=1,NBINS-1 DO XX(J)=XX(J-1)+DEL

; Count the fraction of central bins that are empty. If more than 33%, 
; cut the number of bins and try again.

HALFMAX = (HI-LO)/4.
XMID = (HI+LO)/2.
Q = WHERE( ABS(XX-XMID) LE HALFMAX )
NPTS = N_ELEMENTS(Q)*1.
Q = WHERE((ABS(XX-XMID) LE HALFMAX) AND (YY EQ 0))
EPTS = N_ELEMENTS(Q)*1.
EMPTY_FRAC = EPTS/NPTS
PRINT,'EMPTY: ',EMPTY_FRAC
PRINT,' '
IF (EMPTY_FRAC GE .333) AND (NBINS GT 20) THEN GOTO,LUPE

; Now find the coordinates of the corners of the bins:
NX=2*NBINS
ZX=FLTARR(NX)
ZY=FLTARR(NX)
K=-1
FOR J=0,NBINS-1 DO BEGIN
  K=K+1
  ZX(K)=XX(J)-DEL/2.
  ZY(K)=YY(J)
  K=K+1
  ZX(K)=XX(J)+DEL/2
  ZY(K)=YY(J)
ENDFOR
; Plot, then fill, the bins:
PLOT,ZX,ZY
FOR J=0,NBINS-1 DO BEGIN
  IF YY(J) GT 0 THEN BEGIN
     A=[XX(J)-DEL/2.,XX(J)+DEL/2.,XX(J)-DEL/2.,XX(J)+DEL/2.] 
     B=[0.,0.,YY(J),YY(J)]
     POLYFILL,A,B,orientation=45
  ENDIF
ENDFOR

RETURN
END
