;--------------------------------------------------------------------
; Procedure PHISTO
;
; IDL procedure to simplify the production of a histogram plot.
;
; Written By: BA Franz, Applied Research Corp.
; Date:       17 September 1992
;
; Required Inputs:
;   data - any array of data for which a histogram is desired
;
; Optional Inputs:
;   bmin=bmin - minimum bin boundary to include (Def=min(data))
;   bmax=bmax - maximum bin boundary to include (Def=max(data))
;   binsize=binsize - width of bin in data units (Def=(bmax-bmin)/20)
;   nbin=nbin - number of bins to generate between (Def=20).  Only
;               used when binsize not specified.
;   title=title - main plot title (Def=!p.title)
;   xtitle=xtitle - x-axis title (Def=!x.title)
;   ytitle=ytitle - y-axis title (Def=!y.title)
;   linestyle=linstyle - (Def=0)
;   psym=psym - (Def=10)
;   bad_data=bad_data
;   /over - if specified = over-plot mode
;   /prob - compute probability density function instead of simple
;   /norm - normalize to peak
;           histogram
;
;--------------------------------------------------------------------
;
pro phisto,data,binsize=binsize,bmin=bmin,bmax=bmax,nbin=nbin, $
           title=title,xtitle=xtitle,ytitle=ytitle,            $
           linestyle=linestyle,psym=psym,                      $
           over=over,xstyle=xstyle,                            $
           x,y,stats=stats,bad_data=bad_data,prob=prob,color=color,norm=norm


if (n_elements(bad_data) eq 0) then bad_data=-16000

select = where(data gt bad_data,npts)

if (select(0) eq -1) then begin
    npts = n_elements(data)
    select = lindgen(npts)
endif

if (n_elements(bmin) eq 0) then bmin=min(data(select))
if (n_elements(bmax) eq 0) then bmax=max(data(select))
if (bmin eq bmax) then begin
    bmin = bmin-0.1
    bmax = bmax+0.1
endif
if (not keyword_set(binsize)) then begin
    if (not keyword_set(nbin)) then nbin=20
    binsize = float(bmax-bmin)/nbin
endif else begin
    nbin = fix(float(bmax-bmin)/binsize)
endelse

if (not keyword_set(    title)) then title=!p.title
if (not keyword_set(   xtitle)) then xtitle=!x.title
if (not keyword_set(   ytitle)) then ytitle=!y.title
if (not keyword_set(linestyle)) then linestyle=0
if (not keyword_set(     over)) then over=0
if (n_elements(psym) eq 0)      then psym=10
if (not keyword_set(   xstyle)) then xstyle=0
if (n_elements(color) eq 0)     then color=!p.color

;
; Calculate bin centers
;
bin_center = findgen(nbin)*binsize + bmin + binsize/2.0

;
; Don't want to plot an empty bin beyond bmax, so adjust range slightly
;
bmax2 = bmax - 0.0001*binsize

;
; Calculate Histogram
;
x = bin_center
y = histogram(double(data(select)),binsize=binsize,max=bmax2,min=bmin)

if (keyword_set(prob)) then $
    y = float(y)/npts/binsize

if (keyword_set(norm)) then $
    y = float(y)/max(float(y))

print,total(y*binsize)


if (over eq 0) then begin

    plot, x, y, $
          title=title,xtitle=xtitle,ytitle=ytitle, linestyle=linestyle,  $
          psym=psym,xstyle=xstyle,color=color

endif else begin

    oplot, x, y, $
          linestyle=linestyle,  $
          psym=psym,color=color

endelse

if (keyword_set(stats)) then begin
    sigma = stdev(data(select),mean)
    md    = med(data(select))
    xyouts,!cxmin+0.7*(!cxmax-!cxmin),!cymin+0.90*(!cymax-!cymin), $
           'Median = '+strtrim(string(md),2),charsize=0.7
    xyouts,!cxmin+0.7*(!cxmax-!cxmin),!cymin+0.82*(!cymax-!cymin), $
           'Mean   = '+strtrim(string(mean),2),charsize=0.7
    xyouts,!cxmin+0.7*(!cxmax-!cxmin),!cymin+0.74*(!cymax-!cymin), $
           'Stdev  = '+strtrim(string(sigma),2),charsize=0.7
    xyouts,!cxmin+0.7*(!cxmax-!cxmin),!cymin+0.68*(!cymax-!cymin), $
           'Npts   = '+strtrim(string(npts),2),charsize=0.7
endif

return
end
