;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    PHISTO allows the user to compare two histograms on the same plot.
;
;--------------------------------------------------------------------
; Procedure PHISTO
;
; IDL procedure to simplify the production of a histogram plot.
;
; Written By: BA Franz, Applied Research Corp.
; Date:       17 September 1992
;
; Required Inputs:
;   data - any array of data for which a histogram is desired
;
; Optional Inputs:
;   bmin=bmin - minimum bin boundary to include (Def=min(data))
;   bmax=bmax - maximum bin boundary to include (Def=max(data))
;   binsize=binsize - width of bin in data units (Def=(bmax-bmin)/20)
;   nbin=nbin - number of bins to generate between (Def=20).  Only
;               used when binsize not specified.
;   title=title - main plot title (Def=!p.title)
;   xtitle=xtitle - x-axis title (Def=!x.title)
;   ytitle=ytitle - y-axis title (Def=!y.title)
;   linestyle=linstyle - (Def=0)
;   psym=psym - (Def=10)
;   bad_data=bad_data
;   /prob - compute probability density function instead of simple
;           histogram
;
;--------------------------------------------------------------------
;
pro phisto_comp,data1,data2,binsize=binsize,bmin=bmin,bmax=bmax,nbin=nbin, $
           title=title,xtitle=xtitle,ytitle=ytitle,            $
           linestyle=linestyle,psym=psym,                      $
           over=over,xstyle=xstyle,                            $
           stats=stats,bad_data=bad_data,                      $
           prob=prob,norm=norm,perc=perc


if (n_elements(bad_data) eq 0) then bad_data=-16000

s1 = where(data1 gt bad_data,npts1)
s2 = where(data2 gt bad_data,npts2)

if (npts1 gt 0 and npts2 gt 0) then begin
    x1 = data1(s1)
    x2 = data2(s2)
endif else begin
    print,'Insufficient good data to plot'
    return
end

data = [x1,x2]

if (n_elements(bmin) eq 0) then bmin=min(data)
if (n_elements(bmax) eq 0) then bmax=max(data)

if (not keyword_set(binsize)) then begin
    if (not keyword_set(nbin)) then nbin=20
    binsize = float(bmax-bmin)/nbin
endif else begin
    nbin = fix(float(bmax-bmin)/binsize)
endelse

if (not keyword_set(    title)) then title=!p.title
if (not keyword_set(   xtitle)) then xtitle=!x.title
if (not keyword_set(   ytitle)) then ytitle=!y.title
if (not keyword_set(linestyle)) then linestyle=0
if (not keyword_set(     over)) then over=0
if (n_elements(psym) eq 0)      then psym=10
if (not keyword_set(   xstyle)) then xstyle=0

;
; Calculate bin centers
;
bin_center = findgen(nbin)*binsize + bmin + binsize/2.0

;
; Don't want to plot an empty bin beyond bmax, so adjust range slightly
;
bmax2 = bmax - 0.0001*binsize

;
; Calculate Histogram
;
x  = bin_center
y1 = histogram(double(x1),binsize=binsize,max=bmax2,min=bmin)
y2 = histogram(double(x2),binsize=binsize,max=bmax2,min=bmin)

if (keyword_set(prob)) then begin
    y1 = float(y1)/npts1/binsize
    y2 = float(y2)/npts2/binsize
endif

if (keyword_set(norm)) then begin
    y1 = float(y1)/max(y1)
    y2 = float(y2)/max(y2)
endif

if (keyword_set(perc)) then begin
    y1 = float(y1)/npts1*100.
    y2 = float(y2)/npts2*100.
endif

yr = [min([y1,y2]),max([y1,y2])]

;print,total(y1),total(y2)

s1 = where(y1 eq max(y1)) & s2 = where(y2 eq max(y2))
sd1   = stdev(x1,mean1)
sd2   = stdev(x2,mean2)
med1  = med(x1)
med2  = med(x2)
mode1 = x(s1(0))
mode2 = x(s2(0))

;print,'Data 1: ',mean1,med1,mode1,sd1
;print,'Data 2: ',mean2,med2,mode2,sd2

print,''
for i=0,nbin-1 do print,x(i),y1(i),y2(i)

plot, x, y1, $
      title=title,xtitle=xtitle,ytitle=ytitle,line=0,  $
      psym=psym,xstyle=xstyle,yr=yr

oplot, x, y2, $
      linestyle=2,  $
      psym=psym

return
end
