;------------------------------------------------------------------------------
; resample - resample f(x) to an evenly spaced, user specified x.
;
; Average and interpolate as needed to up-sample or down-sample f(x)
; from an unevenly spaced x to evenly spaced x. Procedure will average
; to the approximate final resolution, and then interpolate to the
; final spacing.
;
; Inputs:
; xin - dependent variable, any spacing, assumed monotonic
; yin - f(xin)
; xmin - start (min) value of xout
; xmax - end (max) vale of xout
; dx - spacing for xout
;
; fill=fill - if set, values outside range of xin will be set to fill
;             rather than extrapolated
;
; Outputs:
; xout - output dependent variable
; yout = f(xout) 
; 
; 
; B. Franz, March 2008
;------------------------------------------------------------------------------
pro resample,xin,yin,xmin,xmax,delx,xout,yout,fill=fill

  dx   = delx*1.0
  nx   = (xmax-xmin)/dx+1
  xout = xmin + findgen(nx)*dx

  i = lindgen(n_elements(xin)-1)
  d = min(xin[i+1]-xin[i])
  if (d lt dx) then begin
 
    xtmp = fltarr(nx)
    ytmp = fltarr(nx)

    for i=0L,nx-1 do begin
      s = where(xin gt xout[i]-dx/2 and xin le xout[i]+dx/2+dx/1000,ns)
      if (ns eq 1) then begin
          xtmp[i] = xin[s[0]]
          ytmp[i] = yin[s[0]]
      endif else if (ns gt 1) then begin
          xtmp[i] = avg(xin[s]) 
          ytmp[i] = avg(yin[s]) 
      endif else begin
          xtmp[i] = -999
          ytmp[i] = -999
      endelse
    endfor

    s = where(xtmp gt -999)
    xtmp = xtmp(s)
    ytmp = ytmp(s)

    yout = interpol(ytmp,xtmp,xout)

  endif else begin

    yout = interpol(yin,xin,xout)

  endelse
  

  if (n_elements(fill) ne 0) then begin
     s = where(xout le (min(xin)-1),ns)
     if (ns gt 0) then yout(s) = fill
     s = where(xout ge (max(xin)+1),ns)
     if (ns gt 0) then yout(s) = fill
  endif

  return
end

