;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    GEONAV calculates navigation parameters of a scanning sensor on the
;    surface of an ellipsoid.
;
pro geonav,pos,smat,coef,sun,nsta,ninc,npix,xlat,xlon,solz,sola,senz,sena
;
;  This subroutine performs navigation of a scanning sensor on the 
;  surface of an ellipsoid based on an input orbit position vector and 
;  spacecraft orientation matrix.  It uses a closed-form algorithm for 
;  determining points on the ellipsoidal surface which involves 
;  determining the intersection of the scan plan with the ellipsoid.  
;  The scan angle values are based on the SeaWiFS LAC scan parameters,  
;  i.e., 1285 pixels centered at nadir and 1.5835 milliradians per 
;  pixel.  The scan angles navigated are determined by the scan 
;  parameters passed to the routine, i.e., the start pixel, the number  
;  of pixels and the increment.  (For GAC data these values are 147, 
;  248 and 4, respectively.)  The reference ellipsoid is set according 
;  to the scan intersection coefficients in the calling sequence; a 
;  useful model is an equatorial radius of 6378.137 km. and a 
;  flattening factor of 1/298.257, used by both the Geodetic Reference 
;  System (GRS) 1980 and the World Geodetic System (WGS) 1984.
;
;  It then computes geometric parameters using the pixel locations on
;  the Earth, the spaecraft position vector and the unit Sun vector in
;  the geocentric rotating reference frame.  The outputs are arrays of
;  geodetic latitude and longitude, solar zenith and azimuth and sensor
;  zenith and azimuth.  The azimuth angles are measured from local
;  North toward East.  Flag values of 999. are returned for any pixels
;  whose scan angle is past the Earth's horizon.
;  Calling ArgumentsA
;  Name		Type 	I/O	Description
;
;  pos(3)	R*4	 I	Orbit Position Vector (km)
;  smat(3,3)	R*4	 I	Sensor Orientation Matrix
;  coef(6)	R*4	 I	Scan path coefficients
;  sun(3)	R*4	 I	Sun unit vector in geocentric rotating
;				 reference frame
;  nsta		I*4	 I	Scan start pixel number
;				 SeaWifs LAC:  1
;				 SeaWifs GAC:  147
;  ninc		R*4	 I	Pixel number increment
;				 SeaWifs LAC:  1
;				 SeaWifs GAC:  4
;  npix		I*4	 I	Pixels per scan (maximum value of 1285)
;				 SeaWiFS LAC:  1285
;				 SeaWiFS GAC:  248
;  xlat(*)	R*4	 O	Pixel geodetic latitudes
;  xlon(*)	R*4	 O	Pixel geodetic longitudes
;  solz(*)	R*4	 O	Pixel solar zenith angles
;  sola(*)	R*4	 O	Pixel solar azimuth angles
;  senz(*)	R*4	 O	Pixel sensor zenith angles
;  sena(*)	R*4	 O	Pixel sensor azimuth angles
;
;	Subprograms Called:
;
;	CROSSP		Compute cross product of two vectors
;
;
;	Program written by:	Frederick S. Patt
;				General Sciences Corporation
;				October 20, 1992
;
;	Modification History:
;   
; Created new version for computing effect of out-of-plane component of
;  SeaWiFS scan geometry.  November 24, 1997.
;
; 
 xlat=fltarr(npix)&xlon=fltarr(npix)
 solz=fltarr(npix)&sola=fltarr(npix)
 senz=fltarr(npix)&sena=fltarr(npix)
 geovec=fltarr(3,npix)&x1=fltarr(3,npix)
 ea=fltarr(3,npix)& no=fltarr(3,npix)& up=fltarr(3,npix)
 rh=fltarr(3,npix)& sl=fltarr(3,npix)& rl=fltarr(3,npix)


 	sinc=0.0015911d0
 	f = 1/298.257
 	omf2 = (1.d0-f)*(1.d0-f)
 	rd = 1.d0/omf2
 	alp = 1.2*sinc		; 1.2 pixels out-of-scan

; Compute sines and cosines of scan angles 
 	t = sinc*(ninc*indgen(npix)+nsta-643)
 	sint=sin(t)*cos(alp)
 	cost=cos(t)*cos(alp)
 	sina=sin(alp)

;  Compute sensor-to-surface vectors for all scan angles
	  o = coef(0)*cost*cost+coef(1)*cost*sint+coef(2)*sint*sint
	  p = coef(3)*cost+coef(4)*sint
	  q = replicate(coef(5),npix)
	  r = p*p-4.d0*q*o

;  Check for scan past edge of Earth
	  n1 = where(r lt 0.)
	  if (n1(0) ne -1) then begin
	    xlat(n1) = 999.
	    xlon(n1) = 999.
	    solz(n1) = 999.
	    sola(n1) = 999.
	    senz(n1) = 999.
	    sena(n1) = 999.
	  endif
	  n = where(r ge 0.)
;  Solve for magnitude of sensor-to-pixel vector and compute components
	    d = (-p(n)-sqrt(r(n)))/(2.d0*o(n))

; Compute sensitivity to out-of-plane component
	    h = 2.0*(smat(1,0)*pos(0)+smat(1,1)*pos(1)+smat(1,2)*pos(2)*rd)
	    d = d*(1.d0 + h*sina/sqrt(r))
	    x1(0,n)=d*cost(n)
	    x1(1,n)=d*sina
	    x1(2,n)=d*sint(n)
;  Transform vector from sensor to geocentric frame
	    
	    rh = transpose(smat)#x1
	    for i=0,2 do geovec(i,*) =  pos(i) + rh(i,*)
;    Compute the local vertical, East and North unit vectors  
	    uxy = geovec(0,*)*geovec(0,*)+geovec(1,*)*geovec(1,*)
	    temp = sqrt(geovec(2,*)*geovec(2,*) + omf2*omf2*uxy)
	    up(0,*) = omf2*geovec(0,*)/temp
	    up(1,*) = omf2*geovec(1,*)/temp
	    up(2,*) = geovec(2,*)/temp
	    upxy = sqrt(up(0,*)*up(0,*)+up(1,*)*up(1,*))
	    ea(0,*) = -up(1,*)/upxy
	    ea(1,*) = up(0,*)/upxy
	    ea(2,*) = 0.0
;	    no=crossp(up,ea)
	    no(0,*) = -up(2,*)*ea(1,*)
	    no(1,*) = up(2,*)*ea(0,*)
	    no(2,*) = up(0,*)*ea(1,*) - up(1,*)*ea(0,*)

;    Compute geodetic latitude and longitude
	    xlat(n) = !radeg*asin(up(2,n))
	    xlon(n) = !radeg*atan(up(1,n),up(0,n))

;     Transform the pixel-to-spacecraft and Sun vectors into the local
;	frame
	    rl(0,*) = -ea(0,*)*rh(0,*) - ea(1,*)*rh(1,*) - ea(2,*)*rh(2,*)
	    rl(1,*) = -no(0,*)*rh(0,*) - no(1,*)*rh(1,*) - no(2,*)*rh(2,*)
            rl(2,*) = -up(0,*)*rh(0,*) - up(1,*)*rh(1,*) - up(2,*)*rh(2,*)
	    sl(0,*) = sun#ea
	    sl(1,*) = sun#no
	    sl(2,*) = sun#up
;	    sl(1,*) = no(0,*)*sun(0,*) + no(1,*)*sun(1,*) + no(2,*)*sun(2,*)
;	    sl(2,*) = up(0,*)*sun(0,*) + up(1,*)*sun(1,*) + up(2,*)*sun(2,*)

;    Compute the solar zenith and azimuth
	    solz(n)=!radeg*atan(sqrt(sl(0,n)*sl(0,n)+sl(1,n)*sl(1,n)),sl(2,n))
;      Check for zenith close to zero
;	    if (solz(i) gt 0.05d0) then begin
	      sola(n) = !radeg*atan(sl(0,n),sl(1,n))
;	    endif else begin
;	      sola(i) = 0.0d0
;	    endelse
;            if (sola(i) lt 0.d0) then sola(i) = sola(i) + 360.0d0

;    Compute the sensor zenith and azimuth
	    senz(n)=!radeg*atan(sqrt(rl(0,n)*rl(0,n)+rl(1,n)*rl(1,n)),rl(2,n))
;      Check for zenith close to zero
;	    if (senz(i) gt 0.05d0) then begin
	      sena(n) = !radeg*atan(rl(0,n),rl(1,n))
;	    endif else begin
;	      sena(i) = 0.0d0
;	    endelse
;           if (sena(i) lt 0.d0) then sena(i) = sena(i) + 360.0d0
;	  endelse
;	endfor

	return
	end
