;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    KEP2TLE converts Keplerian to two-line element format.
;
;Line 1
;Column     Description
; 01-01     Line Number of Element Data
; 03-07     Satellite Number
; 10-11     International Designator (Last two digits of launch year)
; 12-14     International Designator (Launch number of the year)
; 15-17     International Designator (Piece of launch)
; 19-20     Epoch Year (Last two digits of year)
; 21-32     Epoch (Julian Day and fractional portion of the day)
; 34-43     First Time Derivative of the Mean Motion
;        or Ballistic Coefficient (Depending on ephemeris type)
; 45-52     Second Time Derivative of Mean Motion (decimal point assumed;
;           blank if N/A)
; 54-61     BSTAR drag term if GP4 general perturbation theory was used.
;           Otherwise, radiation pressure coefficient.  (Decimal point assumed)
; 63-63     Ephemeris type
; 65-68     Element number
; 69-69     Check Sum (Modulo 10)
;           (Letters, blanks, periods, plus signs = 0; minus signs = 1)
;
;Line 2
;Column     Description
; 01-01     Line Number of Element Data
; 03-07     Satellite Number
; 09-16     Inclination [Degrees]
; 18-25     Right Ascension of the Ascending Node [Degrees]
; 27-33     Eccentricity (decimal point assumed)
; 35-42     Argument of Perigee [Degrees]
; 44-51     Mean Anomaly [Degrees]
; 53-63     Mean Motion [Revs per day]
; 64-68     Revolution number at epoch [Revs]
; 69-69     Check Sum (Modulo 10)
;
;All other columns are blank or fixed.
;
;Example:
;
;1 00000U 82072A   96184.13571009 +.00000002 +00000-0 +10198-4 0 09572
;2 00000 098.0724  272.097 0008024 091.3018 268.9100 14.57180717742713
;

function kep2tle,kelm,time

    mu = 3.986005E5					; km^3/s^2

    meanmot = sqrt(mu/kelm.sma^3)*86400./(2.D*!pi)	; rev/day

    idnum1 = '00000U '
    idesig = '82072A   '
    epoch  = string(time,format='(F14.8)')+' '
    dMMdt  = '+.00000000 '
    dMMdt2 = '+00000-0 '
    drag   = '+10000-4 '
    etype  = '0 '
    elmnum = '0000'
    chsum1 = '0'

    idnum2 = '00000 '
    incl   = string(kelm.incl,format='(F8.4)')+' '
    ra     = string(kelm.ra,format='(F8.3)')+' '
    eccen  = string(kelm.eccen) & eccen  = strmid(eccen,strpos(eccen,".")+1,7)+' '
    ap     = string(kelm.ap,format='(F8.4)')+' '
    ma     = string(kelm.ma,format='(F8.4)')+' '
    mm     = string(meanmot,format='(F11.8)')
    orbnum = '00000'
    chsum2 = '0'
    
    
    tle = strarr(2)
    tle(0) = '1 '+idnum1+idesig+epoch+dMMdt+dMMdt2+drag+etype+elmnum+chsum1
    tle(1) = '2 '+idnum2+incl+ra+eccen+ap+ma+mm+orbnum+chsum2

    return, tle

end
