;+NAME/ONE LINE DESCRIPTPION OF ROUTINE:
;    LONLAT2GEOVEC calculates the geocentric pixel location vector from
;    the geodetic longitude.
;
; -----------------------------------------------------------------
; pro lonlat2geovec
;
; Calculates geocentric pixel location vector from geodetic longitude.
;
; Inputs:
;     lon	- longitude of pixel in degrees
;     lat	- longitude of pixel in degrees
;
; Outputs:
;     geovec    - 3-component geocentric vectors in km, with
;                 dimensions equivalent to the input lon.
;
; Written By:
;     Bryan Franz, SAIC GSC, NASA/SIMBIOS Project, July 1998.
;
; -----------------------------------------------------------------

function lonlat2geovec,lon,lat
 
    Re = 6378.137                                 ; Earth radius in km
    f  = 1/298.257                                ; Earth flattening factor

    npts = n_elements(lon)
    sz   = size(lon)
   
    rlon = reform([lon],npts)*!pi/180.            ; geodetic lon in radians
    rlat = reform([lat],npts)*!pi/180.            ; geodetic lat in radians
    phi  = atan(tan(rlat)*(1-f)^2)                ; geocentric latitude
    R    = Re*(1-f)/sqrt(1-(2-f)*f*(cos(phi)^2))  ; dist to Earth surface

    vec      = fltarr(3,npts)
    vec(0,*) = R*cos(phi)*cos(rlon)
    vec(1,*) = R*cos(phi)*sin(rlon)
    vec(2,*) = R*sin(phi)

    case (sz(0)) of
         2: vec = reform(vec,3,sz(1),sz(2))
         3: vec = reform(vec,3,sz(1),sz(2),sz(3))
      else: vec = reform(vec)
    endcase

    return,vec
end
