;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    LONLAT2VIEWANGLE calculates the solar and sensor view angles from geodetic
;    longitude and latitude, position vector, and solar unit vector.
;
; -----------------------------------------------------------------
; pro lonlat2viewangle
;
; Calculates solar and sensor view angles from geodetic longitude 
; and latitude, spacecraft position vector, and solar unit vector.
;
; Inputs:
;     lon	- longitude of pixel in degrees
;     lat	- longitude of pixel in degrees
;     pos(3)	- vector from earth center to spacecraft in ECR, km
;     usun(3)	- unit vector from earth center to sun in ECR
;
; Outputs:
;     solz	- solar zenith angle of pixel in degrees
;     sola	- solar azimuth angle of pixel in degrees
;     senz	- sensor zenith angle of pixel in degrees
;     sena	- sensor azimuth angle of pixel in degrees
;
; Notes:
;     Currently only handles one lon/lat position per call.
;
; Written By:
;     Bryan Franz, SAIC GSC, NASA/SIMBIOS Project, April 1998.
;     (with much help from geonav.pro by Fred Patt)
;
; -----------------------------------------------------------------

pro lonlat2viewangle,lon,lat,pos,usun,solz,sola,senz,sena

    Re = 6378.137D0       ; Earth radius in km
    f  = 1.D0/298.257     ; Earth flattening factor

    rmat   = fltarr(3,3) ; Rotation matrix
    rlon   = lon*!pi/180.D0
    rlat   = lat*!pi/180.D0

    ;
    ; First, we must define the axes (in ECR space) of a
    ; pixel-local coordinate system, with the z-axis along
    ; the geodetic pixel normal, x-axis pointing east, and
    ; y-axis pointing north.
    ;
    up    = [cos(rlat)*cos(rlon),cos(rlat)*sin(rlon),sin(rlat)]
    upxy  = sqrt(up(0)*up(0)+up(1)*up(1))
    ea    = [-up(1)/upxy,up(0)/upxy,0.0]
    no    = crossp(up,ea)

    ;
    ; Now we need the pixel-to-spacecraft vector in ECR,
    ; Compute geocentric pixel location vector in km and subtract
    ; from spacecraft position.
    ;
    phi   = atan(tan(rlat)*(1-f)^2)                ; geocentric latitude
    R     = Re*(1-f)/sqrt(1-(2-f)*f*(cos(phi)^2))  ; dist to Earth surface
    gvec  = R*[cos(phi)*cos(rlon),cos(phi)*sin(rlon),sin(phi)]
    scvec = pos - gvec

    ;
    ; Now we can transform the pixel-to-spacecraft and Sun 
    ; vectors into the local frame.
    ;
    rmat(0,*) = ea
    rmat(1,*) = no
    rmat(2,*) = up
    senl      = rmat # scvec
    sunl      = rmat # usun 

    ;
    ; Compute the solar zenith and azimuth
    ;
    solz = atan(sqrt(sunl(0)*sunl(0)+sunl(1)*sunl(1)),sunl(2)) * !radeg
    if (solz gt 0.05d0) then begin
        sola = atan(sunl(0),sunl(1)) * !radeg
    endif else begin
        sola = 0.0d0
    endelse
    if (sola lt 0.d0) then $
        sola = sola + 360.0d0

    ;
    ; Compute the sensor zenith and azimuth
    ;
    senz = atan(sqrt(senl(0)*senl(0)+senl(1)*senl(1)),senl(2)) * !radeg
    if (senz gt 0.05d0) then begin
        sena = atan(senl(0),senl(1)) * !radeg
    endif else begin
        sena = 0.0d0
    endelse
    if (sena lt 0.d0) then $
        sena = sena + 360.0d0

;    print,pos
;    print,scvec
;    print,gvec
;    print,lon,lat
;    print,solz,sola
;    print,senz,sena

    return
end


