;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    NAVIGATE_LINE generates location information for a single scan line.
;
;------------------------------------------------------------------------------
;
; Navigate_Line
;
; This procedure generates geodetic longitudes, geodetic latitudes, spacecraft
; zenith and azimuth angles, and solar zenith and azimuth angles for one
; SeaWiFS scan line, based on an orbit position vector, a sun unit vector, and
; a sensor orientation matrix.  For a complete description and derivation of
; the algorithms used, see "Exact closed-form geolocation algorithm for Earth
; survey sensors" by F.S. Patt and W.W. Gregg, 1994, Int. J. Remote Sensing,
; 15, 3719-3734.
;
; This procedure has been adapted from the SeaWiFS MOPS IDL procedures
; NAVLINE.PRO, NAVPIX.PRO, GEONLINE.PRO, and GEONPIX.PRO.
;
; AUTHOR:   Gene Eplee
;           General Sciences Corp.
;           286-0953
;           22 August 1997
; 
; Modification History:
;
; 02/98, B. Franz, updated to new ifov value and added out-of-plane
; correction.  Also fixed bug in crossp calculation of the no-vector, that 
; was resulting in erroneous viewing angles.
;
;------------------------------------------------------------------------------
;
; CALLING SEQUENCE:
;   Navigate_Line,orb_vec,sun_ref,sen_mat,scan_ell,longitude,latitude,$
;                 sen_zenith,sen_azimuth,sol_zenith,sol_azimuth,gac=gac
;
; ARGUMENTS: 
;   orb_vec(3)        R*4    I    Orbit position vector (km)
;   sun_ref(3)        R*4    I    Sun unit vector ub ECEF frame
;   sen_mat(3,3)      R*4    I    Sensor orientation matrix
;   scan_ell(6)       R*4    I    Scan path coefficients
;   longitude(1285)   R*4    O    Pixel geodetic longitudes
;   latitude(1285)    R*4    O    Pixel geodetic latitudes
;   sen_zenith(1285)  R*4    O    Spacecraft zenith angles
;   sen_azimuth(1285) R*4    O    Spacecraft azimuth angles
;   sol_zenith(1285)  R*4    O    Solar zenith angles
;   sol_azimuth(1285) R*4    O    Solar azimuth angles
;
;
; KEYWORDS:
;   GAC causes results for GAC scan lines to be returned.
;
; PROCEDURE:
;
; SUBROUTINES CALLED:
;
;------------------------------------------------------------------------------
;
; MODIFICATION HISTORY:
;
;------------------------------------------------------------------------------
;
PRO Navigate_Line,orb_vec,sun_ref,sen_mat,scan_ell,longitude,latitude,$
                  sen_zenith,sen_azimuth,sol_zenith,sol_azimuth,gac=gac

;
; Define the required constants,  set the input parameters to R*8, and define
;    the necessary arrays.
;

;
;  Define the navigation parameters.
;

   pi     = 3.141592653589793d0	; Pi
   radeg  = 180.0d0/pi		; Rad to deg conversion factor
   au     = 149.59787066d+6	; Astronomical Unit (km)
   f      = 1.0d0/298.257d0	; Earth flattening factor
   ifov   = 0.0015911d0         ; instrument field-of-view  (radians)
   ifovd  = ifov*radeg          ; instrument fieod-of-view (degrees)
   mld    = 384401.0d0		; Mean lunar distance (km)
   omegae = 7.29211585494d-5	; Earth rotation rate (rad/s)
   omf2   = (1.0d0-f)*(1.0d0-f)	; (1-f)**2
   re     = 6378.137d0		; Earth equatorial radius (km)
   rem    = 6371.0d0		; Earth mean radius (km)
   scan_rate = 6.0D0            ; SeaWiFS scan rate (lines/s)
   xk     = 0.0056932D0		; Constant of aberration

   pos    = DOUBLE(orb_vec)
   rm     = DOUBLE(sen_mat)
   coef   = DOUBLE(scan_ell)
   sun    = DOUBLE(sun_ref)
   scan   = DBLARR(1285)
   lat    = DBLARR(1285)
   lon    = DBLARR(1285)
   q      = DBLARR(1285)
   Qx     = DBLARR(1285)
   Qy     = DBLARR(1285)
   Qz     = DBLARR(1285)
   rmtq   = DBLARR(3,1285)
   geovec = DBLARR(3,1285)
   tmp    = DBLARR(1285)
   uxy    = DBLARR(1285)
   temp   = DBLARR(1285)
   up     = DBLARR(3,1285)
   upxy   = DBLARR(1285)
   ea     = DBLARR(3,1285)
   no     = DBLARR(3,1285)
   sv     = DBLARR(1285)
   sn     = DBLARR(1285)
   se     = DBLARR(1285)
   senz    = DBLARR(1285)
   sena    = DBLARR(1285)
   sunv   = DBLARR(1285)
   sunn   = DBLARR(1285)
   sune   = DBLARR(1285)
   solz   = DBLARR(1285)
   sola   = DBLARR(1285)

;
; Compute elevation (out-of-plane) angle
;
   elev = ifov*1.2
   sinl = sin(elev)
   cosl = cos(elev)

;
; Compute the cosines and sines for all scan angles.
;    Compute the index for GAC scan lines.
;
   ipix   = INDGEN(1285)+1
   scan   = DOUBLE(ipix-643)*ifov
   cosa   = COS(scan)*cosl
   sina   = SIN(scan)*cosl
   idx    = 146 + (4*(ipix(0:247)-1))

;
; Compute correction factor for out-of-plane angle
;
   h = (rm(1,0)*pos(0)+rm(1,1)*pos(1)+rm(1,2)*pos(2)/omf2)*2.d0

;
; Compute the sensor-to-surface vectors for all scan angles.
;
   a = coef(0)*cosa*cosa + coef(1)*cosa*sina + coef(2)*sina*sina
   b = coef(3)*cosa + coef(4)*sina
   c = coef(5)

;
; Begin the solution of the quadratic equation
;
   r = b*b - 4.0d0*a*c
;
; Check for scan angles past the edge of the Earth.
;
   i = WHERE(r LT 0.0,count)
   IF (count NE 0) THEN BEGIN
      lat(i)  = 999.0
      lon(i)  = 999.0
      senz(i)  = 999.0
      solz(i) = 999.0
   ENDIF
;
; Solve for the magnitude of the sensor-to-pixel vector and compute its
;    components.
;
   i = WHERE(r GE 0.0,count)
   IF (count NE 0) THEN BEGIN
      q(i)  = (-b-SQRT(r(i)))/(2.0d0*a) * (1.D0 + sinl*h/sqrt(r(i)))
      Qx(i) = q(i)*cosa(i)
      Qy(i) = q(i)*sinl 
      Qz(i) = q(i)*sina(i)

;
; Transform the vector from the sensor frame to the geocentric frame
;
      FOR j=0,2 DO BEGIN
         rmtq(j,i)   = Qx(i)*rm(0,j) + Qy(i)*rm(1,j) + Qz(i)*rm(2,j)
         geovec(j,i) = rmtq(j,i) + pos(j)
      ENDFOR
;
; Compute the geodetic latitude and longitude for the scan line.
;
      tmp(i) = SQRT(geovec(0,i)*geovec(0,i) + geovec(1,i)*geovec(1,i))*omf2
      lat(i) = radeg*ATAN(geovec(2,i),tmp(i))
      lon(i) = radeg*ATAN(geovec(1,i),geovec(0,i))

;
; Compute the local vertical, East, and North unit vectors.
;
      uxy(i)  = geovec(0,i)*geovec(0,i) + geovec(1,i)*geovec(1,i)
      temp(i) = SQRT(geovec(2,i)*geovec(2,i) + omf2*omf2*uxy(i))
      up(0,i) = omf2*geovec(0,i)/temp(i)
      up(1,i) = omf2*geovec(1,i)/temp(i)
      up(2,i) = geovec(2,i)/temp(i)
      upxy(i) = SQRT(up(0,i)*up(0,i) + up(1,i)*up(1,i))
      ea(0,i) = -up(1,i)/upxy
      ea(1,i) = up(0,i)/upxy
      for j=0,n_elements(i)-1 do no(*,i(j)) = CROSSP(up(*,i(j)),ea(*,i(j)))
;
; Decompose the spacecraft vector into vertical (up), North (no), and East (ea)
;    components.
;
      sv(i)  = TOTAL(-rmtq(*,i)*up(*,i),1)
      sn(i)  = TOTAL(-rmtq(*,i)*no(*,i),1)
      se(i)  = TOTAL(-rmtq(*,i)*ea(*,i),1)
;
; Compute the spacecraft zenith and azimuth angles.
;
      senz(i) = radeg*ATAN(SQRT(sn(i)*sn(i)+se(i)*se(i)),sv(i))
      sena(i) = radeg*ATAN(se(i),sn(i))

;
; Decompose the sun vector into vertical (up), North (no), and East (ea)
;    components.
;
      sunv(i) = TOTAL(sun(*,i)*up(*,i),1)
      sunn(i) = TOTAL(sun(*,i)*no(*,i),1)
      sune(i) = TOTAL(sun(*,i)*ea(*,i),1)
;
; Compute the solar zenith and azimuth angles.
;
      solz(i) = radeg*ATAN(SQRT(sunn(i)*sunn(i)+sune(i)*sune(i)),sunv(i))
      sola(i) = radeg*ATAN(sune(i),sunn(i))

   ENDIF

;
; Convert the results to single precision.
;
   latitude    = FLOAT(lat)
   longitude   = FLOAT(lon)
   sen_zenith  = FLOAT(senz)
   sen_azimuth = FLOAT(sena)
   sol_zenith  = FLOAT(solz)
   sol_azimuth = FLOAT(sola)

;
; Compute the values for GAC scan lines if required.
;
   IF KEYWORD_SET(gac) THEN BEGIN
      latitude    = latitude(idx)
      longitude   = longitude(idx)
      sen_zenith  = sen_zenith(idx)
      sen_azimuth = sen_azimuth(idx)
      sol_zenith  = sol_zenith(idx)
      sol_azimuth = sol_azimuth(idx)
   ENDIF

END
