;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    ORIENT performs a calculation of the sensor orientation.
;
;  This subroutine performs a simple calculation of the sensor
;  orientation from the orbit position vector and input values of the
;  attitude offset angles.  The calculations assume that the angles
;  represent the yaw, roll and pitch offsets between the local vertical
;  reference frame (at the spacecraft position) and the sensor frame. 
;  Sensor tilt angles are assumed to be included in the pitch angle. 
;  The outputs are the matrix which represents the transformation from
;  the geocentric rotating to sensor frame, and the coefficients which
;  represent the Earth scan track in the sensor frame.
;
;  The reference ellipsoid uses an equatorial radius of 6378.137 km and
;  a flattening factor of 1/298.257 (WGS 1984). 
pro orient,pos,vel,att,smat,coef

on=fltarr(3)&p=fltarr(3)&v=fltarr(3)&x=fltarr(3)&y=fltarr(3)
z=fltarr(3)&sm1=fltarr(3,3)&sm2=fltarr(3,3)&sm3=fltarr(3,3)
smat=fltarr(3,3)&coef=fltarr(6)
	re = 6378.137d0&rem = 6371.d0
;	radeg = 180.d0/3.14159265359d0
	f = 1./298.257d0
        omegae = 7.29211585494d-5
	omf2 = (1.d0-f)*(1.d0-f)

;  Compute constants for navigation model using Earth radius values
	rd=1.d0/omf2

;  Determine local vertical reference axes
;   Uses method of local ellipsoid approximation good to 0.3 arcsecond

	    v = vel
	    p = pos
;	    v(0) = v(0) - p(1)*omegae
;    	    v(1) = v(1) + p(0)*omegae
;  Compute X axis as local nadir vector
	pm = sqrt(p(0)*p(0)+p(1)*p(1)+p(2)*p(2))
	omf2p = (omf2*rem + pm - rem)/pm
	pxy = p(0)*p(0)+p(1)*p(1)
	temp = sqrt(p(2)*p(2) + omf2p*omf2p*pxy)
	x(0) = -omf2p*p(0)/temp
	x(1) = -omf2p*p(1)/temp
	x(2) = -p(2)/temp

;  Compute Z axis along orbit normal
	on=crossp(v,x)
	onm = sqrt(on(0)*on(0)+on(1)*on(1)+on(2)*on(2))
	z = on/onm
;  Compute Y axis to complete orthonormal triad
	y=crossp(z,x)
;  Convert Euler angles to matrix
	euler,att,sm1

;  Compute attitude matrix in geocentric frame
;   Store local vertical reference vectors in matrix 
	    sm2(0,*)=x
	    sm2(1,*)=y
	    sm2(2,*)=z

;   Apply attitude offset matrix 
	sm3=sm1#sm2

;  Compute coefficients of intersection ellipse in scan plane
	coef(0) = 1.d0+(rd-1.d0)*sm3(0,2)*sm3(0,2)
	coef(1) = (rd-1.d0)*sm3(0,2)*sm3(2,2)*2.d0
	coef(2) = 1.d0+(rd-1.d0)*sm3(2,2)*sm3(2,2)
	coef(3) = (sm3(0,0)*p(0)+sm3(0,1)*p(1)+sm3(0,2)*p(2)*rd)*2.d0
	coef(4) = (sm3(2,0)*p(0)+sm3(2,1)*p(1)+sm3(2,2)*p(2)*rd)*2.d0
	coef(5) = p(0)*p(0)+p(1)*p(1)+p(2)*p(2)*rd-re*re

;  Transfer sensor orientation matrix to output array
	smat = sm3

	return
	end

	pro euler,a,xm
;  Computes coordinate transformation matrix corresponding to Euler 
;   sequence; assumes order of rotations is X, Y, Z
;
;  Reference:  Wertz, Appendix E

xm1=fltarr(3,3)&xm2=fltarr(3,3)&xm3=fltarr(3,3)&xm=fltarr(3,3)
xmm=fltarr(3,3)

	c1=cos(a(0)/!radeg)
	s1=sin(a(0)/!radeg)
	c2=cos(a(1)/!radeg)
	s2=sin(a(1)/!radeg)
	c3=cos(a(2)/!radeg)
	s3=sin(a(2)/!radeg)
;  Convert individual rotations to matrices
	xm1(0,0)=1.d0
	xm1(1,1)=c1
	xm1(2,2)=c1
	xm1(1,2)=s1
	xm1(2,1)=-s1
	xm2(1,1)=1.d0
	xm2(0,0)=c2
	xm2(2,2)=c2
	xm2(2,0)=s2
	xm2(0,2)=-s2
	xm3(2,2)=1.d0
	xm3(1,1)=c3
	xm3(0,0)=c3
	xm3(0,1)=s3
	xm3(1,0)=-s3
;  Compute total rotation as xm3*xm2*xm1
	xmm=xm2#xm1
	xm=xm3#xmm
	return
	end

