;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    SUNRISESET - determines nearest sun rise/set time
;
;------------------------------------------------------------------------
; PROCEDURE SUNRISESET
;
; IDL procedure to compute the nearest Sun rise and Sun set times. If input
; time is during daylight, the millisecs of day of the preceeding rise
; and subsequent set times will be returned. If the input time is at night,
; the program will search forward in time for the next rise and set.
;
; Programmer: Bryan A. Franz, SAIC-GSC
; Date Written: 8/2000
;
; Inputs:
;   year  - GMT year (scalar)
;   day   - GMT day of each measurement (scalar)
;   msec  - GMT millisecs of day (scalar)
;   lon   - longitude (scalar, deg)
;   lat   - latitude (scalar, deg)
;
; Outputs:
;   msec_r - Sun rise time in millisecs of day, relative to input year
;            and day (can be negative).
;   msec_r - Sun set time in millisecs of day, relative to input year
;            and day.
;
;------------------------------------------------------------------------
pro sunriseset,year,day,msec,lon,lat,msec_r,msec_s

    ; 
    ; Get solar zenith angle for every 10-secs of a two-day period
    ; centered on the input time.
    ;
    dsec = 10
    nsec = fix(86400/dsec*2)

    year1 = fltarr(nsec) + year
    day1  = fltarr(nsec) + day
    lon1  = fltarr(nsec) + lon
    lat1  = fltarr(nsec) + lat

    msec1 = (lindgen(nsec)-nsec/2)*dsec*1000 + msec

    lonlat2solz,lon1,lat1,year1,day1,msec1,solz1

    i = where(msec1 eq msec) & i=i(0)
    
    ;
    ; Search to find time of solar-zenith crossing 90-deg
    ;
    if (solz1(i) lt 90) then begin
        ; Sun is up, so back-up to find sunrise
        irise = i
        while (solz1(irise) lt 90) do irise=irise-1
        ; Sun is up, so go forward to find sunset
        iset = i
        while (solz1(iset) lt 90) do iset=iset+1
    endif else begin
        ; Sun is down, so go  forward to find sunrise
        irise = i
        while (solz1(irise) lt 90) do irise=irise+1
        ; go forward to find sunset
        iset = irise
        while (solz1(iset) lt 90) do iset=iset+1
    endelse

    msec_r = msec1(irise)
    msec_s = msec1(iset)

    return
end