;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    OCHDF2GENERIC generates a generic level 1 file from an hdf file.
;
pro ochdf2generic,ifile,ofile,odir=odir,level=level,latrange=latrange

    if (n_elements(level)    eq 0) then level=1
    if (n_elements(latrange) eq 0) then latrange=[-90.1,90.1]

    ;
    ; open the file and get the ID
    ;
    fileID = HDF_SD_START( ifile, /READ )

    ;
    ; Get attributes pertaining to the full scene
    ;
    year          = get_hdf_attr(fileID=fileID,attr="Start Year") & year = year(0)
    day           = get_hdf_attr(fileID=fileID,attr="Start Day")  & day  = day (0)
    scansPerScene = get_hdf_attr(fileID=fileID,attr="Number of Scan Lines")
    pixPerScan    = get_hdf_attr(fileID=fileID,attr="Pixels per Scan Line")
    linesPerScan  = get_hdf_attr(fileID=fileID,attr="Lines per Scan")
    linesPerScene = scansPerScene(0) * linesPerScan(0)

    ;
    ; If this is a subscened file, get attributes for the subscene
    ;
    tip_ctr_lat = get_hdf_attr(fileID=fileID,attr="Tip Center Latitude")
    if (tip_ctr_lat(0) ne  -999) then begin
        sline = get_hdf_attr(fileID=fileID,attr="Tip Upper Left Line")  - 1
        spix  = get_hdf_attr(fileID=fileID,attr="Tip Upper Left Pixel") - 1
        nline = get_hdf_attr(fileID=fileID,attr="Tip Size Pixel")
        npix  = get_hdf_attr(fileID=fileID,attr="Tip Size Line")
        subscene        = 1
    endif else begin
        sline           = 0
        spix            = 0
        nline           = linesPerScene
        npix            = pixPerScan
        subscene        = 0
    endelse

    ;
    ; Compute and store the view angles for the scene (or subscene)
    ;    
    ochdfnav,fileID,sline=sline,spix=spix,nline=nline,npix=npix, $
             lon,lat,sunz,suna,senz,sena,latrange=latrange

    ;
    ; Get time for each line of full scene
    ;
    msec = rd_sds(fileID,'msec')
    msec = reform( (lonarr(linesPerScan(0))+1) # msec, linesPerScene)
    msec_start = msec(0)
    msec = msec(sline:sline+nline-1)
    maxday = 365 + abs(leapchk(year))

    ;
    ; Set-up for loading of image data
    ;
    if (level eq 1) then begin

        ;
        ;  Create output data structure
        ;
        r = generic_l1_str(1,npix)

        ;
        ; Get image data scaling parameters (y = m x + b)
        ;
        m = fltarr(8)
        m(0) = get_hdf_attr(fileID=fileID,attr='slope',sds='l1b_b1_data')
        m(1) = get_hdf_attr(fileID=fileID,attr='slope',sds='l1b_b2_data')
        m(2) = get_hdf_attr(fileID=fileID,attr='slope',sds='l1b_b3_data')
        m(3) = get_hdf_attr(fileID=fileID,attr='slope',sds='l1b_b4_data')
        m(4) = get_hdf_attr(fileID=fileID,attr='slope',sds='l1b_b5_data')
        m(5) = get_hdf_attr(fileID=fileID,attr='slope',sds='l1b_b6_data')
        m(6) = get_hdf_attr(fileID=fileID,attr='slope',sds='l1b_b7_data')
        m(7) = get_hdf_attr(fileID=fileID,attr='slope',sds='l1b_b8_data')
        b = fltarr(8)
        b(0) = get_hdf_attr(fileID=fileID,attr='intercept',sds='l1b_b1_data')
        b(1) = get_hdf_attr(fileID=fileID,attr='intercept',sds='l1b_b2_data')
        b(2) = get_hdf_attr(fileID=fileID,attr='intercept',sds='l1b_b3_data')
        b(3) = get_hdf_attr(fileID=fileID,attr='intercept',sds='l1b_b4_data')
        b(4) = get_hdf_attr(fileID=fileID,attr='intercept',sds='l1b_b5_data')
        b(5) = get_hdf_attr(fileID=fileID,attr='intercept',sds='l1b_b6_data')
        b(6) = get_hdf_attr(fileID=fileID,attr='intercept',sds='l1b_b7_data')
        b(7) = get_hdf_attr(fileID=fileID,attr='intercept',sds='l1b_b8_data')

        ;
        ; Build the L1B header
        ;
        hdr = bytarr(str_len(r))
        hdr(0: 3) = byte(long(2),0,4)
        hdr(4: 7) = byte(long(str_len(r)),0,4)
        hdr(8:11) = byte(long(npix),0,4)

        type='L1B'

    endif else begin

        ;
        ;  Create output data structure
        ;
        r = generic_l2sg_str(1,npix)

        ;
        ; Get image data scaling parameters (y = m x + b)
        ;
        m = fltarr(8)
        m(0) = get_hdf_attr(fileID=fileID,attr='slope',sds='nLw_412')
        m(1) = get_hdf_attr(fileID=fileID,attr='slope',sds='nLw_443')
        m(2) = get_hdf_attr(fileID=fileID,attr='slope',sds='nLw_490')
        m(3) = get_hdf_attr(fileID=fileID,attr='slope',sds='nLw_520')
        m(4) = get_hdf_attr(fileID=fileID,attr='slope',sds='nLw_565')
        m(5) = get_hdf_attr(fileID=fileID,attr='slope',sds='La_670')
        m(6) = get_hdf_attr(fileID=fileID,attr='slope',sds='eps_68')
        m(7) = get_hdf_attr(fileID=fileID,attr='slope',sds='tau_865')
        b = fltarr(8)
        b(0) = get_hdf_attr(fileID=fileID,attr='intercept',sds='nLw_412')
        b(1) = get_hdf_attr(fileID=fileID,attr='intercept',sds='nLw_443')
        b(2) = get_hdf_attr(fileID=fileID,attr='intercept',sds='nLw_490')
        b(3) = get_hdf_attr(fileID=fileID,attr='intercept',sds='nLw_520')
        b(4) = get_hdf_attr(fileID=fileID,attr='intercept',sds='nLw_565')
        b(5) = get_hdf_attr(fileID=fileID,attr='intercept',sds='La_670')
        b(6) = get_hdf_attr(fileID=fileID,attr='intercept',sds='eps_68')
        b(7) = get_hdf_attr(fileID=fileID,attr='intercept',sds='tau_865')

        ;
        ; Build the L2 header
        ;
        hdr = bytarr(str_len(r))
        hdr(0: 3)  = byte(long(2),0,4)
        hdr(4: 7)  = byte(long(str_len(r)),0,4)
        hdr(8:11)  = byte(long(npix),0,4)
        hdr(12:15) = byte(1L,0,4)

        type='L2'

    endelse

    ;
    ; Open output file and write header
    ;
    if (n_elements(odir)  eq 0) then odir='.'
    if (n_elements(ofile) eq 0) then begin
        hour   = (msec(0)/1000L/3600L)
        minute = (msec(0)/1000L - hour*3600L)/60L
        sec    = (msec(0)/1000L - hour*3600L - minute*60L)
        ofile = odir+'/'+'O'+ $
                num2str(year)+num2str(day,3)+num2str(hour,2)+ $
                num2str(minute,2)+num2str(sec,2)+ $
                '.'+type+'_'+num2str(npix)
    endif
    print,'Writing file ',ofile,' from ',ifile
    openw,/get_lun,lun,ofile
    writeu,lun,hdr

    ;
    ; Now, read one scan at a time and write to output file
    ;
    for i=0,nline-1 do begin

      lat1 = lat(0)
      lat2 = lat(npix-1)

      if (i mod 10 eq 0) then print,'reading scan ',i

      if ((lat1 ge latrange(0) and lat1 le latrange(1))  or $
          (lat2 ge latrange(0) and lat2 le latrange(1))) then begin

        print,'writing scan ',i

        ;
        ; Load scan times
        ;
        r.year = year
        r.day  = day
        r.msec = msec(i)

        ;
        ; If the day changed between the start of the scene and the start 
        ; of the subscene, adjust the day/year accordingly.
        ;
        if (msec(i) lt msec_start) then begin
            r.day  = r.day+1
            if (day gt maxday) then begin
                r.year = r.year + 1
                r.day  = r.day  - maxday
            endif
        endif

        ;
        ; Load standard navigation
        ;
        r.lon  = lon (*,i)
        r.lat  = lat (*,i)
        r.sunz = sunz(*,i)

        ;
        ; Load image data (Lt or nLw)
        ;
        if (level eq 1) then begin

            r.suna = suna(*,i)
            r.senz = senz(*,i)
            r.sena = sena(*,i)

            ;
            ; Load the image data arrays
            ;
            r.data(0,*) = (reform(rd_sds(fileID,'l1b_b1_data',start=[0,i],count=[npix,1]), $
                           1,npix) and 8191) * m(0)+b(0)
            r.data(1,*) = (reform(rd_sds(fileID,'l1b_b2_data',start=[0,i],count=[npix,1]), $
                           1,npix) and 8191) * m(1)+b(1)
            r.data(2,*) = (reform(rd_sds(fileID,'l1b_b3_data',start=[0,i],count=[npix,1]), $
                           1,npix) and 8191) * m(2)+b(2)
            r.data(3,*) = (reform(rd_sds(fileID,'l1b_b4_data',start=[0,i],count=[npix,1]), $
                           1,npix) and 8191) * m(3)+b(3)
            r.data(4,*) = (reform(rd_sds(fileID,'l1b_b5_data',start=[0,i],count=[npix,1]), $
                           1,npix) and 8191) * m(4)+b(4)
            r.data(5,*) = (reform(rd_sds(fileID,'l1b_b6_data',start=[0,i],count=[npix,1]), $
                           1,npix) and 8191) * m(5)+b(5)
            r.data(6,*) = (reform(rd_sds(fileID,'l1b_b7_data',start=[0,i],count=[npix,1]), $
                           1,npix) and 8191) * m(6)+b(6)
            r.data(7,*) = (reform(rd_sds(fileID,'l1b_b8_data',start=[0,i],count=[npix,1]), $
                           1,npix) and 8191) * m(7)+b(7)

        endif else begin

            ;
            ; Load the image data arrays
            ;
            r.nLw(0,*) = reform(rd_sds(fileID,'nLw_412',start=[0,i],count=[npix,1]), $
                          1,npix) * m(0)+b(0)
            r.nLw(1,*) = reform(rd_sds(fileID,'nLw_443',start=[0,i],count=[npix,1]), $
                          1,npix) * m(1)+b(1)
            r.nLw(2,*) = reform(rd_sds(fileID,'nLw_490',start=[0,i],count=[npix,1]), $
                          1,npix) * m(2)+b(2)
            r.nLw(3,*) = reform(rd_sds(fileID,'nLw_520',start=[0,i],count=[npix,1]), $
                          1,npix) * m(3)+b(3)
            r.nLw(4,*) = reform(rd_sds(fileID,'nLw_565',start=[0,i],count=[npix,1]), $
                          1,npix) * m(4)+b(4)
            r.nLw(5,*) = reform(rd_sds(fileID,'La_670' ,start=[0,i],count=[npix,1]), $
                          1,npix) * m(5)+b(5)
            r.nLw(6,*) = reform(rd_sds(fileID,'eps_68' ,start=[0,i],count=[npix,1]), $
                          1,npix) * m(6)+b(6)
            r.nLw(7,*) = reform(rd_sds(fileID,'tau_865',start=[0,i],count=[npix,1]), $
                          1,npix) * m(7)+b(7)

        endelse

        ;
        ; Write the output record
        ;
        writeu,lun,r

      endif

    endfor

    free_lun,lun

    return
end    
