;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    OCHDFNAV generates navigation information for each pixel from an
;    hdf file.
;
; -----------------------------------------------------------------
; pro ochdfnav
;
; Generates navigation information interpolated to each pixel, using
; information extracted from a NASDA OCTS hdf file.
;
; Inputs:
;     file		- input filename or file ID from hdf_sd_start
;
; Optional Inputs:
;     sline=sline	- start line of output grid (Def=0)
;     nline=nline	- number of lines in output grid (Def=scans*10)
;     spix=spix		- start pixel of output grid (Def=0)
;     npix=npix		- number of pixels in output grid (Def=1594)
;
; Outputs:
;     lon (npix,nline)	- pixel longitudes in deg
;     lat (npix,nline)	- pixel latitudes in deg
;     solz(npix,nline)	- pixel solar zenith angles in deg
;     sola(npix,nline)	- pixel solar azimuth angles in deg
;     senz(npix,nline)	- pixel sensor zenith angles in deg
;     sena(npix,nline)	- pixel sensor azimuth angles in deg
;
; Notes:
;
;
; Written By:
;     Bryan Franz, SAIC GSC, NASA/SIMBIOS Project, April 1998.
;
; -----------------------------------------------------------------

pro ochdfnav,fileID,lon,lat,solz,sola,senz,sena, $
             sline=sline,spix=spix,nline=nline,npix=npix,latrange=latrange

    det   = rd_sds(fileID,'det') - 1  &  det = det(0)
    pxl   = rd_sds(fileID,'pxl') - 1   ; Control point pixels
    inlon = rd_sds(fileID,'lon')       ; Geodetic longitude
    inlat = rd_sds(fileID,'lat')       ; Geodetic latitude
    usun  = rd_sds(fileID,'sun_ref')   ; Geocentric ECR solar reference vector 
    pos   = rd_sds(fileID,'orb_vec')   ; Geocentric ECR S/C position

    ;
    ; Convert sun vector to unit vector
    ;
    for i=0,n_elements(usun)/3-1 do $
        usun(*,i) = usun(*,i)/sqrt(total(usun(*,i)^2))

    ;
    ; Define control-point grid
    ;
    nrow  = n_elements(inlon)/n_elements(pxl) ; Number of control-point lines
    ncol  = n_elements(pxl)                   ; Number of control-point pixs
    col   = long(pxl)                         ; Scan pixel numbers
    row   = lindgen(nrow)*10 + det            ; Line numbers

    ;
    ; Define output grid
    ;
    if (n_elements(spix)  eq 0) then spix  = 0
    if (n_elements(sline) eq 0) then sline = 0
    if (n_elements(npix)  eq 0) then npix  = 1594
    if (n_elements(nline) eq 0) then nline = nrow*10

    epix  = spix + npix - 1
    eline = sline + nline - 1

    nlon  = npix                              ; Number of output pixs
    nlat  = nline < 10*nrow                   ; Number of output lines
    ilon  = lindgen(nlon) + spix              ; Pix numbers of output grid
    ilat  = lindgen(nlat) + sline             ; Line numbers of output grid
    xout  = findgen(nlon)                     ; Sub-grid pixels
    yout  = findgen(nlat)                     ; Sub-grid lines
    indx  = anywhere(ilat,row)                ; index of output lines to
                                              ; control point lines
    ;
    ; Compute solar and sensor zenith and azimuth at input
    ; control points from information provided.
    ;
    print,'Computing view angles at input control points'

    insolz = inlon*0.0
    insola = inlon*0.0
    insenz = inlon*0.0
    insena = inlon*0.0

    for j=0,nrow-1 do for i=0,ncol-1 do begin
        lonlat2viewangle,inlon(i,j),inlat(i,j),pos(*,j),usun(*,j), $
                         solz1,sola1,senz1,sena1
        insolz(i,j) = solz1
        insola(i,j) = sola1
        insenz(i,j) = senz1
        insena(i,j) = sena1
    endfor

    ;
    ; We now have all the information at each control point, so we can
    ; interpolate to all pixels, all lines.
    ;
    lon   = fltarr(nlon,nlat)
    lat   = fltarr(nlon,nlat)
    solz  = fltarr(nlon,nlat)
    sola  = fltarr(nlon,nlat)
    senz  = fltarr(nlon,nlat)
    sena  = fltarr(nlon,nlat)

    ;
    ; Interpolate angles across each control-point line
    ;
    print,'Interpolating rows'
    xctl = float(col)
    for j=0,nrow-1 do begin
        jout = row(j) - sline
        if (row(j) ge sline and row(j) le eline) then begin
            lon (*,jout) = spline(xctl,reform(inlon (*,j)),ilon)
            lat (*,jout) = spline(xctl,reform(inlat (*,j)),ilon)
            solz(*,jout) = spline(xctl,reform(insolz(*,j)),ilon)
            sola(*,jout) = spline(xctl,reform(insola(*,j)),ilon)
            senz(*,jout) = spline(xctl,reform(insenz(*,j)),ilon)
            sena(*,jout) = spline(xctl,reform(insena(*,j)),ilon)
        endif
    endfor

    ;
    ; Fill missing lines by interpolating columns
    ;
    print,'Interpolating columns'
    yctl = yout(indx)
    for i=0,npix-1 do begin
        lat (i,*) = spline(yctl,reform(lat (i,indx)),yout)
        lon (i,*) = spline(yctl,reform(lon (i,indx)),yout)
        solz(i,*) = spline(yctl,reform(solz(i,indx)),yout)
        sola(i,*) = spline(yctl,reform(sola(i,indx)),yout)
        senz(i,*) = spline(yctl,reform(senz(i,indx)),yout)
        sena(i,*) = spline(yctl,reform(sena(i,indx)),yout)
        if (i mod 10 eq 0) then print,i
    endfor

    return
end


