;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    GETL0RECS returns the records in a level 0 file.
;
;----------------------------------------------------------------------
; Function getL0recs
;
; Returns L0 records from the specified file as a byte array.
;
; Synopsis:
;     recs = getL0recs(file [,zstart,zstop,type=type,indx=indx])
;
; Output:
;     recs - 21504 x n byte array of L0 data
;
; Required Inputs: 
;     file - string specifying the name of a valid L0 file
;
; Optional Inputs:
;     zstart - zulu time string specifying the start of the time-period
;              of intrest (default = beginning of data)
;     zstop  - zulu time string specifying the end of the time-period
;              of intrest (default = end of data)
;
; Keyword Inputs:
;     type - valid inputs are 'LAC','GAC','SCAL','LCAL','ICAL','TDIC'.
;            (default is GAC, unless it can be determined from the filename)
;     indx - the minor frame index for the file.  If not provided, it
;            will be generated by indx=getL0indx(file)
;
; Written By:
;     BA Franz, GSC, 8 March 1996
;
;----------------------------------------------------------------------
function getL0recs,file,zstart,zstop,type=type,indx=indx,hdr=hdr

    len = 21504L

    if (n_elements(hdr) eq 0) then $
        hdr = 512L

    ;
    ; Determine minor frame ID type to retrieve
    ;
    case n_elements(type) of
        ;
        ; If no type was specified, try to get it from the filename,
        ; otherwise, assume GAC.
        ;
        0 : begin
            case 1 of
                strpos(file,'GAC' ) ne -1 : ID = 15
                strpos(file,'HNSG') ne -1 : ID = 0
                strpos(file,'LAC' ) ne -1 : ID = 0
                else : ID = -1
            endcase
            end
        ;
        ; Assign ID from type provided
        ;
        else : begin
            case strupcase(type) of
                'GAC'  : ID = 15 
                'LAC'  : ID = 0
                'SCAL' : ID = 2
                'LCAL' : ID = 1
                'ICAL' : ID = 3
                'TDIC' : ID = 4
                'ALL'  : ID = -1
                else   : begin
                         message,'Invalid minor frame type.',/info
                         return,-1
                         endelse
            endcase
            endelse
    endcase

    ;
    ; Generate the mnf index if it was not provided
    ;
    if (n_elements(indx) eq 0) then begin
        print,'Generating indx for ',file,'.'
        indx = getL0indx(file)
    endif

    ;
    ; Determine start and end seconds.  Use entire file if no times
    ; were specified.
    ;
    if (ID ge 0) then begin
        s  = where(indx.type eq ID, ns)
        s1 = min(indx(s).time)
        s2 = max(indx(s).time)
    endif else begin
        s1 = min(indx.time)
        s2 = max(indx.time)
    endelse

    if (n_elements(zstart) ne 0) then $
        s1 = z2sec(zstart)
    if (n_elements(zstop)  ne 0) then $
        s2 = z2sec(zstop)

    ;
    ; Get record numbers that fall within the time-range
    ; and ID criteria.
    ;
    if (ID ge 0) then begin
        s = where( (indx.type eq ID) and $ 
                   (indx.time ge s1) and $
                   (indx.time le s2), ns )
    endif else begin
        s = where( (indx.time ge s1) and $
                   (indx.time lt s2), ns )
    endelse

    ;
    ; Read records from file.  Records are assumed contiguous.
    ;
    if (ns gt 0) then begin
        print,'Reading ',ns,' mnfs from ',file
        openr,/get_lun,lun,file
        recs = bytarr(len,ns)
        r    = assoc(lun, bytarr(len), hdr)
        for i=0L,ns-1 do begin
            if (i mod 100 eq 0) then print,i
            recs(*,i) = r(s(i))
        endfor
        free_lun,lun
        return,recs
    endif else begin
        message,'No minor frames in specified time-range',/info
        return, -1
    endelse

end
