;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    RDBACKORBIT reads seawifs backorbit data into a byte array.
;
;-----------------------------------------------------------------
; rdbackorbit - reads seawifs backorbit data into byte array
;
; Synopsis:
;     msgs = rdbackorbit( file, indx )
;
;     file	- input backorbit data filename
;     msgs	- SOH messages in S/C message format, stored as
;		  byte array 300 x n.
;     indx	- optional output list of messages source addresses
;
; Description:
;     This function reads back-orbit data in the original, 3502-byte
;     HDLC packet format.  Each HDLC record consists of an 8-byte 
;     header, followed by a series of variable-length spacecraft 
;     messages originating from the GIM or SCM memory subsystem.  Each 
;     of these messages are just wrappers around another spacecraft 
;     message originating from one of the SOH subsystems.  It is the
;     SOH subsystem messages that this function returns.
;
; Written By:
;     B. A. Franz, General Sciences Corp., 7/97
;
;
;-----------------------------------------------------------------

function rdbackorbit,file,indx

    limitlen = 300	; SAC length
    minlen   = 48       ; RXS length
    hdlclen  = 3502	; HDLC record length
    gimscm   = 4497     ; Desired destination address
    
    ;
    ; Create file association and read entire file into byte array
    ;
    openr,/get_lun,lun,file
    f = fstat(lun)
    r = assoc(lun,bytarr(hdlclen))
    nrecs = f.size/hdlclen
    
    ;
    ; Create fixed-length storage
    ;
    nmax = f.size/minlen + 1
    msg  = bytarr(limitlen, nmax)
    indx = intarr(nmax)

    ;
    ; Step through HDLC records
    ;
    i = 0L				; SOH message counter
    for ir=0,nrecs-1 do begin

        p    = 8L			; Start of GIM/SCM packet
        flen = fix(r(p:p+1,ir),0)	; Length of GIM/SCM packet

        ;
        ; Step through GIM/SCM messages, grap internal SOH messages
        ;
        repeat begin

            if (flen le 0 or p+flen gt hdlclen) then begin
                print,'Bad length field at record ',i,' (',flen,')'
                goto,break
            endif

            ;
            ; Get source and destination address
            ;
            dest = fix(r(p+22:p+23,ir),0)
            addr = fix(r(p+24:p+25,ir),0)

            ;
            ; If this is the kind of packet we want, store it.
            ; Note: I'm guessing here based on a few files.  I found
            ; cases where the destination was not 4497 and the
            ; message structure was unrecognizable, so I'm filtering
            ; those messages out here.
            ;
            if (dest eq gimscm) then begin

                indx(i) = addr			; Store address
                len = flen-10-4 < limitlen	; Length of SOH message

                ;
                ; Store SOH message. Reset the length field, as the
                ; original value included the trailing four bytes of 
                ; the GIM/SCM message.
                ;
                msg(0:len-1,i) = r(p+10:p+10+len-1,ir)
                msg(0:1,i)     = byte(len,0,2)
                i = i+1

            endif

            ;
            ; Update message pointer
            ;
            p = p+flen
            if (p lt hdlclen-3) then $
                flen = fix(r(p:p+1,ir),0) $
            else $
                flen = 0

        endrep until (flen le 0 or p ge hdlclen)

       break:

    endfor

    msg  = msg(*,0:i-1)
    indx = indx(0:i-1)

    return, msg

end
