;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    INT_PAR - simple integration of instantaneous PAR to get daily PAR
;
;------------------------------------------------------------------------
; FUNCTION INT_PAR
;
; IDL function to take a set of individual PAR values (within one day)
; and integrate over a 24-hour period to derive the daily average PAR.
;
; Programmer: Bryan A. Franz, SAIC-GSC
; Date Written: 8/2000
;
; Inputs:
;   year  - GMT year of each measurement (vector or scalar)
;   day   - GMT day of each measurement (vector or scalar)
;   msec  - GMT millisecs of day of each measurement (vector)
;   lon   - longitude of measurement (vector or scalar)
;   lat   - latitude of measurement (vector or scalar)
;   par   - instantaneous PAR values (vector)
;
; Keyword Inputs:
;   plotit - set this keyword to get a plot of the PAR values showing
;            the spline interpolation used for the integration.
;   cubic  - set this keyword to use cubic rather than linear 
;            interpolation to fill-out the integration range.
;
; Outputs:
;   paravg  - daily averaged PAR
;
;------------------------------------------------------------------------
function int_par,year,day,msec,lon,lat,par,plotit=plotit,cubic=cubic

    day1  = day + msec/86400.D0/1000.D0
    par1  = par

    ;
    ; Get sun rise/set times to bound integration; assume par=0
    ; when solar zenith is 90-deg.
    ;
    sunriseset,year(0),day(0),msec(0),lon(0),lat(0),msec_r, msec_s

    day_r = day(0) + msec_r/86400.D0/1000.D0
    day_s = day(0) + msec_s/86400.D0/1000.D0

    day1 = [day_r,day1,day_s]
    par1 = [0.0,par1,0.0]

    ;
    ; Interpolate par over integration range to 1-second intervals
    ;
    nsec = (msec_s - msec_r)/1000
    day2 = (findgen(nsec))/86400.D0 + day1(0)
    if (keyword_set(cubic)) then begin
        par2 = spline(day1,par1,day2) & par2 = par2 > 0.0
    endif else begin 
        par2 = lspline(day1,par1,day2) & par2 = par2 > 0.0
    endelse

    ;
    ; Integrate using 5-point Newton-Cotes
    ;
    avgpar = int_tabulated(day2,par2)

    ;
    ; Plot if requested
    ;
    if (keyword_set(plotit)) then begin
        plot,day2,par2,psym=3, $
            xtitle='Day of '+strtrim(string(year(0)),2), $
            ytitle='Instantaneous PAR', $
            title='PAR, Daily Average = '+strtrim(string(avgpar),2)
        oplot,day1,par1,psym=4
    endif

    return,avgpar
end

