;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    FSTR_READ performs a structured read using a user specified structure
;    definition on a binary file.
;
;--------------------------------------------------------------------
; Function FSTR_READ
;
; IDL function to perform a structured read using a user specified
; IDL structure definition on a Fortran binary file.  
;
; Written By: 
;     BA Franz, GSC, 3/96
;
; Inputs:
;     filename - name of file you wish to read
;     struct   - string containing the structure name
;     nrecs    - optional keyword, number of records to read
;     offset   - optional keyword, number of first record to read
;
; Output:
;     function returns an array of records, -1 if error.
;
;--------------------------------------------------------------------

function fstr_read,filename,struct,nrecs=nrecs,offset=offset

;
; Initialize return structure
;
ret_struct = -1 

;
; Check that file exists
;
file = findfile(filename)
if (n_elements(file) eq 0) then begin
    print,'File not found'
    return,ret_struct
end

;
; File exists, open it.
;
openr,lun,/get_lun,filename,/f77

;
; Get file attributes
;
f        = fstat(lun)
stat     = execute('rec = '+struct+'(1)')
len      = str_len(rec)
num_recs = long(f.size/(len+8))
;
; If file has no records then exit
;
if (num_recs le 0) then begin
    print,'File has no records.' 
    return,ret_struct
end

;
; Set offset.  If out of range then exit.
;
if (not keyword_set(offset)) then offset = 0
if (offset ge num_recs) then begin
    print,'No records beyond specified offset.'
    return,ret_struct
end

;
; Set number of records to read.
;
if (not keyword_set(nrecs)) then begin
    nrecs = num_recs - offset
    if (nrecs le 0) then begin
        print,'No records beyond specified offset.'
        return,ret_struct
    endif
endif else begin
    if ((nrecs+offset) gt num_recs) then begin
        nrecs = num_recs-offset
        print,'Only',nrecs,' records beyond specified offset.'
    endif
endelse

;
; Make structure to store all records
;
stat   = execute('rec = '+struct+'(nrecs)')
currec = rec(0)

;
; Read records sequentially
;
for i=0L,nrecs-1 do begin
    readu,lun,currec
    rec(i) = currec
endfor

;
; Clean-up
;
free_lun,lun

return,rec
end
