;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    STR_PRINT prints fields of an array of structures in column format.
;
; NAME:
;	print_struct
;
; PURPOSE:
;	Print the fields of an array of structures in nice column format,
;	with header line of field names.
;
; CALLING SEQUENCE:
;
;	print_struct, structure, tags_print, LUN_OUT=Lun
;
; INPUTS:
;	structure = array of structured variables
;
;	tags_print = string array specifying the names of fields to print.
;			Default is to print all fields which are not arrays.
; KEYWORDS:
;	LUN_OUT = Logical unit number for output to a file,
;		default is to print to standard output.
;
; EXTERNAL CALLS:
;	function N_struct
; PROCEDURE:
;	Check the types and lengths of fields to decide formats,
;	then loop and print requested fields.
; HISTORY:
;	Written, Frank Varosi NASA/GSFC 1991.
;	F.V.1993, fixed up the print formats.
;-
pro print_struct, structure, tags_print, LUN_OUT=Lun
	Nstruct = N_struct( structure, Ntag )

	if (Nstruct LE 0) then begin
		message,"expecting a structure",/INFO
		return
	   endif

	if (Nstruct EQ 1) then structure = [structure]

	tags = [tag_names( structure )]
	Npr = N_elements( tags_print )

	if (Npr LE 0) then begin

		for i=0,Ntag-1 do begin

		    if  (N_elements( structure(0).(i) ) LE 1) AND $
			(N_tags( structure(0).(i) ) LE 0) then begin

			  if N_elements( tagi ) LE 0 then tagi = [i] $
						     else tagi = [ tagi, i ]

		      endif
		  endfor

		pi = tagi
		tags_print = tags(pi)

	  endif else begin

		ptags = [strupcase( tags_print )]

		for i=0,Npr-1 do begin

		    w = where( tags EQ ptags(i), nf )

		    if (nf GT 0) then begin

			  if N_elements( tagi ) LE 0 then begin
				tagi = [w(0)]
				pi = [i]
			    endif else begin
				tagi = [ tagi, w(0) ]
				pi = [ pi, i ]
			     endelse

		      endif else message,"Tag <"+ptags(i)+"> not found",/CONTIN

		  endfor

		tags_print = strupcase( tags_print(pi) )
	   endelse

	Npr = N_elements( tagi )
	vtypes = intarr( Npr )
	sLens = intarr( Npr )
	formats = strarr( Npr )
	title = "  #"
	ncht = strlen( tags_print ) + 2
	minch = [ 0, 5, 8, 12, 11, 11, 11, 0 ]

	for i=0,Npr-1 do begin
		st = size( structure(0).(tagi(i)) )
		vtypes(i) = st(st(0)+1)
		CASE vtypes(i) OF
		1:	formats(i) = "(I" + strtrim( ncht(i)>5, 2 ) + ")"
		2:	formats(i) = "(I" + strtrim( ncht(i)>8, 2 ) + ")"
		3:	formats(i) = "(I" + strtrim( ncht(i)>12, 2 ) + ")"
		7: BEGIN
			sLens(i) = $
			 ( max( strlen( structure.(tagi(i)) ) ) + 2 ) > ncht(i)
			formats(i) = "(A" + strtrim( sLens(i), 2 ) + ")"
		     END
		else:	formats(i) = "(G" + strtrim( ncht(i)>11, 2 ) + ".3)"
		ENDCASE
		minch(7) = sLens(i)
		title = title + string( tags_print(i), $
		 FORM = "(A" + strtrim( ncht(i) > minch(vtypes(i)), 2 ) + ")" )
	  endfor

	if N_elements( Lun ) EQ 1 then printf,Lun,title  else print,title

	for n=0,Nstruct-1 do begin

		text = string( n, FORM="(I3)" )

		for i=0,Npr-1 do text = text + $
			string( structure(n).(tagi(i)), FORM=formats(i) )

		if N_elements( Lun ) EQ 1 then printf,Lun,text  else print,text
	  endfor
end
