;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    STR_READ_FIELD performs a structured read on an input structure
;    definition field.
;
;--------------------------------------------------------------------
; Function STR_READ
;
; IDL function to perform a structured read using a user specified
; IDL structure definition.  
;
; Writen by: BA Franz, ARC
;
; Inputs:
;     filename - name of file you wish to read
;     struct   - string containing the structure name
;     nrecs    - optional keyword, number of records to read
;     offset   - optional keyword, number of first record to read
;
; Output:
;     function returns an array of records, -1 if error.
;
;--------------------------------------------------------------------

function str_read_field,filename,struct,field,nrecs=nrecs,offset=offset,hdrlen=hdrlen

;
; Initialize return structure
;
ret_struct = -1 

;
; Check that file exists
;
file = findfile(filename)
if (n_elements(file) eq 0) then begin
    print,'File not found'
    return,ret_struct
end

;
; File exists, open it.
;
openr,lun,/get_lun,filename

;
; Get file attributes
;
f = fstat(lun)
stat = execute('rec = '+struct+'(1)')
len = str_len(rec)
num_recs = long(f.size/len)
;
; If file has no records then exit
;
if (num_recs le 0) then begin
    print,'File has no records.' 
    return,ret_struct
end

;
; Set offset.  If out of range then exit.
;
if (not keyword_set(offset)) then offset = 0L else offset=long(offset)
if (offset ge num_recs) then begin
    print,'No records beyond specified offset.'
    return,ret_struct
end

;
; Set number of records to read.
;
if (not keyword_set(nrecs)) then begin
    nrecs = num_recs - offset
    if (nrecs le 0) then begin
        print,'No records beyond specified offset.'
        return,ret_struct
    endif
endif else begin
    if ((nrecs+offset) gt num_recs) then begin
        nrecs = num_recs-offset
        print,'Only',nrecs,' records beyond specified offset.'
    endif
endelse
data = fltarr

;
; Compute byte offset to start of read
;
if (n_elements(hdrlen) eq 0) then hdrlen = 0L
bytoff = offset * len + hdrlen

;
; Form call to associated read
;    
assoc_call = 'r = assoc( lun,'+ struct + '('+ strtrim(string(nrecs),2) + '),' $
                              + strtrim(string(bytoff),2) + ')'

;
; Execute associated read.  If no error, then set return structure.
;
if (execute(assoc_call)) then ret_struct = r(0) 

;
; Clean-up
;
free_lun,lun
r = 0
f = 0

return,ret_struct
end
