;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    SPEC_DIR provides a complete file specification by appending a default
;    disk or directory.
;
; NAME:
;	SPEC_DIR
; PURPOSE:
;	Provide a complete file specification by appending a default disk
;	or directory if necessary.
;
; CALLING SEQUENCE:
;	File_spec = SPEC_DIR(filename,[extension])
; INPUT:
;	filename - character string giving partial specification of a file
;		name.  VMS examples include 'UIT$USER2:TEST.DAT', or 
;		'[.SUB]TEST'.   Unix examples include 
;		'/home/idl/lib', or '$IDL_HOME/pro'.   
;
; OPTIONAL INPUT:
;	exten - string giving a default file name extension to be used if
;		filename does not contain one.  Do not include the period.
;
; OUTPUT:
;	File_spec - Complete file specification using default disk or 
;		directory when necessary.  If the default directory
;		is UIT$USER1:[IDL] then, for the above VMS examples, the
;		output would be 'UIT$USER2:[IDL]TEST.DAT'
;		'UIT$USER2:[IDL.SUB]TEST'. 
;
; METHOD:
;	For Unix, SPEC_DIR will simply append the default directory obtained
;	from the CD command (if necessary).   Under VMS one must also 
;	determine if the disk and/or directory is already specified.    Under 
;	VMS, SPEC_DIR will also try to translate disk and directory logical 
;	names.
;
; REVISION HISTORY:
;	Written W. Landsman         STX         July, 1987
;	Revised for use on VAXes and on SUNs, W.  Landsman, STX   August 1991
;-
function spec_dir,filename,extension
 On_error,2                                     ;Return to user

 fdecomp,filename,disk,dir,name,ext             ;Decompose filename

 if (ext EQ '') and ( N_params() GT 1) then $   ;Use supplied default extension?
                    ext = extension

 if (dir EQ '') and (!VERSION.OS NE "vms") then begin

    cd,current=dir
    if name NE '' then dir = dir + '/'    ;Get current default directory

 endif else begin

   if ( disk EQ '' ) or ( dir EQ '' ) then begin
     cd,current=defdir                          ;Get current default directory
     fdecomp,defdir,curdisk,curdir
     if disk EQ '' then disk = curdisk else begin
       if !VERSION.OS EQ "vms" then begin
         logname = strmid(disk,0,strpos(disk,':'))
         test = trnlog(logname,fname)
         if test then begin
            if strmid(fname,strlen(fname)-1,1) EQ ']' then begin
               if strmid(fname,strlen(fname)-2,1) NE '.' then $
                                  return,spec_dir(fname+name,ext)
            endif else return,spec_dir(fname+':'+dir+name,ext)
         endif
       endif
     endelse

    if dir eq '' then dir = curdir else if !VERSION.OS EQ 'vms' then begin
        if strpos(dir,'.') eq 1 then dir = $
           strmid(curdir,0,strlen(curdir)-1) + strmid(dir,1,strlen(dir)-1)
     endif

 endif
 
 endelse

 if ext ne '' then ext = '.'+ext

 if !VERSION.OS ne "vms" then return,dir+name+ext else  $             ;Unix
                               return,strupcase(disk+dir+name+ext)   ;VMS

end
;DISCLAIMER:
;
;This software was written at the Cosmology Data Analysis Center in
;support of the Cosmic Background Explorer (COBE) Project under NASA
;contract number NAS5-30750.
;
;This software may be used, copied, modified or redistributed so long
;as it is not sold and this disclaimer is distributed along with the
;software.  If you modify the software please indicate your
;modifications in a prominent place in the source code.  
;
;All routines are provided "as is" without any express or implied
;warranties whatsoever.  All routines are distributed without guarantee
;of support.  If errors are found in this code it is requested that you
;contact us by sending email to the address below to report the errors
;but we make no claims regarding timely fixes.  This software has been 
;used for analysis of COBE data but has not been validated and has not 
;been used to create validated data sets of any type.
;
;Please send bug reports to CGIS@COBECL.DNET.NASA.GOV.


