;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    UNMASK converts a scalar or array of bit masks into a binary array.
;
;------------------------------------------------------------
; Function UNMASK
;
; Written By:  BA Franz, Applied Research Corp, 1991
;
; IDL function to convert a scalar or array of bit masks into
; an array of 0's and 1's.
;
; Inputs:
;   inmask   - scalar or array of mask elements (Byte, Word, Long)
;
; Output:
;   maskarr  - bytarr(nbits,nelements) where nbits is the number of
;              bits in mask and nelements is the number of mask 
;              elements.  A value of 1 means the bit was set.  The
;              maskarr(0,0) is the zeroth bit of the first mask.
;
; Calling Sequence:
;   maskarr = UNMASK(inmask)
;------------------------------------------------------------
function unmask,inmask

;
; Get integer type to determine number of bits in mask
;
s   = size(inmask) 
i   = s(0)
typ = s(i+1)
case typ of
    1 : nbits =  8
    2 : nbits = 16
    3 : nbits = 32
    else : begin
        message,'Invalid data type.  Must be BYTE, WORD, or LONG.',/info
        return,-1
    endelse
endcase

;
; Determine number of mask elements and set-up output array
;
npts  = n_elements(inmask)        ; Number of mask elements
tmask = inmask                    ; Temporary mask
mask_array = bytarr(nbits,npts)   ; Output array
shft_array = intarr(npts) - 1     ; Array of -1's for shift control

;
; Repeat for nbits.
; Check to see if last bit is set.  If it is, then set the output
; array element to 1.  Shift the array of mask elements right 1 bit.
;
for i = 0,nbits-1 do begin
    select = where(abs(tmask mod 2) eq 1) 
    if (select(0) ne -1) then mask_array(i,select) = 1
    tmask = ishft(tmask,shft_array)
endfor

return,mask_array
end
