;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    ZPARCHECK verifies data type of parameters in a procedure call.
;
; PURPOSE:
;	Routine to check user parameters to a procedure and prints
;       a message/returns if the parameters don't match
;
; CALLING SEQUENCE:
;	zparcheck,progname,parameter,parnum,types,dimens,message
;
; ARGUMENTS:
;	progname  I str name of     -  calling procedure
;	parameter I any-variable    -  parameter passed to the routine
;	parnum    I int             -  parameter number for error
;                                       message output
;	types     I int vec or sclr -  valid data types for parameter
;		 	1 - byte        2 - integer  3 - int*4
;			4 - real*4      5 - real*8   6 - complex
;			7 - string      8 - structure
;	ndimens   I int sclr or vec -  number of dimensions
;					of allowed dimensions.
;	message -[I] str            -  string message to be printed
;				      	if an error is found).
;
; WARNINGS:
;	Do not misuse government software.
; EXAMPLES:
;  ;;;In this example, the routine simply returns since
;  ;;;the number '10' is an integer scaler
;  UIDL> zparcheck,'DOGANDPONY',10,7,2,0,'Error you dummy'
;  UIDL>
;  ;;;Here we get an error since 10 is not a long scaler
;  UIDL> zparcheck,'DOGANDPONY',10,7,3,0,'Error you dummy'
;  Parameter 7 (Error you dummy)  of routine DOGANDPONY is an invalid
;                                                             data type
;  Valid dimensions are: scaler
;  Valid types are:  longword
;  UIDL>
;#
;
;       
; COMMON BLOCKS:
;        None
;
; PROCEDURES (AND OTHER PROGRAMMING NOTES):
;	If there is an error in the parameters then a
;	a RETALL issued
;
; PERTINENT ALGORITHMS, LIBRARY CALLS, ETC.:
;	None
;
; MODIFICATION HISTORY:
;
;	version 1  D. Lindler  Dec. 86
;	documentation updated.  M. Greason, May 1990.
;.TITLE
;Routine ZPARCHECK
;-
;
PRO ZPARCHECK,PROGNAME,PARAMETER,PARNUM,TYPES,DIMENS,MESSAGE
; convert types and ndimens to vectors if scalars supplied
;
vtypes=intarr(n_elements(types)) + types
vndims=intarr(n_elements(dimens)) + dimens
;
; get type and size of parameter
;
s=size(parameter)
ndim=s(0)
type=s(ndim+1)
;
; check if parameter defined.
;
if type eq 0 then begin
	err=' is undefined.'
	goto,abort
endif
;
; check for valid dimensions
;
valid=where(ndim eq vndims,nvalid)
if nvalid lt 1 then begin
	err='has wrong number of dimensions'
	goto,abort
endif
;
; check for valid type
;
valid=where(type eq vtypes,ngood)
if ngood lt 1 then begin
	err='is an invalid data type'
	goto,abort
endif
;
return
;
; bad parameter
;
abort:
mess=' '
if n_params(0) lt 6 then message=''
if message ne '' then mess=' ('+message+') '
print,string(7b) + 'Parameter '+strtrim(parnum,2)+mess,$
	' of routine ',strupcase(progname)+' ',err
sdim=' '
for i=0,n_elements(vndims)-1 do begin
	if vndims(i) eq 0 then sdim=sdim+'scalar' $
			  else sdim=sdim+string(vndims(i),'(i3)')
end
print,'Valid dimensions are:'+sdim
;
stype=' '
for i=0,n_elements(vtypes)-1 do begin
	case vtypes(i) of
		1: stype=stype+' byte'
		2: stype=stype+' integer'
		3: stype=stype+' longword'
		4: stype=stype+' real*4'
		5: stype=stype+' real*8'
		6: stype=stype+' complex'
		7: stype=stype+' string'
                8: stype=stype+' structure'
	endcase
endfor
print,'Valid types are:'+stype
;if !debug then stop
retall  ; zparcheck
end
;DISCLAIMER:
;
;This software was written at the Cosmology Data Analysis Center in
;support of the Cosmic Background Explorer (COBE) Project under NASA
;contract number NAS5-30750.
;
;This software may be used, copied, modified or redistributed so long
;as it is not sold and this disclaimer is distributed along with the
;software.  If you modify the software please indicate your
;modifications in a prominent place in the source code.  
;
;All routines are provided "as is" without any express or implied
;warranties whatsoever.  All routines are distributed without guarantee
;of support.  If errors are found in this code it is requested that you
;contact us by sending email to the address below to report the errors
;but we make no claims regarding timely fixes.  This software has been 
;used for analysis of COBE data but has not been validated and has not 
;been used to create validated data sets of any type.
;
;Please send bug reports to CGIS@COBECL.DNET.NASA.GOV.


