; Define a data structure appropriate to VIIRS lookup table
; Original definitions in $ADL_HOME/xml/VIIRS/VIIRS_S*.xml
; Gwyn Fireman, November 2012

function geo_param_struct
; adapted from include/ViirsGeo/VcstGeoParameters.h
  MAX_BAND_NUMBER_PLUS_ONE = 37
  MAX_POLY_DEGREE_PLUS_ONE = 5
  MAX_THERMISTOR_ID_LEN = 40
  NUM_AGG_ZONES = 5
  NUM_ELECTRONICS_SIDE = 2
  NUM_THERMISTORS = 26
  DIM2 = 2
  DIM3 = 3
  DIM6 = 6
  DIM10 = 10
  DIM32 = 32
  DIM64 = 64
  PAD_DIM2 = 2
  PAD_DIM4 = 4
  PAD_DIM6 = 6
  matrix = dblarr(DIM3,DIM3)
  struct = {revision: bytarr(DIM10), $
            pad1: bytarr(PAD_DIM6), $
            band_number: 0L, $
            pad2: bytarr(PAD_DIM4), $
            latch_to_center: 0.0D, $
            t_reset: 0.0D, $
            N_samp: uintarr(MAX_BAND_NUMBER_PLUS_ONE), $
            pad3: bytarr(PAD_DIM6), $
            focal_length: dblarr(MAX_BAND_NUMBER_PLUS_ONE), $
            det_space_track: dblarr(MAX_BAND_NUMBER_PLUS_ONE), $
            det_space_scan: dblarr(MAX_BAND_NUMBER_PLUS_ONE), $
            DNB_space_track: dblarr(DIM32), $
            DNB_space_scan: dblarr(DIM32), $
            det_position: dblarr(DIM2,MAX_BAND_NUMBER_PLUS_ONE), $
            band_position: dblarr(MAX_BAND_NUMBER_PLUS_ONE), $
            earth_view_delay: 0.0D, $
            detector_sampling_rate: 0.0D, $
            scan_length: 0.0D, $
            agg_zone_bounds: lonarr(NUM_AGG_ZONES), $
            DNB_aggregation: lonarr(DIM2,DIM32), $
            DNB_ag_zone_bounds: lonarr(DIM3,DIM64), $
            pad4: bytarr(PAD_DIM4), $
            scan_ang_coef_tel: 0.0D, $
            scan_ang_coef_mirr: 0.0D, $
            scan_ang_offsets: dblarr(DIM2), $
            enc_scale: 0.0D, $
            mirr_abs_limit: dblarr(DIM2), $
            mirr_del_limit: 0.0D, $
            tel_abs_limit: dblarr(DIM2), $
            tel_del_limit: 0.0D, $
            sample_impulse_mirr: 0L, $
            sample_impulse_tel: 0L, $
            A_bit_adj: lonarr(DIM2), $
            B_HAM_adj: lonarr(DIM2), $
            t_encoder: 0.0D, $
            mirr_side1_range: dblarr(DIM2), $
            alpha: 0.0D, $
            beta: 0.0D, $
            gammaa: 0.0D, $
            T_inst2sc: matrix, $
            T_mirr2inst: matrix, $
            T_aft2inst: matrix, $
            T_inst2SD: matrix, $
            T_tel2inst: matrix, $
            num_thermistor: 0L, $
            thermistor_id: bytarr(NUM_THERMISTORS,MAX_THERMISTOR_ID_LEN), $
            pad5: bytarr(PAD_DIM4), $
            thermistor_coeffs: dblarr(DIM6,NUM_THERMISTORS), $
            Mag: matrix, $
            basis_in: matrix, $
            basis_out: matrix, $
            poly_coef_mirr: dblarr(MAX_POLY_DEGREE_PLUS_ONE), $
            poly_coef_tel: dblarr(MAX_POLY_DEGREE_PLUS_ONE,NUM_ELECTRONICS_SIDE), $
            tel_ref: 0.0D, $
            min_cos_view: 0.0D, $
            band_type: 0L, $
            num_detectors: 0L, $
            poly_degree: 0L, $
            N_frame: 0U, $
            pad6: bytarr(PAD_DIM2) }
  return, struct
end

function define_viirs_lut, lut_type

; VIIRS band order, for reference
  bands = ['I01','I02','I03','I04','I05', $
           'M01','M02','M03','M04','M05', $
           'M06','M07','M08','M09','M10', $
           'M11','M12','M13','M14','M15', $
           'M16','DNB']

; common dimension constants
  DIM_NUM4 = 4
  MAX_NUM_DETECTOR = 32
  NUM_ELECTRONICS_SIDE = 2
  NUM_M_I_GAIN_STATES = 2
  NUM_MIRROR_SIDES = 2
  NUM_TDET_LEVEL = 5
  NUM_TELE_LEVEL = 5
  NUM_VIIRS_BAND = 22
  SDSM_DETECTOR = 8

; define data structure according to LUT type
  case lut_type of

;------ radiance calibration ------

    "VIIRS-SDR-COEFF-A-LUT": $  ; Photoelectrons/Vi
       lut = dblarr( $
             DIM_NUM4, $
             NUM_TDET_LEVEL, $
             NUM_MIRROR_SIDES, $
             NUM_M_I_GAIN_STATES, $
             MAX_NUM_DETECTOR, $
             NUM_VIIRS_BAND, $
             NUM_ELECTRONICS_SIDE )

    "VIIRS-SDR-COEFF-B-LUT": $  ; V/counti
       lut = dblarr( $
             DIM_NUM4, $
             NUM_TELE_LEVEL, $
             NUM_MIRROR_SIDES, $
             MAX_NUM_DETECTOR, $
             NUM_VIIRS_BAND, $
             NUM_ELECTRONICS_SIDE )

    "VIIRS-SDR-DELTA-C-LUT": begin
      MAX_NUM_C_COEF = 4
      NUM_FOCAL_PLANES = 3
      telecThermistorNum = 25
      DeltaC = dblarr($         ; W/(m2um sr cntsi)
               telecThermistorNum, $
               MAX_NUM_C_COEF, $
               NUM_MIRROR_SIDES, $
               NUM_M_I_GAIN_STATES, $
               MAX_NUM_DETECTOR, $
               NUM_VIIRS_BAND, $
               NUM_ELECTRONICS_SIDE )
      Tele = dblarr($           ; Kelvin
             NUM_TELE_LEVEL, $
             NUM_ELECTRONICS_SIDE )
      Tdet = dblarr($           ; Kelvin
             NUM_TDET_LEVEL, $
             NUM_FOCAL_PLANES, $
             NUM_ELECTRONICS_SIDE )
      lut = {DeltaC:DeltaC, Tele:Tele, Tdet:Tdet}
    end

    "VIIRS-SDR-F-LUT": begin    ; unitless
      MAX_NUM_GAIN = 3
      lut = dblarr( $
            NUM_MIRROR_SIDES, $
            MAX_NUM_GAIN, $
            MAX_NUM_DETECTOR, $
            NUM_VIIRS_BAND )
    end

    "VIIRS-SDR-GAIN-LUT": $     ; W/(m^2 um sr)/photoelectron
       lut = dblarr(NUM_VIIRS_BAND)

    "VIIRS-SDR-RSR-LUT": begin
      reflectiveBandNum_x2y2 = 14
      X2_Y2_DIM = 14000
      x2 = dblarr(X2_Y2_DIM)                      ; nanometers
      y2 = dblarr(X2_Y2_DIM)                      ; unitless
      numX2Y2Pts = lonarr(reflectiveBandNum_x2y2) ; unitless
      lut = {x2:x2, y2:y2, numX2Y2Pts:numX2Y2Pts}
    end

    "VIIRS-SDR-DG-ANOMALY-DN-LIMITS-LUT": begin
      NUM_REFL_750M_DG_BANDS = 6
      NUM_DETECTORS_750M = 16
      PRO_VIIRS_MIN_MAX_DIM = 2
      lut = fltarr(PRO_VIIRS_MIN_MAX_DIM, $ ;unitless
                   NUM_DETECTORS_750M, $
                   NUM_REFL_750M_DG_BANDS)
    end

    "VIIRS-SOLAR-DIFF-SDSM-BRDF-LUT": begin
      SDSM_BRDF_EL = 46
      SDSM_BRDF_AZ = 12
      lut = { elevation:fltarr(SDSM_BRDF_EL,SDSM_DETECTOR), $             ; degrees
              azimuth:  fltarr(SDSM_BRDF_AZ,SDSM_DETECTOR), $             ; degrees
              factors:  fltarr(SDSM_BRDF_EL,SDSM_BRDF_AZ,SDSM_DETECTOR) } ; sr^-1
    end

    "VIIRS-SOLAR-DIFF-SDSM-TIME-LUT": begin
      SDSM_OFFSET = 5
      lut = fltarr(SDSM_OFFSET) ; scan fraction
    end

    "VIIRS-SOLAR-DIFF-SDSM-TRANS-SCREEN-LUT": begin
      SDSM_EL = 25
      SDSM_AZ = 19
      lut = { elevation:fltarr(SDSM_EL,SDSM_DETECTOR), $        ; degrees
              azimuth:  fltarr(SDSM_AZ,SDSM_DETECTOR), $        ; degrees
              factors:  fltarr(SDSM_EL,SDSM_AZ,SDSM_DETECTOR) } ; unitless
    end

    "VCST SDSM BRDF": begin
      SDSM_BRDF_DEC = 60        ; info from Jeff McIntire
      SDSM_BRDF_AZI = 80
      azimuth = dblarr(SDSM_BRDF_AZI,SDSM_DETECTOR)           ; degrees
      declination = dblarr(SDSM_BRDF_DEC,SDSM_DETECTOR)       ; degrees
      tau = dblarr(SDSM_BRDF_DEC,SDSM_BRDF_AZI,SDSM_DETECTOR) ; sr^-1
      lut = {azimuth:azimuth, declination:declination, tau:tau}
    end

    "VIIRS-RSBAUTOCAL-BRDF-SCREEN-TRANSMISSION-PRODUCT-RTA-VIEW-LUT": begin
      BRDF_TAU_RTA_AZ = 80      ; info from Jeff McIntire
      BRDF_TAU_RTA_DEC = 60
      NUM_WAVELENGTH_SAMPLES = 14
      lut = { azimuth:     dblarr(BRDF_TAU_RTA_AZ), $        ; unitless
              declination: dblarr(BRDF_TAU_RTA_DEC), $       ; unitless
              lambda:      dblarr(NUM_WAVELENGTH_SAMPLES), $ ; micrometers
              brdf_tau_RTA:dblarr(NUM_WAVELENGTH_SAMPLES, $  ; unitless
                                  BRDF_TAU_RTA_DEC, $
                                  BRDF_TAU_RTA_AZ) }
    end

;------ geolocation ------

    "VIIRS-SDR-GEO-IMG-PARAM-LUT": lut = geo_param_struct()
    "VIIRS-SDR-GEO-MOD-PARAM-LUT": lut = geo_param_struct()
    "VIIRS-SDR-GEO-DNB-PARAM-LUT": lut = geo_param_struct()

    "Terrain-Eco-ANC-Tile": begin
; definitions from include/ViirsGeo/VcstTerrainCorrect.h
      TEBOX_ROWS = 1664
      TEBOX_COLS = 1664
      BOX_FILE_STR = 32
      lut = { mslhgt: intarr(TEBOX_COLS,TEBOX_ROWS), $
              maxhgt: intarr(TEBOX_COLS,TEBOX_ROWS), $
              minhgt: intarr(TEBOX_COLS,TEBOX_ROWS), $
              bathy:  intarr(TEBOX_COLS,TEBOX_ROWS), $
              sfcgp:  intarr(TEBOX_COLS,TEBOX_ROWS), $
              egSep:  bytarr(TEBOX_COLS,TEBOX_ROWS), $
              qst:    bytarr(TEBOX_COLS,TEBOX_ROWS), $
              upIET_mslhgt: 0LL, $
              upIET_mmhgt:  0LL, $
              upIET_bathy:  0LL, $
              upIET_sfcgp:  0LL, $
              upIET_egs:    0LL, $
              upIET_qst:    0LL, $
              upIET_resolv: 0LL, $
              updated_IET:  0LL, $
              boxfile: string("X",format="(A"+string(BOX_FILE_STR)+")"), $
              boxnum:       0L, $
              boxrow:       0L, $
              boxcol:       0L, $
              ulrow_offset: 0L, $
              ulcol_offset: 0L, $
              num_nofill:   0L, $
              nOrS: "X" }
    end

;------ others ------

    else: begin
      print, "Don't know structure of LUT type: ",lut_type
      print, "Find definition in $ADL_HOME/xml/VIIRS."
      return, -1
    end
  endcase

  return, lut
end
