; IVOBC_Temps.pro
; 
; Function to convert temperature data numbers from a VIIRS IVOBC file
; into physical units.
;
; Parameters:
; obcfile    Path to a VIIRS IVOBC file
; lutfile    Optional path to coefficient file;
;            defaults to VIIRS-SDR-TELE-COEFFS-LUT in OPER area.
; 
; Written Sept. 2012 by G. Fireman
; 

;----------------------------------------------------------------------

function read_telecoeffs, lutfile, quiet=quiet

; find and test VIIRS-SDR-TELE-COEFFS-LUT
if n_elements(lutfile) eq 0 then begin
  lutfile = file_search('$OCSSWROOT/run/var/viirsn/cal/OPER/*VIIRS-SDR-TELE-COEFFS-LUT*',count=nfiles)
  if nfiles eq 0 then begin
    print, 'No VIIRS-SDR-TELE-COEFFS-LUT specified or found in $OCSSWROOT/run/var/viirsn/cal/OPER; returning.'
    return, 0
  endif
  lutfile = lutfile[0]
  if not keyword_set(quiet) then print, 'No VIIRS-SDR-TELE-COEFFS-LUT specified; using ',lutfile
endif
if not file_test(/read,lutfile) then begin
  print, lutfile, ' not readable; returning.'
  return, 0
endif

; Define order of thermistor coefficients
; taken from ADL/SDR/VIIRS/Cal/src/Preprocess.cpp
; Descriptions taken from Operational Algorithm Description VIIRS SDR
; (Document Number D41868 Revision A)
thermistors = [ $
              strarr(6),$               ; placeholders for BB thermistors (BB_TMPS)
              'mf_tel_blkhd_py',$       ; Mainframe Telescope Bulkhead Therm 33, RT16 (Node 1022 in Radiometry Model)
              'mf_scan_cavity_nxp',$    ; Mainframe Scan Cavity NX PZ Thermistor 46
              'mf_scan_cavity_baf_pz',$ ; Mainframe Scan Cavity Baffle NZ Precision Thermistor 9
              'mf_scan_cavity_baf_nz',$ ; Mainframe Scan Cavity Baffle NZ Precision Thermistor 10
              'ap_lw_cca',$             ; Long wave IR CCA Information stored in the Engineering packet.
              'ap_vn_cca',$             ; Visible/Near IR CCA information stored in the Engineering packet
              'dp_dnb_cca',$            ; Digital Preprocessor DNB Circuit Card Assembly Thermistor 60 
              'ap_sm_cca',$             ; Short/Med wave IR CCA information stored in the Engineering packet.
              'mf_ao_blkhd_px_nz',$     ; MF_AO_BLKHD_PX_NZ Therm 44
              'mf_ao_blkhd_nx_pz',$     ; MF_AO_BLKHD_NX_PZ Therm 45
              'mf_stopassy_baff_nz',$   ; On Baffle NZ of Apert Stop Assy between HAM and FM2 Therm 14
              'mf_fold_mir_bkhd_ct',$   ; MF_FOLD MIRROR BLKHD Center Therm 6
              'mf_ham_blkhd',$          ; MF HAM Bulkhead Therm 43
              'ft_lw_cfpa_hi_rsl',$     ; focal plane Temperature Controller Long Wave IR CFPA high resolution temperature
              'ft_lw_cfpa_lo_rsl',$     ; focal plane Temperature Controller Long Wave IR CFPA wide range temperature
              'ft_sm_cfpa_hi_rsl',$     ; focal plane Temperature Controller Short Midwave IR CFPA high resolution temperature
              'ft_sm_cfpa_lo_rsl',$     ; focal plane Temperature Controller Short Midwave IR CFPA wide range temperature
              'ft_vis_nir_fpa',$        ; focal plane Temperature Controller Visual/Near IR fpa
              'ham_tmp1',$              ; Half Angle Mirror T1 Precision Thermistor 07, Radiatively Coupled Temperature 
              'ham_tmp2' ]              ; Half Angle Mirror T2 Precision Thermistor 08, Radiatively Coupled Temperature 

; define coefficient array and read from file
NUM_MAX_THERMISTORS = n_elements(thermistors)
NUM_TEMP_COEFFS = 6
teleCoeffs = fltarr(NUM_TEMP_COEFFS,NUM_MAX_THERMISTORS)
openr, lun, /get_lun, lutfile, /swap_if_little_endian
readu, lun, teleCoeffs
free_lun, lun

; define structure and return
; ignore BB_TMPS
return, {thermistors:thermistors[6:*],teleCoeffs:teleCoeffs[*,6:*]}
end

;----------------------------------------------------------------------

function read_ivobc, file, dataset
group = '/All_Data/VIIRS-OBC-IP_All'

; open
file_id = h5f_open(file)
grup_id = h5g_open(file_id, group)
data_id = h5d_open(grup_id, dataset)

; read
data = h5d_read(data_id)

; close
h5d_close, data_id
h5g_close, grup_id
h5f_close, file_id

return, data
end

;----------------------------------------------------------------------

function convert_temp_poly, DN, coeffs
K = coeffs[0] + DN*(coeffs[1] + DN*(coeffs[2] + DN*(coeffs[3] + DN*(coeffs[4] + DN*(coeffs[5])))))
return, K
end

;----------------------------------------------------------------------

function ivobc_temps, obcfile, lutfile, quiet = quiet

; read temperature coefficients from LUT
coeffs = read_telecoeffs(lutfile, quiet = quiet)
if size(coeffs,/tname) ne 'STRUCT' then begin
  print,'Error reading coefficients; returning'
  return, 0
endif
ntemps = n_elements(coeffs.thermistors)

; read scan start time from IVOBC file
EV_START_TIME = read_ivobc(obcfile,'EV_START_TIME')
cmd = 'EV_START_TIME:EV_START_TIME'

; read thermistor data from IVOBC file
for i = 0,ntemps-1 do begin
  dataset = coeffs.thermistors[i]
  data = read_ivobc(obcfile,dataset)                     ; read dn  
  data = convert_temp_poly(data, coeffs.teleCoeffs[*,i]) ; convert to physical units
  result = execute(dataset+' = data')                    ; make a new variable
  cmd += ','+dataset+':'+dataset                         ; add to output structure
endfor

; create output structure
result = execute('struct = {'+cmd+'}')

return, struct
end

;----------------------------------------------------------------------
