;+
; NAME:
;       plot_2D_hist
; PURPOSE:
;       Create and plot a density image (2D histogram) from arrays of (x,y) 
;		points using imagxy and tvim
;		Basically, a wrapper for imagxy and tvim
;
; CALLING EXAMPLES:
;       plot_2D_hist, x, y, NPIX=64, XRAN=[0,20], YRAN=[-5,5] 
;       plot_2D_hist, x, y, xran=[0.01,100],yran=[0.01,100], /ylog, /xlog
; INPUTS:
;       X = array (any dimension) of x values.
;       Y = array of y values;
; KEYWORDS:
;       XRAN and YRAN : specify the x,y range to be mapped into the image.
;                       Common imagxy, xminc,xmaxc, yminc,ymaxc
;                       can also specify the x,y range if keywords are not used,
;                       otherwise the defaults = min-max ranges of x and y.
;       NPIXELS = 1 or 2 element integer array specifying size of result,
;                       (single value means square image), default = [64,64].
;		YLOG and YLOG : select log scaling for output plot
;		RELATIVE : scales output to max bin.
;		(XY)TITLE : sets titles for plot
;		OUTFILE : the output filename (default='density_plot.gif')
;		PNG : produces a png file instead of a gif
;		CTBL : the colortable of choice (default=39, i.e. rainbow+white) 
; OUTPUTS:
;       Result of procedure is a gif image of the density of (x,y) points 
; REQUIRES:
;	imagxy and tvim (which requires color_key, step_ct, prank, etc...)
;	from the Earth Space Research Group (ESRG), UCSB
;	There is a phantom reference to ftp://crseo.ucsb.edu/pub/idl/
;	I found the ESRG library at:
;	http://www.astro.washington.edu/deutsch/idl/htmlhelp/slibrary28.html
;
;	Since the code uses WRITE_GIF, it also requires that your version of IDL
;	still supports gif :) ... or you could try the PNG option...
; HISTORY:
;      written by Sean Bailey, Futuretech Corporation April 9, 2002
;-

PRO plot_2D_hist, indep, dep, NPIXELS=npix, XRAN=xran, YRAN=yran, $
			XLOG=xlog, YLOG=ylog, relative=relative, PNG=png, CTBL=ctbl, $
			title=title, xtitle=xtitle, ytitle=ytitle, outfile=outfile
	
	IF NOT KEYWORD_SET(outfile) THEN BEGIN
		IF NOT KEYWORD_SET(PNG) THEN outfile = 'density_plot.gif' $
		ELSE outfile = 'density_plot.png'
	ENDIF
	
	IF KEYWORD_SET(XLOG) THEN BEGIN 
		x = alog10(indep) 
		xranhist = alog10(xran)
	ENDIF ELSE BEGIN
		x = indep
		xranhist = xran
	ENDELSE
	IF KEYWORD_SET(YLOG) THEN BEGIN
		y = alog10(dep) 
		yranhist = alog10(yran)
	ENDIF ELSE BEGIN
		y = dep
		yranhist = yran
	ENDELSE

	idx = where(finite(x) eq 1 and finite(y) eq 1)
	IF idx[0] GE 0 THEN BEGIN
		x = x(idx)
		y = y(idx)
	ENDIF
	IF N_elements( npix ) LE 0 THEN npix = [64,64]
	IF N_elements( npix ) EQ 1 THEN npix = replicate( npix(0), 2 )
	npix = Long( npix ) < Long( 2.^15-1 )
	barwidth=0.5
     
	hist = imagxy(x,y,npix=npix,xran=xranhist,yran=yranhist)
	; it will work with hist_2d, but it's more work... :0
	;hist = hist_2D(x,y,bin1=0.05,bin2=0.05,max1=xranhist[1],max2=yranhist[1],min1=xranhist[0],min2=yranhist[0])
	hist = 100.*(hist/total(hist,/nan))
	
	IF KEYWORD_SET(relative) THEN BEGIN
		maxhist = max(hist)
		hist = float(hist_equal(hist))/2.55
		stitle = strcompress(('Relative Density [max = '+string(maxhist)+'%]'))
	ENDIF ELSE stitle = 'Density (%)'
	
	IF NOT KEYWORD_SET(CTBL) then ctbl = 39
	loadct,ctbl,ncolors=256
	tvlct,255,255,255,0 & tvlct,0,0,0,!d.n_colors-1
	rgb = [255,255,255]
	rgb = [0,0,0]

	tvim, hist, /scale, nodata=0,rgb_nodata=rgb,interp=2, xrange=xran, yrange=yran, $
		xlog=xlog, ylog=ylog, xtitle=xtitle, ytitle=ytitle, title=title, $
		barwidth=barwidth, stitle=stitle, /one2one
	order=0
	IF KEYWORD_SET(PNG) THEN order = 2
	output = tvrd(order=order)
	tvlct,r,g,b,/get
	IF NOT KEYWORD_SET(PNG) THEN write_gif,outfile,output,r,g,b $
	ELSE write_png,outfile,output,r,g,b
END
