;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    REGRID rebins to new, evenly spaced grid.
;
;=============================================================
; Name:
;	regrid - simple rebinning to new, evenly spaced grid
;
; Synopsis:
;	map2 = regrid(map1, nx2, ny2[, /avg])
;
; Inputs:
;	map1 - 2-D rasterized map
;	nx2  - desired dimension of first indice
;	ny2  - desired dimension of second indice
;	
; Keyword Inputs:
;	avg  - if non-zero, bins will be averaged when multiple
;              entries from map1 fall into the same map2 bins.
;              If not specified, bins will be assigned the value
;              of the last element from map1 which maps to each
;              element of map2, which is much faster.
;
; Outputs:
;	map2 - nx2 x ny2 array of same type as map1
;
; Notes:
;	Use of the "avg" will result in a much less efficient
;	conversion, since without averaging, no looping is
;	neccessary.
;
; Written By:
; 	BA Franz, General Sciences Corp., 5 February 96
;
;=============================================================

function regrid,map1,nx2,ny2,avg=avg

    if (n_params(0) ne 3) then begin
        print,'Usage:  map2 = regrid(map1, nx2, ny2[, /avg]'
        return,-1
    endif

    s     = size(map1)			
    if (s(0) ne 2) then begin
        print,'regrid : invalid input map',/info
        return,-1
    endif
    nx1   = s(1)		
    ny1   = s(2)
    x1    = findgen(nx1)
    y1    = findgen(ny1)

    ;
    ; Create arrays of the x and y coordinates of map1 elements in
    ; the map1 grid.
    ;
    xmap1 = x1 # (1 + fltarr(ny1))
    ymap1 = (1 + fltarr(nx1)) # y1

    ;
    ; Create arrays of the x and y coordinates of map1 elements in
    ; the map2 grid.
    ;
    xmap2 = xmap1 * nx2/nx1
    ymap2 = ymap1 * nx2/nx1
    
    ;
    ; Create map2
    ;
    map2 = map1(0) * bytarr( nx2, ny2 )

    if ( keyword_set(avg) ) then begin

        sum2 = fltarr( nx2, ny2 )
        for j=0,ny1-1 do for i=0,nx1-1 do begin
            i2 = xmap2(i,j)
            j2 = ymap2(i,j)
            map2(i2,j2) = map2(x,y) + map1(i,j)
            sum2(i2,j2) = sum2(i2,j2) + 1.
        endfor

        s = where(sum2 gt 1.0)
        if (s(0) ne -1) then $
            map2(s) = map2(s)/sum2(s)

    endif else begin

        map2(xmap2,ymap2) = map1

    endelse

    return, map2

end

        
