PRO PCA, data, eigenval, eigenvect, percentages, proj_obj, proj_atr, $
      MATRIX=A,COVARIANCE=cov,SSQ=ssq,SILENT=silent
     
; this version has changed TQLI to NR_TQLI and TRED2 to NR_TRED2.
; JAH 4/19/94

;+NAME:
;	PCA
;
; PURPOSE:
;	Carry out a Principal Components Analysis (Karhunen-Loeve Transform)
;
; CALLING SEQUENCE:
;	PCA, data, eigenval, eigenvect, percentages, proj_obj, proj_atr, 
;		[MATRIX =, TEXTOUT = ,/COVARIANCE, /SSQ, /SILENT ]
;
; INPUT PARAMETERS:
;	data -  2-d data matrix, data(i,j) contains the ith sample of the
;               jth variable.  DATA shoud be dimensioned (NSAMP,NVAR), where
;               NSAMP is the total number of samples and NVAR is the total
;               number of variable parameters.  For example, consider
;               a scatter plot of temperature vs relative humidity.
;               The first index of DATA cycles over all measurments and the
;               second index cycles over two parameters: T and Rh.  IF
;               T and Rh are 1-D arrays containing all sampled values then
;               DATA may be constructed as DATA=[[T],[Rh]].
;
; OPTIONAL INPUT KEYWORD PARAMETERS:
;	/COVARIANCE - if this keyword set, then the PCA will be carried out
;		on the covariance matrix (rare), the default is to use the
;		correlation matrix
;	/SSQ - if this keyword is set, then the PCA will be carried out on
;		on the sums-of-squares & cross-products matrix (rare)
;
; OPTIONAL OUTPUT PARAMETERS:
; eigenval	NVAR element vector containing the sorted eigenvalues
;	
; eigenvect 	NVAR x NVAR matrix containing the corresponding 
;		eigenvectors, first index cycles over different eigenvectors
;               second index cycles over original coordinate directions
;	
; percentages	NVAR element vector containing the cumulative percentage 
;		variances associated with the principal components
;	
; proj_obj 	NSAMP by NVAR matrix containing the projections of the 
;		objects on the principal components
;	
; proj_atr	NVAR by NVAR	matrix containing the projections of 
;		the attributes on the principal components
; 	
; MATRIX	analysed matrix, either the covariance matrix if
;               /COVARIANCE is set, the "sum of squares and
;               cross-products" matrix if /SSQ is set, or the (by
;               default) correlation matrix.  MATRIX will have
;               dimensions NVAR x NVAR
;
; NOTES:
;	This procedure performs Principal Components Analysis (Karhunen-Loeve
;	Transform) according to the method described in "Multivariate Data 
;	Analysis" by Murtagh & Heck [Reidel : Dordrecht 1987], pp. 33-48. 
;
;	Keywords /COVARIANCE and /SSQ are mutually exclusive.
;
;	The printout contains only (at most) the first seven principle 
;	eigenvectors.    However, the output variables EIGENVECT contain 
;	all the eigenvectors
;	
;       Different authors scale the covariance matrix in different ways.
;	The eigenvalues output by PCA may have to be scaled by 1/NSAMP or
;	1/(NSAMP-1) to agree with other author's calculations when /COVAR is 
;	set.
;
;	Users of V3.5.0 or later could change the calls to TQLI and TRED2
;	to NR_TQLI and NR_TRED2
;
; EXAMPLE:
;;	PCA analysis on a scatter plot of two variable, t1,t2, which we
;;      will assume are linearly dependent on 2 random gaussian variables
;;      x1,x2 as follows,
;;
;    nsamp=10000
;    x1=randomn(iseed,nsamp)   
;    x2=randomn(iseed,nsamp)
;    t1=5*x1+2*x2             ; <t1*t1> = <t2*t2> = 25*<x1^2>+4*<x2^2> = 29
;    t2=5*x1-2*x2             ; <t1*t2> = 25*<x1^2>-4*<x2^2> = 21
;                             ; 
;    data=[[t1],[t2]]         ; nsamp measurements of 2 parameters
;    pca,data,eval,evec,per   ;  
;                             ;  |1-ev    21/29|       eigenvalue 
;    print,eval               ;  |             | = 0   equation on  
;    print,evec               ;  |21/29   1-ev |       correlation matrix
;    print,per                ; 
;                             ; ev=1+-(21/29) = 1.7248, .2752
;
;    t=findrng(0,2*!pi,100)
;    xx1=evec(0,0)*20                       ; 1st eigenvector
;    yy1=evec(0,1)*20
;    xx2=sqrt(eval(1)/eval(0))*evec(1,0)*20 ; 2nd eigenvector
;    yy2=sqrt(eval(1)/eval(0))*evec(1,1)*20
;    plot,t1,t2,psym=3,xra=[-20,20],yra=[-20,20],pos=boxpos(/asp)        
;    oplot,xx1*[-1,1],yy1*[-1,1] & oplot,xx2*[-1,1],yy2*[-1,1]
;    oplot,cos(t)*xx1+sin(t)*xx2, cos(t)*yy1+sin(t)*yy2
;    putstr,'eigenvector 1',11.,16.6,chars=1.3
;    putstr,'eigenvector 2',5.4,-7.1,chars=1.3
;
;
; COPYRIGHT:
;	Copyright 1993, Hughes STX Corporation, Lanham MD 20706.
;
; REVISION HISTORY:
;	Immanuel Freedman (after Murtagh F. and Heck A.).     December 1993
;       code cleanup and corrected output description (PJR)   June     1997
;- 

; Constants
TOLERANCE = 1.0E-5              ; are array elements near-zero ?

IF N_PARAMS() EQ 0  THEN xhelp,'pca'

SZ = size(data)
if SZ(0) NE 2 THEN BEGIN
  HELP,data
  MESSAGE,'ERROR - Data matrix is not two-dimensional'
ENDIF

Nsamp = sz(1)   &  Mattr = sz(2) ;Number of objects and attributes

CASE 1 of
  keyword_set(cov): begin
    temp = replicate(1.0, Nsamp)
    column_mean = (temp # data)/Nsamp
    X = (data - temp # transpose(column_mean))  ; subtract row average
  end
  keyword_set(ssq): X = data 
  else: begin
    temp = replicate( 1.0, Nsamp )
    column_mean = (temp # data)/ Nsamp
    X = (data - temp # transpose(column_mean))  ; subtract row average
    S = sqrt(temp # (X*X)) 
    X = X/(temp # S)                            ; normalize by sqrt(variance)
  end                                           ;   of each variable
endcase

A = transpose(X) # X
; Carry out eigenreduction
nr_tred2, A, D, E               ; D contains diagonal, E contains off-diagonal
nr_tqli, D, E, A                ; D contains the eigen-values, A(*,i) -vectors

; Use TOLERANCE to decide if eigenquantities are sufficiently near zero

index = where(abs(D) LE TOLERANCE*MAX(abs(D)),count) 
if count NE 0 THEN D(index)=0
index = where(abs(A) LE TOLERANCE*MAX(abs(A)),count) 
if count NE 0 THEN A(index)=0

index = sort(D)                 ; Order by increasing eigenvalue
D = D(index) & E=E(index)
A = A(*,index)

; Eigenvalues expressed as percentage variance and ...

temp = replicate(1.0, Mattr )
W1 = 100.0 * reverse(D)/total(D)

;... Cumulative percentage variance

C = fltarr( Mattr, Mattr) 
C(where(lindgen(Mattr,Mattr) LE (Mattr+1)*(lindgen(Mattr)#temp))) = 1
W = C # W1

;Define returned parameters

eigenval = reverse(D)
eigenvect = reverse(transpose(A))
percentages = W

; Output eigen-values and -vectors 

; Obtain projection of row-point on principal axes  (Murtagh & Heck convention)

projx = X # A

; Use TOLERANCE again...

index = where(abs(projx) LE TOLERANCE*MAX(abs(projx)),count)
if count NE 0 THEN projx(index)=0
proj_obj = reverse( transpose(projx) )

; Obtain projection of column-points on principal axes

projy = transpose(projx)#X

; Use TOLERANCE again...

index = where(abs(projy) LE TOLERANCE*MAX(abs(projy)),count)
if count NE 0 THEN projy(index) = 0

; scale by square root of eigenvalues...

temp = replicate( 1.0, Nsamp )
proj_atr = reverse(projy)/(sqrt(W)#temp)

RETURN
END



