pro read_toms,file,ozone,lons,lats
;+
; ROUTINE:  read_toms,file,ozone
;
; PURPOSE:  read toms ascii data files 
;
; USEAGE:   read_toms,ozone,lons,lats
;
; INPUT:    
;  file     Name of the toms archive data file.
;
; OUTPUT:
;  ozone    ozone in dobson units in an global output grid
;           Longitudes:  288 bins centered on 179.375 W to 179.375 E  
;           (1.25 degree steps)  
;           Latitudes :  180 bins centered on  89.5   S to  89.5   N  
;           (1.00 degree steps)  
;
;  lons     vector of longitudes (degrees)
;  lats     vector of latitudes  (degrees)
;
;
; DISCUSSION:
;
;   TOMS ozone data files from the Earth Probe satellite may be
;   obtained via anonymous ftp from:
;
;   /anonymous@jwocky.gsfc.nasa.gov:/pub/eptoms
;
;   Each file contains global noon time measurements of total column 
;   ozone for one day.  The data format is directly readable by this 
;   procedure.  Data is available for 1996, 1997 and 1998 (in
;   directories  oz96, oz97 and oz98)
;
; EXAMPLE:  
;
;           read_toms,'ga970930.ept',ozone,lons,lats
;           tvim,ozone,xrange=lons,yrange=lats,/scale
;           map_set,0,0,/cont,/cyl,/noerase,pos=boxpos(/get)
;
;; find toms ozone at sgp site for each day in
;; september and october of 1997
;
;           files=findfile('ozone/*.ept')
;           lat=36.605 & lon=-97.485
;           oz=fltarr(n_elements(files))
;           for i=0,n_elements(files)-1 do begin &$
;             print,files(i) &$
;             read_toms,files(i),ozone,lons,lats &$
;             fill_image,ozone,where(ozone eq 0.) &$
;             oz(i)=interp(ozone,lons,lon,lats,lat) &$
;           endfor
;
;
;            
;
; AUTHOR:   Paul Ricchiazzi                        04 Jun 98
;           Institute for Computational Earth System Science
;           University of California, Santa Barbara
;           paul@icess.ucsb.edu
;
; REVISIONS:
;
;-
;

line=''
ozone=fltarr(288,180)
buf=intarr(288)

openr,lun,/get_lun,file
for i=1,3 do readf,lun,line

for j=0,179 do begin
  readf,lun,f='((1x,25i3))',buf
  ozone(*,j)=buf
endfor
free_lun,lun

lons=findrng(-179.375,179.375,288)
lats=findrng(-89.5,89.5,180)
return
end
