function roots,xx,x
;+
; ROUTINE:  roots
;
; PURPOSE:  find floating point index, f, such that x=interpolate(xx,f)
;           where xx is a (possibly non-monotonic) table of values and 
;           x is a scalor.  ROOTS returns -1 if no roots are found.
;
; USEAGE:   result=roots(xx,x)
;
; INPUT:    
;  xx       a table of real numbers, need not be monotonic 
;  x        a real number
;
; DISCUSSION:
;           Though ROOTS isn't sophisticated, it is thorough.  It
;           checks all the places where ff=(x-xx)/(shift(xx,-1)-xx) is
;           between [0,1).  and returns the value ii+ff, where ii is
;           the index for which x is bounded by xx(ii) and xx(ii+1).
;           NOTE: Sparsely populated tables may not produce a
;           satisfactory accuracy.
;           
;           
;
; EXAMPLE:  
;
;;  find the roots of the equation t^3-10*t^2-200*exp(-t)+100. = 0
;
;   t=findrng(.0,10,1000)
;   xx=t^3-10*t^2-200*exp(-t)+100.
;   plot,t,xx 
;   f=roots(xx,0)
;   oplot,interpolate(t,f),interpolate(xx,f),psym=2
;   oplot,!x.crange,[0,0],color=100
;   print,'     t(f)     =',interpolate(t,f)
;
;;  find zeroes of legendre polynomial
;
;   x=findrng(-1.,1.,10000)
;   n=16 & f=legendre(n,x) & r=roots(f,0.) & print,interpolate(x,r)
;
; AUTHOR:   Paul Ricchiazzi                        03 Apr 97
;           Institute for Computational Earth System Science
;           University of California, Santa Barbara
;           paul@icess.ucsb.edu
;
; REVISIONS:
;
;-
;

nn=n_elements(xx)

denom=shift(xx,-1)-xx
iok=where(denom ne 0., nok)
f=(xx-xx)-1
f(iok)=(x-xx(iok))/denom(iok)
f=f(0:nn-2)
iroot=where(f ge 0. and f lt 1.,nroot)
if nroot eq 0 then return,-1
return,iroot+f(iroot)

end

