function rowcol,ix=ix,iy=iy,kx=kx,ky=ky
;+
; ROUTINE:  rowcol
;
; PURPOSE:  compute the plot frame position of next plot
;
; USEAGE:   result=rowcol(ix=ix,iy=iy)
;
; INPUT:    
;
; KEYWORD INPUT:
;
;   ix      the x position index starting from 0 on left, 
;           ix<0 indicates rightmost column of plots
;
;   iy      the y position index starting from 0 on bottom
;           iy<0 indicates top row of plots
;
; OUTPUT:
;   result  1 if plot position matches ix or iy position index,
;           otherwise zero
;
; DISCUSSION:
;   This routine is intended to simplify labeling setup for multiframe
;   plots.  Use ROWCOL in IF statements to set the XTITLE and YTITLE
;   as appropriate for plots in the left or right column or bottom or
;   top row.  Since ROWCOL interprets a negative IX or IY as the
;   rightmost or top row of plots, respectively, you don't need to
;   explicitly refer to !p.multi in your plot routines when all you
;   want to do is set a special value of XTITLE for the right column
;   or top row of plots.
;
; EXAMPLE:  
;
;           !p.multi=[0,2,3]
;
;; execute these lines four times:
;
;           if rowcol(ix=0) then ytitle='ylabel' else ytitle=''
;           if rowcol(iy=0) then xtitle='xlabel' else xtitle=''
;           plot,dist(20),xtitle=xtitle,ytitle=ytitle
;
; AUTHOR:   Paul Ricchiazzi                        09 Apr 97
;           Institute for Computational Earth System Science
;           University of California, Santa Barbara
;           paul@icess.ucsb.edu
;
; REVISIONS:
; nov97: now works for column major ordering
;
;-
;

if !p.multi(0) eq 0 then pind=0 else pind=!p.multi(1)*!p.multi(2)-!p.multi(0)
ny=(!p.multi(2)>1)
nx=(!p.multi(1)>1)

if !p.multi(4) eq 1 then begin
  ky=ny-1- (pind mod ny)
  kx=pind/ny
endif else begin
  kx=pind mod nx
  ky=nx-1-pind/nx
endelse

if n_elements(ix) ne 0 then begin
  jx=ix
  if jx lt 0 then jx=nx-1
endif else begin
  jx=kx
endelse

if n_elements(iy) ne 0 then begin
  jy=iy
  if jy lt 0 then jy=ny-1
endif else begin
  jy=ky
endelse
 
return,jx eq kx and jy eq ky
end
