pro sconvol,x,y,fwhm,xx,yy,type=type,isample=isample,osample=osample
;+
; ROUTINE:  sconvol
;
; PURPOSE:  convolve a spectrum defined on an arbitrary 1-d grid
;           with a response function.  
;
; USEAGE:   sconvol,x,y,fwhm,xx,yy,type=type,isample=isample,osample=osample
;
; INPUT:    
;
;   x 
;     x value 
;
;   y 
;     spectral values
;
;   fwhm   
;     filter width at half of maximum value in x units
;
; KEYWORD INPUT:
;   type  
;     filter type 
;     0 = square      (default)
;     1 = triangular
;
;   isample
;     number of pre-convolution samples taken within a given fwhm interval
;     (default = 10) 
;
;   osample
;     number of output samples provided within a given fwhm interval
;     (default = 1)
;
; OUTPUT:
;
;   xx
;      x values at sample points, output values will be at multiples of
;      of OSAMPLE*FWHM starting at min(XX)+.5*FWHM
;
;   yy
;      response values at xx
;
; DISCUSSION:
;
; EXAMPLE: degrade MODTRAN spectral transmission to LOWTRAN resolution
; 
;      modtrns,buf,/noplot
;      w=reform(buf(0,*))
;      t=w & t(*)=1.
;      for i=1,11 do t=t*reform(buf(i,*))
;      sconvol,w,t,20,ww,tt,type=0
;      plot,10000./ww,tt,xrange=[.25,1]
;
;
;; show oxygen A line at 1 nm resolution
;
;      sconvol,10000/w,t,.001,w1,t1
;      plot,10000/w,t,xrange=[.755,.77]
;      oplot,w1,t1,psym=10
;
;; and at 20 cm-1 resolution
;      sconvol,w,t,20,w20,t20
;      plot,10000./w,t,xrange=[.755,.77]
;      oplot,10000./w20,t20,psym=10

;
; AUTHOR:   Paul Ricchiazzi                        20 Nov 96
;           Institute for Computational Earth System Science
;           University of California, Santa Barbara
;           paul@icess.ucsb.edu
;
; REVISIONS:
;
;-
;

if not keyword_set(type) then type = 0
if not keyword_set(isample) then isample=10
if not keyword_set(osample) then osample=1

xmin=min(x,max=xmax)

nn=1+isample*(xmax-xmin)/fwhm

xs=findrng(xmin,xmax,nn)
ys=interp(y,x,xs)

case type of
  0: filter=replicate(1,isample)
  1: filter=[1+findgen(isample/2),1+reverse(isample-isample/2)]
end

yc=convol(ys,filter,/edge_truncate)/total(filter)

xx=findrng(xmin+.5*fwhm,xmax-.5*fwhm,1+osample*(xmax-xmin-fwhm)/fwhm)
yy=interp(yc,xs,xx)
 
return
end

