/*
**-----------------------------------------------------------------------------
**
** NAME:
**
**      ies_bds.c
**
** PURPOSE:
**      This C function is used to demonstrate how to pass simple 
**      variable from IDL to a C function using the IDL function
**      CALL_EXTERNAL. 
**
** CATEGORY:
**      Dynamic Link
**
** ROUTINES:
**      READ_ies
**      read_bds
**
**-----------------------------------------------------------------------------
** READ_IES
**
**      INPUTS:
**              filename        A string with the Filename (an IDL STRING struct)
**              starting_sample The starting record number 
**              ending_sample   The starting record number 
**              field_number    The field to extract
**
**      OUTPUTS:
**              ies_data        2D floating point array with values from
**                              the input file. Each column corresponds
**                              to the Vfield.  
**
**      SIDE EFFECTS:
**              None.
**
**
** IDL EXAMPLE:
**-----------------------------------------------------------------------------
;; The following is the commands that would be used to call this
;; routine in IDL
;;
	filename = "/CERES/instrument/data/out_exp/IES_Hour_00"
	starting_sample=long(10);       
	ending_sample=long(20); 

	result = CALL_EXTERNAL('ies_bds.so', 'read_ies',        $
				filename,                       $
				starting_sample,                $
				ending_sample,                  $
				ies_data)

**-----------------------------------------------------------------------------
**
** READ_BDS
**
** MODIFICATION HISTORY:
**      Written April, 1996                                     LEM
**      Add function to get number of records   07/11/96        LEM 
**      Corrected bug in subsetting from        07/11/96        LEM
**              the starting position   
**      Added list_sds and list_vdata           07/18/96        LEM
**      Added capability to read vdatas         02/13/97        Kevin McIntire
**      Added capability to read sds and 
**              attributes                      04/28/98        Kam-Pui Lee
**
**-----------------------------------------------------------------------------
*/

/*
-----------------------------------
	Include Files
-----------------------------------
*/
#include <stdio.h>
#include <string.h>
#include <hdf.h>
#include <vattr.h>
/* #include <export.h> */

#define BDS_MAX_ITEMS   30      
#define FIELD_NAME_MAX  64
#define IES_MAX_ITEMS   30      
#define IES_RECORD_SIZE 136
/* #define _HDF_VDATA 0xffffffff */

/*
-----------------------------------
	Declare the structure 
	for an IDL string 
	(From IDL User's Guide).
-----------------------------------
*/

typedef struct {
   unsigned short slen;         /* length of the string         */
   short stype;                 /* Type of string               */      
   char *s;                     /* Pointer to chararcter array  */
} STRING;

typedef struct 
{
	float   items[IES_MAX_ITEMS];
} IES_ROW;

/*  for the Release 2 */
typedef struct 
{
	float32         Colat_TOA;
	float32         Long_TOA;
	float32         Colat_Surface;
	float32         Long_Surface;
	float32         View_Zenith_Angle;
	float32         Solar_Zenith_Angle;
	float32         Rel_Az_Angle;
	float32         Az_View_Angle_North;
	float32         CrossTrack_Angle;
	float32         Along_Track_Angle;

	float32         Cone_Angle;
	float32         Clock_Angle;
	float32         Cone_Angle_Rate;
	float32         Clock_Angle_Rate;
	float64         Sat_Vel_X;
	float64         Sat_Vel_Y;
	float64         Sat_Vel_Z;
	float64         Radius_From_Earth;

	float32         Tot_Filt_Radiance;
	float32         SW_Filt_Radiance;
	float32         Win_Filt_Radiance;

	float32         Colat_Sat;
	float32         Long_Sat;
	float32         Colat_Sun;
	float32         Long_Sun;
/*      float32         Earth_Sun_Dist;  */
	uint16          Sample_Number;
	uint16          ScanLine_Number;
	float64         Observation_Time;
	uint32          Quality_Flags;
	uint32          Absolute_ScanLine_Number;

} IES_RECORD_TYPE;

typedef struct 
{
	float   items[BDS_MAX_ITEMS];
} BDS_ROW;

/*
---------------------------------------------------------------
	get_attr_string
---------------------------------------------------------------
*/
char    *get_attr_string(char *buffer, int32 data_type, int32 count)
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int             i,j;
	char            dummy[4096];
	char            temp[256];
	int32           icount;
	int8            *ptr_I8;
	uint8           *ptr_U8;
	int16           *ptr_I16;
	uint16          *ptr_U16;
	int32           *ptr_I32;
	uint32          *ptr_U32;
	float32         *ptr_F32;
	float64         *ptr_F64;
	size_t          length;
	int32           status;
	char            *loc;

	dummy[0]=NULL;
	switch (data_type) 
	{
		case DFNT_CHAR8:
		case DFNT_UCHAR8:
			*(buffer+(count*DFKNTsize(data_type))) = NULL;
			strcpy(dummy,buffer);
			break;
		case DFNT_INT8:
			ptr_I8 = (int8 *)buffer;
			for(icount=0;icount<count;icount++)
			{
				temp[0]=NULL;
				sprintf(temp,"%d;",*(ptr_I8+icount));
				strcat(dummy,temp);
				if(strlen(dummy) > 4086) break;
			}
			break;
		case DFNT_UINT8:
			ptr_U8 = (uint8 *)buffer;
			for(icount=0;icount<count;icount++)
			{
				temp[0]=NULL;
				sprintf(temp,"%u;",*(ptr_U8+icount));
				strcat(dummy,temp);
				if(strlen(dummy) > 4086) break;
			}
			break;
		case DFNT_INT16:
			ptr_I16 = (int16 *)buffer;
			for(icount=0;icount<count;icount++)
			{
				temp[0]=NULL;
				sprintf(temp,"%d;",*(ptr_I16+icount));
				strcat(dummy,temp);
				if(strlen(dummy) > 4086) break;
			}
			break;
		case DFNT_UINT16:
			ptr_U16 = (uint16 *)buffer;
			for(icount=0;icount<count;icount++)
			{
				temp[0]=NULL;
				sprintf(temp,"%u;",*(ptr_U16+icount));
				strcat(dummy,temp);
				if(strlen(dummy) > 4086) break;
			}
				break;
		case DFNT_INT32:
			ptr_I32 = (int32 *)buffer;
			for(icount=0;icount<count;icount++)
			{
				temp[0]=NULL;
				sprintf(temp,"%d;",*(ptr_I32+icount));
				strcat(dummy,temp);
				if(strlen(dummy) > 4086) break;
			}
			break;
		case DFNT_UINT32:
			ptr_U32 = (uint32 *)buffer;
			for(icount=0;icount<count;icount++)
			{
				temp[0]=NULL;
				sprintf(temp,"%u;",*(ptr_U32+icount));
				strcat(dummy,temp);
				if(strlen(dummy) > 4086) break;
			}
			break;
		case DFNT_FLOAT32:
			ptr_F32 = (float32 *)buffer;
			for(icount=0;icount<count;icount++)
			{
				temp[0]=NULL;
				sprintf(temp,"%g;",*(ptr_F32+icount));
				strcat(dummy,temp);
				if(strlen(dummy) > 4086) break;
			}
			break;
		case DFNT_FLOAT64:
			ptr_F64 = (float64 *)buffer;
			for(icount=0;icount<count;icount++)
			{
				temp[0]=NULL;
				sprintf(temp,"%g;",*(ptr_F64+icount));
				strcat(dummy,temp);
				if(strlen(dummy) > 4086) break;
			}
			break;
		default:
			fprintf(stderr,"Does not support this type: %d\n",data_type);
			strcpy(dummy,"Unknown");
	}
		if(data_type > DFNT_CHAR8)
		{
			length=strlen(dummy);
			if(length > 0) dummy[length-1]=NULL;
		}

   return(dummy);
}

/*
---------------------------------------------------------------
	add_field
---------------------------------------------------------------
*/
void    add_field(char *the_names, char* the_field)
{
	int     i;
	int     index;
	int     temp1;
	int     temp2;

	/* fprintf(stdout, "Name=%s field=%s\n",the_names,the_field); */

	index=strlen(the_names);
	strcat(the_names,the_field);
	temp1=index + strlen(the_field);
	temp2=index + FIELD_NAME_MAX;

	fprintf(stdout, ">>>index=%i temp1=%i temp2=%i\n",index,temp1,temp2);

	for (i=temp1; i<temp2-index; i++)
	{
		the_names[i]=32;
	}       
}

/*
---------------------------------------------------------------
	set_field_names
---------------------------------------------------------------
*/
void    set_field_names(char *the_names, int count, int vid)
{
	int     i;
	char    *temp_name;

	strcpy(the_names,"");
	for (i=0; i<count; i++) 
	{
		temp_name=VFfieldname(vid,i);
		strcat(the_names,temp_name);
		if (i!=count-1)
		{
			strcat(the_names,",");
		}
	}

	/*
	strcpy(the_names,"");
	add_field(the_names,"Colat_TOA");
	strcat(the_names,",");
	add_field(the_names,"Long_TOA");
	*/

}

/*
---------------------------------------------------------------
	print_fields    
---------------------------------------------------------------
*/
void    print_fields(IES_RECORD_TYPE    this_ies)
{
	fprintf(stdout,"-------------------------------------------\n");
	fprintf(stdout,"\tColat_TOA: \t\t%f\n",
			this_ies.Colat_TOA);

	fprintf(stdout,"\tLong_TOA: \t\t%f\n",
			this_ies.Long_TOA);

	fprintf(stdout,"\tColat_Surface: \t\t%f\n",
			this_ies.Colat_Surface);

	fprintf(stdout,"\tLong_Surface: \t\t%f\n",
			this_ies.Long_Surface);

	fprintf(stdout,"\tView_Zenith_Angle: \t%f\n",
			this_ies.View_Zenith_Angle);
}

/*
---------------------------------------------------------------
	get_number_of_ies_records
---------------------------------------------------------------
*/

long get_ies_n_rec(argc, argv)
	int argc;
	void *argv[];
{
	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           current_vid;
	int32           current_vref;
	int32           file_id;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int32           n_records;
	long            result;
	int32           status;
	char            current_name[64];
	char            field_name[32678];
	char            *loc, *field_loc;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/* printf("get_number_of_ies_records was called...\n"); */
	if(argc != 1)
	{
		fprintf(stderr,"simple: Incorrect number of arguments\r\n");
		return(-1);  /* Signal an error */
	}
	/*
	** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id == -1)
	{
	   fprintf(stdout, "\tThe file id for %s is FAIL in get_ies_n_rec!.\n", \
	     filename->s);
	   result = -1;
	   return (result);
	}

	Vstart(file_id);

	current_vref = -1;
	while(1)
	{
	   current_vref=VSgetid(file_id, current_vref);
	   if (current_vref == -1)
	   {
	      fprintf(stdout, "\tThe current_vref for Vdata name is not found in get_ies_n_rec!\n");
	      Vend(file_id);
	      Hclose(file_id);
	      result = -1;
	      return (result);
	   }

	   current_vid=VSattach(file_id,current_vref,"r");
	   if (current_vid == -1)
	   {
	      fprintf(stdout, "\tThe current vid is bad in get_ies_n_rec!\n");
	      Vend(file_id);
	      Hclose(file_id);
	      result = -1;
	      return (result);
	    }

	    status=VSinquire(current_vid,&n_records,NULL,field_name,NULL,current_name);
	    if (status == -1)
	    {
	       fprintf(stdout, "\tThe status for VSinquire in FAIL in get_ies_n_rec!\n");
	       VSdetach(current_vid);
	       Vend(file_id);
	       Hclose(file_id);
	       result = -1;
	       return (result);
	    }

	    loc = strstr(current_name, "IES Hour");
	    if(loc != NULL)
	    {
	       VSdetach(current_vid);
	       Vend(file_id);
	       Hclose(file_id);
	       field_loc = strstr(field_name, "Earth_Sun_Dist");
	       if(field_loc != NULL)
	       {
		 fprintf(stdout, "This is not a Release 2 IES file.\n");
		 result = -1;
	       }
	       else
	       {
		 result=(long)n_records;
	       }
	       break;
	    }
	}
	return(result);
}

/*
---------------------------------------------------------------
	get_vdata_n_rec
---------------------------------------------------------------
*/

long get_vdata_n_rec(argc, argv)
	int argc;
	void *argv[];
{
	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           current_vid;
	int32           current_vref;
	STRING          *data_name;
	int32           file_id;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int32           n_records;
	long            result;
	int32           status;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/* printf("get_vdata_n_rec was called...\n"); */
	if(argc != 2)
	{
		fprintf(stderr,"simple: Incorrect number of arguments\r\n");
		return(-1);  /* Signal an error */
	}
	/*
	** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name=(STRING *) argv[1];
	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id!=-1)
	{
		Vstart(file_id);
		current_vref=VSfind(file_id, data_name->s);
/*                if (current_vref!=-1)  */
		if (current_vref)
		{
		   current_vid=VSattach(file_id,current_vref,"r");
		   if (current_vid!=-1)
		   {

			status=VSinquire(current_vid,&n_records,
				NULL,NULL,NULL,NULL);
			if (status!=-1)
			{
				VSdetach(current_vid);
				Vend(file_id);
				Hclose(file_id);
				result=(long)n_records;
			}
			else
			{
				VSdetach(current_vid);
				Vend(file_id);
				Hclose(file_id);
				result=-1;
			}
		   }
		   else
		   {
			fprintf(stdout,"\tCurrent Vid is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			result=-1;
		   }            
		}
		else
		{
			fprintf(stdout,"\tCurrent Vref is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			result=-1;
		}
	}
	else
	{
		result=-1;
	}
	return(result);
}

/*
---------------------------------------------------------------
	get_vfield_types
---------------------------------------------------------------
*/

char *get_vfield_types(argc, argv)
	int argc;
	void *argv[];
{
	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           current_vfield_index;
	int32           current_vid;
	int32           current_vref;
	STRING          *data_name;
	int32           file_id;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int             i, j, k;
	int32           n_records;
	int32           *output_order;
	int32           *output_types;
	char            result[256];
	int32           status;
	int32           nattrs;
	char            attr_name[256];
	int32           data_type;
	int32           count;
	int32           attr_size;
	size_t          offset;
	int32		loc;
	char            *loc_start;
	char            *loc_end;
	char            *buffer;
	int             *vfield_count;
	int32           *vfield_indices;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	printf("get_vfield_types was called...\n"); 
	if(argc != 6)
	{
		fprintf(stderr,"get_vfield_types: Incorrect number of arguments\r\n");
		strcpy(result, "BAD");
		return(result);  /* Signal an error */
	}
	/*
	** Cast the pointer in argv to the pointer variables
	*/
	/* Changed pointer casts to HDF types.  Kevin McIntire, 2/13/97 */
	filename= (STRING *) argv[0];
	data_name=(STRING *) argv[1];
	vfield_indices=(int32 *) argv[2];
	vfield_count=(int *) argv[3];
	output_types=(int32 *) argv[4];
	output_order=(int32 *) argv[5];

	/*
	for (i=0; i<*vfield_count; i++)
	{
		output_order[i]=i*100;
		output_types[i]=i*200;
	}
	*/
	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_vfield_types");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: %s\r\n", filename->s );
	fprintf(stdout, "\tDataname: %s\r\n", data_name->s );
	for (i=0; i<*vfield_count; i++)
	{
		fprintf(stdout, "\tFieldindices: \t%i\r\n", vfield_indices[i] );
	}

	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");

	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id!=-1)
	{
		Vstart(file_id);
		current_vref=VSfind(file_id, data_name->s);
/*                if (current_vref!=-1)  */
		if (current_vref)
		{
		   current_vid=VSattach(file_id,current_vref,"r");
		   if (current_vid!=-1)
		   {
			for (i=0; i<*vfield_count; i++)
			{
				/* Convert name to field_index */
				current_vfield_index=vfield_indices[i];
				status=VFfieldtype(current_vid,current_vfield_index);
				if (status!=-1)
				{
					output_types[i]=status;
					/* printf("The field type is %d\n",output_types[i]); */
					/* result=0; */
				}
				else
				{
					fprintf(stdout,"\tVFfieldtype is bad!\n");
					strcpy(result, "BAD");
					break;
				}
				status=VFfieldorder(current_vid,current_vfield_index);
				if (status!=-1)
				{
					output_order[i]=status;
					/* printf("The field order is %d\n",output_order[i]); */
					/* result=0; */
				}
				else
				{
					fprintf(stdout,"\tVFfieldorder is bad!\n");
					strcpy(result, "BAD");
					break;
				}
				strcpy(result, "BAD");
				nattrs = VSfnattrs(current_vid, current_vfield_index);
				/* printf("nattrs = %d\n", nattrs); */
				if(nattrs != 0)
				{
				    k = 0;
				    for(j = 0; j < nattrs; j++)
				    {
					status = VSattrinfo(current_vid, current_vfield_index, j, attr_name, &data_type, &count, &attr_size);
					if(status == -1) continue;
					/* printf("%d: attr_name = %s  data_type = %d   count = %d   attr_size = %d\n", \
						j, attr_name, data_type, count, attr_size); */
					buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
					if(buffer != NULL)
					{
						status = VSgetattr(current_vid, current_vfield_index, j, buffer);
						if(status != -1)
						{
							*(buffer+count*DFKNTsize(data_type)) = NULL;
							/* printf("buffer = %s\n", buffer); */
							loc = strcmp(attr_name, "units");
							if(loc == 0)
							{
								strcpy(result, buffer);
								k = 1;
							}
							else
							{
								loc_start = strstr(buffer, "unit is");
								if(loc_start != NULL)
								{
									loc_start = loc_start + 8;
									offset = strspn(loc_start, " ");
									loc_start = loc_start + offset - 1;
									loc_end = strchr(loc_start, ';');
									if(loc_end != NULL)
									{
										*loc_end = NULL;
										strcpy(result, loc_start);
										k = 1;
									}
								}
								else
								{
							   		loc_start = strstr(buffer, "units is");
							   		if(loc_start != NULL)
							   		{
										loc_start = loc_start + 9;
										offset = strspn(loc_start, " ");
										loc_start = loc_start + offset - 1;
										loc_end = strchr(loc_start, ';');
										if(loc_end != NULL)
										{
											*loc_end = NULL;
											strcpy(result, loc_start);
											k = 1;
										}
							   		}
								}
							}
						}
						free(buffer);
						if(k == 1) break;
					}
				    }
				}
				/* printf("result = %s\n", result); */
				if( ! strcmp(result, "BAD"))
				{
				    nattrs = VSnattrs(current_vid);
				    /* printf("nattrs for vdata = %d\n", nattrs); */
				    if(nattrs != 0)
				    {
					k = 0;
					for(j = 0; j < nattrs; j++)
					{
					   status = VSattrinfo(current_vid, _HDF_VDATA, j, attr_name, &data_type, &count, &attr_size);
					   if(status == -1) continue;
					   /* printf("%d: attr_name = %s  data_type = %d   count = %d   attr_size = %d\n", \
						j, attr_name, data_type, count, attr_size); */
					   buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
					   if(buffer != NULL)
					   {
						status = VSgetattr(current_vid,  _HDF_VDATA, j, buffer);
						if(status != -1)
						{
							*(buffer+count*DFKNTsize(data_type)) = NULL;
							/* printf("buffer = %s\n", buffer); */
							loc = strcmp(attr_name, "units");
							if(loc == 0)
							{
								strcpy(result, buffer);
								k = 1;
							}
							else
							{
								loc_start = strstr(buffer, "unit is");
								if(loc_start != NULL)
								{
									loc_start = loc_start + 8;
									offset = strspn(loc_start, " ");
									loc_start = loc_start + offset - 1;
									loc_end = strchr(loc_start, ';');
									if(loc_end != NULL)
									{
										*loc_end = NULL;
										strcpy(result, loc_start);
										k = 1;
									}
								}
								else
								{
							   		loc_start = strstr(buffer, "units is");
							   		if(loc_start != NULL)
							   		{
										loc_start = loc_start + 9;
										offset = strspn(loc_start, " ");
										loc_start = loc_start + offset - 1;
										loc_end = strchr(loc_start, ';');
										if(loc_end != NULL)
										{
											*loc_end = NULL;
											strcpy(result, loc_start);
											k = 1;
										}
							   		}
								}
							}
						}
						free(buffer);
						if(k == 1) break;
					   }
					}
				    }
				}
			}
			VSdetach(current_vid);
			Vend(file_id);
			Hclose(file_id);
		   }
		   else
		   {
			fprintf(stdout,"\tCurrent Vid is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			strcpy(result, "BAD");
		   }            
		}
		else
		{
			fprintf(stdout,"\tCurrent Vref is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			strcpy(result, "BAD");
		}
	}
	else
	{
		strcpy(result, "BAD");
	}
	return(result);
}

/*
---------------------------------------------------------------
	read_ies        
---------------------------------------------------------------
*/
long read_ies(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	unsigned char   *buffer;
	unsigned char   *buffer_ptr;
	int32           buffer_size;
	char            dummy_names[4096];
	IES_RECORD_TYPE ies_record; 
	int32           current_vid;
	int32           current_vref;
	int             *ending_sample;
	char            field_names[4096];
	int32           file_id;
	char            find_name[64];
	int             len;
	uint16          temp16;
	uint32          utemp32;
	float32         temp32;
	float64         temp64;
	int             i;
	int32           number_of_fields;
	long            result;
	int32           size_of_data;
	int32           status;
/*  for VSinquire */
	int32           n_record;
	int32           record_size;
	char            *loc;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int             *starting_sample;
	int             *this_stuff;
	IES_ROW         *ies_data;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		** Insure that the correct number of arguments were passed in 
	*/
	if(argc != 4)
	{
		fprintf(stderr,"simple: Incorrect number of arguments\r\n");
		return(-1);  /* Signal an error */
	}

	/*
	** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	starting_sample=(int *)argv[1];
	/* printf("C>>>starting=%i\n",*starting_sample); */

	ending_sample=(int *)argv[2];
	/* printf("C>>>ending=%i\n",*ending_sample); */

	/* Changed casting to be more correct and to remove annoying compiler warnings.  Kevin McIntire, 2/13/97 */
	/*ies_data=(float *)argv[3];*/
	ies_data = (IES_ROW *) argv[3];
	size_of_data=(int32)(*ending_sample);
	/* size_of_data=(int32)(*ending_sample)-(*starting_sample); */

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	/*
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C read_ies");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");

	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	*/

	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	/* fprintf(stdout,"\tThe file id is: \t%i\n"); */

	if (file_id == -1)
	{
	   fprintf(stdout, "\tThe file id for %s is FAIL in read_ies!\n", filename->s);
	   result = -1;
	   return (result);
	}
 
	Vstart(file_id);

	current_vref = -1;
	while(1)
	{
	  current_vref=VSgetid(file_id, current_vref);
	  if (current_vref == -1)
	  {
	     fprintf(stdout, "\tThe current_vref for Vdata name is not found in read_ies!\n");
	     Vend(file_id);
	     Hclose(file_id);
	     result = -1;
	     return (result);
	  }

	  current_vid=VSattach(file_id,current_vref,"r");
	  if (current_vid == -1)
	  {
	     fprintf(stdout, "\tThe current_vid is bad in read_ies!\n");
	     Vend(file_id);
	     Hclose(file_id);
	     result = -1;
	     return (result);
	  }


	  status = VSinquire(current_vid, &n_record, NULL, dummy_names, &record_size, find_name);
/*
	  fprintf(stdout,"There are %d of records.\n", n_record);
	  fprintf(stdout, "The record size is %d.\n", record_size);
	  fprintf(stdout, "The define size is %d.\n", sizeof(IES_RECORD_TYPE));
	  fprintf(stdout, "The field names: %s.\n", dummy_names);
*/
	  if(status == -1)
	  {
	     fprintf(stdout, "\tThe status for VSinquire in read_ies is FAIL1\n");
	     VSdetach(current_vid);
	     Vend(file_id);
	     Hclose(file_id);
	     result = -1;
	     return (result);
	  }

	  loc = strstr(find_name, "IES Hour");
	  if(loc == NULL)
	  {
	     VSdetach(current_vid);
	     continue;
	  } 
				
	  number_of_fields=VSgetfields(current_vid, dummy_names);
/*        fprintf(stdout,"\tNumber of fields: \t%i\n", number_of_fields);  */
/*
	  fprintf(stdout,"\tDummy Names: \t\t>>%s<<\n", dummy_names);
*/
	  set_field_names(field_names,number_of_fields,current_vid);
/*
	  fprintf(stdout,"\tField names: \t\t>>%s<<\n", field_names);
*/
	  len=strlen(field_names);

/*       printf("\tField Name Length is: \t%i\n",len); */
/*
	 for (i=0; i<len; i++)
	 {
	 fprintf(stdout, "\t[%i]=%c--%i\n",i,field_names[i],field_names[i]);
	 }
*/
	 if (number_of_fields == -1)
	 {
	    fprintf(stdout,"\tNumber of fields is bad!\n");
	    VSdetach(current_vid);
	    Vend(file_id);
	    Hclose(file_id);
	    result=-1;
	    return (result);
	 }

	 status=VSseek(current_vid,0);
	 if (status==-1)
	 {
	    fprintf(stdout,"\tVSseek is Fail in read_ies!\n");
	    VSdetach(current_vid);
	    Vend(file_id);
	    Hclose(file_id);
	    result = -1;
	    return (result);
	 }

	 status=VSsetfields(current_vid, field_names);
/*       fprintf(stdout,"\tStatus of VSsetfields: \t%i\n", status); */
	 if (status == -1)
	 {
	   fprintf(stdout,"\tVSsetfields is bad!\n");
	   VSdetach(current_vid);
	   Vend(file_id);
	   Hclose(file_id);
	   result=-1;
	   return (result);
	}

/* Get size */
/*        buffer_size=IES_RECORD_SIZE*size_of_data;       */
	buffer_size=record_size*size_of_data;   

/*      if (buffer_size>0 && status!=-1) */
	if (buffer_size <= 0 )
	{
	   fprintf(stdout,"\tBuffer size is bad!\n");
	   VSdetach(current_vid);
	   Vend(file_id);
	   Hclose(file_id);
	   result = -1;
	   return (result);
	}

/* Add check on pointer returned from malloc */
	if ((buffer=malloc(buffer_size)) == NULL)
	{
	   printf("Out of memory in read_ies\n");
	   VSdetach(current_vid);
	   Vend(file_id);
	   Hclose(file_id);
	   result = -1;
	   return (result);
	}

	status=VSread(current_vid,buffer, size_of_data,FULL_INTERLACE);
	if (status == -1)
	{
	   fprintf(stdout,"\tVSread is bad!\n");
	   if (buffer != NULL) free(buffer);
	   VSdetach(current_vid);
	   Vend(file_id);
	   Hclose(file_id);
	   result=-1;
	   return (result);
	}

	VSdetach(current_vid);
	Vend(file_id);
	Hclose(file_id);
	break;
      }

/* Uncram/Unpack the buffer */ 
	buffer_ptr=buffer;
	for (i=0; i<size_of_data; i++)
	{
/* Colatitude of CERES FOV at TOA */

	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Colat_TOA=temp32;
	  ies_data[i].items[0]=temp32;

/* Longitude of CERES FOV at TOA */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Long_TOA=temp32;
	  ies_data[i].items[1]=temp32;

/* Colatitude of CERES FOV at Surface */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Colat_Surface=temp32;
	  ies_data[i].items[2]=temp32;

/* Longitude of CERES FOV at Surface */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Long_Surface=temp32;
	  ies_data[i].items[3]=temp32;

/* CERES Viewing Zenith at TOA */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.View_Zenith_Angle=temp32; 
	  ies_data[i].items[4]=temp32;

/* CERES Solar Zenith at TOA */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Solar_Zenith_Angle=temp32;
	  ies_data[i].items[5]=temp32;

/* CERES Relative Azimuth at TOA */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Rel_Az_Angle=temp32;
	  ies_data[i].items[6]=temp32;

/* CERES Viewing Azimuth at TOA wrt North */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Az_View_Angle_North=temp32;
	  ies_data[i].items[7]=temp32;

/* Cross-track Angle of CERES FOV at TOA */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.CrossTrack_Angle=temp32;
	  ies_data[i].items[8]=temp32;

/* Along-track Angle of CERES FOV at TOA */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Along_Track_Angle=temp32;
	  ies_data[i].items[9]=temp32;

/* Cone Angle of CERES FOV at Satellite */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Cone_Angle=temp32;
	  ies_data[i].items[10]=temp32;

/* Clock Angle of CERES FOV at Satellite wrt Inertial Velocity */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Clock_Angle=temp32;
	  ies_data[i].items[11]=temp32;

/* Rate of Change of Cone Angle */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Cone_Angle_Rate=temp32;
	  ies_data[i].items[12]=temp32;

/* Rate of Change of Clock Angle */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Clock_Angle_Rate=temp32;
	  ies_data[i].items[13]=temp32;

/* X Component of Satellite Inertial Velocity */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp64, buffer_ptr, sizeof(float64)); 
	  ies_record.Sat_Vel_X=temp64;
	  ies_data[i].items[14]=(float32)temp64;

/* Y Component of Satellite Inertial Velocity */

	  buffer_ptr=buffer_ptr+sizeof(float64); 
	  memcpy((char *)&temp64, buffer_ptr, sizeof(float64)); 
	  ies_record.Sat_Vel_Y=temp64;
	  ies_data[i].items[15]=(float32)temp64;

/* Z Component of Satellite Inertial Velocity */

	  buffer_ptr=buffer_ptr+sizeof(float64); 
	  memcpy((char *)&temp64, buffer_ptr, sizeof(float64)); 
	  ies_record.Sat_Vel_Z=temp64;
	  ies_data[i].items[16]=(float32)temp64;

/* Radius of Satellite from Center of Earth at Observation */

	  buffer_ptr=buffer_ptr+sizeof(float64); 
	  memcpy((char *)&temp64, buffer_ptr, sizeof(float64)); 
	  ies_record.Radius_From_Earth=temp64;
	  ies_data[i].items[17]=(float32)temp64;

/* CERES TOT Filtered Radiance Upwards */

	  buffer_ptr=buffer_ptr+sizeof(float64); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
/*        printf("Total=%f\n",temp32); */
	  ies_record.Tot_Filt_Radiance=temp32;
	  ies_data[i].items[18]=temp32;

/* CERES SW Filtered Radiance Upwards */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.SW_Filt_Radiance=temp32;
	  ies_data[i].items[19]=temp32;

/* CERES WN Filtered Radiance, Upwards */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Win_Filt_Radiance=temp32;
	  ies_data[i].items[20]=temp32;

/* Colatitude of Satellite at Observation */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Colat_Sat=temp32;
	  ies_data[i].items[21]=temp32;

/* Longitude of Satellite at Observation */
	
	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Long_Sat=temp32;
	  ies_data[i].items[22]=temp32;

/* Colatitude of Sun at Observation */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Colat_Sun=temp32;
	  ies_data[i].items[23]=temp32;

/* Longitude of Sun at Observation */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Long_Sun=temp32;
	  ies_data[i].items[24]=temp32;

/* Earth Sun Distance */
/*
	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp32, buffer_ptr, sizeof(float32)); 
	  ies_record.Earth_Sun_Dist=temp32;
	  ies_data[i].items[25]=temp32;
*/

/* Scan Sample Number */

	  buffer_ptr=buffer_ptr+sizeof(float32); 
	  memcpy((char *)&temp16, buffer_ptr, sizeof(uint16)); 
	  ies_record.Sample_Number=(float32)temp16;
	  ies_data[i].items[25]=(float32)temp16;

/*        fprintf(stdout,"\tSample Number: %u\n", temp16); */

/* Packet Number */

	  buffer_ptr=buffer_ptr+sizeof(uint16); 
	  memcpy((char *)&temp16, buffer_ptr, sizeof(temp16)); 
	  ies_record.ScanLine_Number=(float32)temp16;
	  ies_data[i].items[26]=(float32)temp16;

/*        fprintf(stdout,"\tScanline Number: %u\n", temp16); */

/* Time of Observation */
/* Need to extract Observation Time Better */

	  buffer_ptr=buffer_ptr+sizeof(uint16); 
	  memcpy((char *)&temp64, buffer_ptr, sizeof(float64)); 
	  ies_record.Observation_Time=(float32)temp64;
	  ies_data[i].items[27]=(float32)temp64;

/*        fprintf(stdout,"\tObservation_Time: %d\n", temp64); */

/* IES Quality Flags */

	  buffer_ptr=buffer_ptr+sizeof(float64); 
	  memcpy((char *)&utemp32, buffer_ptr, sizeof(utemp32)); 
	  ies_record.Quality_Flags=(float32)utemp32;
	  ies_data[i].items[28]=(float32)utemp32;

/* Absolute Packet Number */

	  buffer_ptr=buffer_ptr+sizeof(utemp32); 
	  memcpy((char *)&utemp32, buffer_ptr, sizeof(utemp32)); 
	  ies_record.Absolute_ScanLine_Number=(float32)utemp32;
	  ies_data[i].items[29]=(float32)utemp32;

/*        print_fields(ies_record);     */
/*        buffer_ptr=buffer+ (IES_RECORD_SIZE*(i+1)); */
	  buffer_ptr=buffer+ (record_size*(i+1));

/*        fprintf(stdout,"\tQuality Flags: %u\n", utemp32); */
       }         

/* Done processing */
       if (buffer != NULL) free(buffer);
       result=0;
/*
** Now print a line to seperate the output from this function and others
*/
   /* 
   fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
   fprintf(stdout,"Leaving C routine with result=%i\n",result);
   */

   return(result);
}

/*
---------------------------------------------------------------
	get_sds_dims
---------------------------------------------------------------
*/
char    *get_sds_dims(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	char            dummy_names[4096];
	IES_RECORD_TYPE ies_record; 
	int32           dimsizes[8];
	int32           file_id;
	int             i,j;
	int32           index;
	int32           nattrs;
	int32           ndatasets;
	int32           nglobal_attr;
	int32           number_type;
	int32           number_of_fields;
	int             *field_quantity;
	int32           *output_dimensions;
	int32           *output_rank;
	int32           *output_type;
	int32           rank;
	char            result[8196];
	int32           sd_id;
	int32           sds_id;
	char            sds_name[256];
	int32           size_of_data;
	int             size_of_single_record;
	int32           status;
	STRING          *data_name;     
	STRING          *filename;      /* Pointer to IDL string structure      */

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			data_name,
			rank,
			dimensions
	*/
	if(argc != 5)
	{
		fprintf(stderr,"get_sds_dims: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name= (STRING *) argv[1];
	output_rank=(int32 *) argv[2];
	output_dimensions=(int32 *)argv[3];
	output_type=(int32 *)argv[4];

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_sds_dims\n");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	/* fprintf(stdout, "\tRank: \t%i\r\n", *output_rank );  */

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/

	/* First Look at the SDS */
	sd_id=SDstart(filename->s,DFACC_RDONLY);
	index=SDnametoindex(sd_id,data_name->s);
	/* printf("in get_sds_dims, index = %d\n", index); */
	sds_id=SDselect(sd_id,index);
	status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);
	*output_rank=rank;
	*output_type=number_type;
	fprintf(stdout, "\tRank: \t%i\r\n", *output_rank );
	fprintf(stdout, "\tType: \t%i\r\n", *output_type );
	fprintf(stdout, "\tAttribute: \t%i\r\n", nattrs);
	for (i=0; i<rank; i++)
	{
		output_dimensions[i]=dimsizes[i];
		fprintf(stdout, "\tDimensions: \t%i\r\n", output_dimensions[i] );
	}
	status=SDendaccess(sds_id);
	status=SDend(sd_id);

   return(result);
}

/*
---------------------------------------------------------------
	get_sds_attr_name
---------------------------------------------------------------
*/
char    *get_sds_attr_name(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           dimsizes[8];
	int             i,j;
	int32           index;
	int32           nattrs;
	int32           number_type;
	int32           rank;
	char            result[8196];
	int32           sd_id;
	int32           sds_id;
	char            sds_name[256];
	char            attr_name[256];
	int32           data_type;
	int32           count;
	int32           status;
	char            *buffer;
	char            *loc;
	STRING          *data_name;     
	STRING          *filename;      /* Pointer to IDL string structure      */

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			data_name,
	*/
	if(argc != 2)
	{
		fprintf(stderr,"get_sds_attr_name: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name= (STRING *) argv[1];

	result[0] = '\0';
	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_sds_attr_name\n");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	fprintf(stdout, "\tSDS name: \t%s\r\n", data_name->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/

	/* First Look at the SDS */
	sd_id=SDstart(filename->s,DFACC_RDONLY);
	index=SDnametoindex(sd_id,data_name->s);
	sds_id=SDselect(sd_id,index);
	status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);
	if (status == -1)
	{
	   strcpy(result, "BAD");
	   return(result);
	}

	/* printf("nattrs = %d\n", nattrs); */
	if(nattrs > dimsizes[rank-1])
	{
	   j = 0;
	   for (i=0; i<nattrs; i++)
	   {
	      status = SDattrinfo(sds_id, i, attr_name, &data_type, &count);
	      if(status == -1)
	      {
		strcpy(result, "BAD");
		return(result);
	      }
	      else
	      {
		loc = strstr(attr_name, "long_name");
		if(loc == NULL) continue;
		buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
		if(buffer == NULL)
		{
			strcpy(result, "BAD");
			free(buffer);
			return(result);
		}
		*buffer = NULL;
		status = SDreadattr(sds_id, i, buffer);
		if(status == -1)
		{
			fprintf(stderr, "Error to read long_name\n");
			free(buffer);
			continue;
		}
		*(buffer+(count*DFKNTsize(data_type))) = NULL;
		if (j == 0)
		   strcpy(result, buffer);
		else
		{
		   strcat(result, "|");
		   strcat(result, buffer);
		}
		j++;
		free(buffer);
	      }
	   }
	}
	else
	{
	  for (i=0; i<nattrs; i++)
	  {
	     status = SDattrinfo(sds_id, i, attr_name, &data_type, &count);
	     if(status == -1)
	     {
		strcpy(result, "BAD");
		return(result);
	     }
	     else
	     {
		loc = strchr(attr_name, '-');
		if(loc != NULL) *loc = NULL;
		if(i == 0)
		   strcpy(result, attr_name);
		else
		{
		   strcat(result, "|");
		   strcat(result, attr_name);
		}
	     }
	  }
	}
	status=SDendaccess(sds_id);
	status=SDend(sd_id);

   return(result);
}

/*
---------------------------------------------------------------
	get_sds_attr_value
---------------------------------------------------------------
*/
char    *get_sds_attr_value(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           dimsizes[8];
	int             i,j;
	int32           index;
	int32           nattrs;
	int32           number_type;
	int32           rank;
	char            result[8196];
	int32           sd_id;
	int32           sds_id;
	char            sds_name[256];
	char            attr_name[256];
	int32           data_type;
	int32           count;
	int32           status;
	char            *buffer;
	char            *loc;
	STRING          *data_name;     
	STRING          *attr_type;     
	STRING          *filename;      /* Pointer to IDL string structure      */

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			data_name,
			attr_type
	*/
	if(argc != 3)
	{
		fprintf(stderr,"get_sds_attr_value: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name= (STRING *) argv[1];
	attr_type = (STRING *) argv[2];

	result[0] = '\0';
	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_sds_attr_value\n");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	fprintf(stdout, "\tSDS name: \t%s\r\n", data_name->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/

	/* First Look at the SDS */
	sd_id=SDstart(filename->s,DFACC_RDONLY);
	index=SDnametoindex(sd_id,data_name->s);
	/* printf("in get_sds_attr_value, index = %d\n", index); */
	sds_id=SDselect(sd_id,index);
	status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);
	if (status == -1)
	{
	   strcpy(result, "BAD");
	   return(result);
	}

/*      printf("attribute type = %s\n", attr_type->s); */
/*      printf("nattrs = %d\n", nattrs); */
	j = 0;
	for (i=0; i<nattrs; i++)
	{
	   status = SDattrinfo(sds_id, i, attr_name, &data_type, &count);
	   if(status == -1)
	   {
		strcpy(result, "BAD");
		return(result);
	   }
	   else
	   {
		/* printf("attr_name = %s.\n", attr_name); */
		loc = strstr(attr_name, attr_type->s);
		if(loc == NULL) continue;
		buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
		if(buffer == NULL)
		{
			strcpy(result, "BAD");
			free(buffer);
			return(result);
		}
		*buffer = NULL;
		status = SDreadattr(sds_id, i, buffer);
		if(status == -1)
		{
			fprintf(stderr, "Error to read long_name\n");
			free(buffer);
			continue;
		}
		*(buffer+(count*DFKNTsize(data_type))) = NULL;
		if (j == 0)
		   strcpy(result, buffer);
		else
		{
		   strcat(result, "|");
		   strcat(result, buffer);
		}
/*              printf("buffer = %s\n", buffer); */
		j++;
		free(buffer);
	   }
	}
	status=SDendaccess(sds_id);
	status=SDend(sd_id);

   return(result);
}

/*
---------------------------------------------------------------
	get_vdata_attr_value
---------------------------------------------------------------
*/

char *get_vdata_attr_value(argc, argv)
	int argc;
	void *argv[];
{
	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           current_vid;
	int32           current_vref;
	STRING          *data_name;
	int32           file_id;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int             i, j;
	char            result[256];
	int32           status;
	char            *attr_name;
	int32           data_type;
	int32           data_size;
	int32           data_order;
	int32           num_fields;
	char            *loc;
	uint8           *buffer;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	printf("get_vdata_attr_value was called...\n");
	/* printf("argc = %d\n", argc); */
	if(argc != 2)
	{
		fprintf(stderr,"get_vdata_attr_value: Incorrect number of arguments\r\n");
		strcpy(result, "BAD");
		return(result);  /* Signal an error */
	}
	/*
	** Cast the pointer in argv to the pointer variables
	*/
	/* Changed pointer casts to HDF types.  Kevin McIntire, 2/13/97 */
	filename= (STRING *) argv[0];
	data_name=(STRING *) argv[1];

	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_vdata_attr_value");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: %s\r\n", filename->s );
	fprintf(stdout, "\tDataname: %s\r\n", data_name->s );

	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");

	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id!=-1)
	{
		Vstart(file_id);
		current_vref=VSfind(file_id, data_name->s);
		if (current_vref)
		{
		   current_vid=VSattach(file_id,current_vref,"r");
		   if (current_vid!=-1)
		   {
			status = VSisattr(current_vid);
			if(status == 0)
			{
				strcpy(result, "BAD");
			}
			else
			{
				num_fields = VFnfields(current_vid);
				for(i = 0; i < num_fields; i++)
				{       
					strcpy(result, "BAD");
					attr_name = VFfieldname(current_vid, i);
					if(attr_name == NULL) continue;
					/* printf("attr_name = %s\n", attr_name); */
					data_type = VFfieldtype(current_vid, i);
					if(data_type == -1) continue;
					/* printf("data_type = %d\n", data_type); */
					data_order = VFfieldorder(current_vid, i);
					if(data_order == -1) continue;
					/* printf("data_order = %d\n", data_order); */
					data_size = VFfieldesize(current_vid, i);
					if(data_size == -1) continue;
					/* printf("data_size = %d\n", data_size); */
					status = VSsetfields(current_vid, attr_name);
					if(status == -1) continue;
					buffer = (uint8 *)malloc(data_size+1);
					if(buffer == NULL) continue;
					status = VSread(current_vid, buffer, 1, FULL_INTERLACE);
					if(status == -1) continue;
					result[0] = '\0';
					*(buffer + data_size) = NULL;
					loc = strstr((char *)buffer, "REGIONAL");
					if(loc != NULL)
					{
						loc = loc + 8;
						sscanf(loc, "%s", result);
						free(buffer);
						break;
					}

					loc = strstr((char *)buffer, "ZONAL");
					if(loc != NULL)
					{
						loc = loc + 5;
						sscanf(loc, "%s", result);
						free(buffer);
						break;
					}

					loc = strstr((char *)buffer, "GLOBAL");
					if(loc != NULL)
					{
						loc = loc + 7;
						sscanf(loc, "%s", result);
						free(buffer);
						break;
					}
					strcpy(result, "BAD");
					free(buffer);
				}
					/* printf("result = %s\n", result); */
			}
			VSdetach(current_vid);
			Vend(file_id);
			Hclose(file_id);
		   }
		   else
		   {
			fprintf(stdout,"\tCurrent Vid is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			strcpy(result, "BAD");
		   }            
		}
		else
		{
			fprintf(stdout,"\tCurrent Vref is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			strcpy(result, "BAD");
		}
	}
	else
	{
		strcpy(result, "BAD");
	}
	return(result);
}

/*
---------------------------------------------------------------
	list_sds
---------------------------------------------------------------
*/
char    *list_sds(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	char            dummy_names[4096];
	IES_RECORD_TYPE ies_record; 
	int32           current_vid;
	int32           current_vref;
	int32           dimsizes[8];
	char            field_names[4096];
	int32           file_id;
	int             i,j;
	int32           nattrs;
	int32           ndatasets;
	int32           nglobal_attr;
	int32           number_type;
	int32           number_of_fields;
	int32           *field_quantity;
	int32           rank;
	/* char         result[8196]; */
	char            result[65536];
	int32           sd_id;
	int32           sds_id;
	char            sds_name[256];
	int32           size_of_data;
	int             size_of_single_record;
	int32           status;
	int             default_name;
	int32           data_type;
	int32           count;
	char            attr_name[256];
	char            *loc;
	char            *buffer;
	STRING          *data_name;     
	STRING          *filename;      /* Pointer to IDL string structure      */
	char            *the_fields[100];


	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			number of fields
	*/
	if(argc != 2)
	{
		fprintf(stderr,"list_sds: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	field_quantity=(int32 *)argv[1];

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C list_sds");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/

	/* First Look at the SDS */
	sd_id=SDstart(filename->s,DFACC_RDONLY);
	status=SDfileinfo(sd_id,&ndatasets,&nglobal_attr);

	*field_quantity=ndatasets;
	for (i=0; i<ndatasets; i++)
	{
		default_name = 0;
		sds_id=SDselect(sd_id,i);
		status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);

		loc=strstr(sds_name, "fakeDim");
		if(loc != NULL)
			default_name = 1;
		else
		{
			loc=strstr(sds_name, "Data-Set");
			if(loc != NULL)
				default_name = 1;
		}

		if(default_name)
		{
			for (j=0; j<nattrs; j++)
			{
				status = SDattrinfo(sds_id, j, attr_name, &data_type, &count);
				if(status == -1)
					continue;
				else
				{
					loc = strstr(attr_name, "long_name");
					if(loc == NULL) continue;
					buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
					if(buffer == NULL) continue;
					*buffer = NULL;
					status = SDreadattr(sds_id, j, buffer);
					if(status == -1)
					{
						free(buffer);
						break;
					}
					*(buffer+(count*DFKNTsize(data_type))) = NULL;
					strcat(sds_name, ":");
					strncat(sds_name, buffer, 80);
					free(buffer);
					break;
				}
			}
		}
			
	
		if((strlen(result) + strlen(sds_name)) > 65535) break;
		if (i==0) 
		{
			strcpy(result,sds_name);
		}
		else
		{
			strcat(result,"|");
			strcat(result,sds_name);
		}


		/*      
		fprintf(stdout,"Name=%s\n",sds_name);
		fprintf(stdout,"Rank=%i\n",rank);
		for (j=0; j<rank; j++)
		{
			fprintf(stdout,"Dimsizes[%i]=%i\n",j,dimsizes[j]);
		}
		fprintf(stdout,"Number Type=%i\n",number_type);
		fprintf(stdout,"Attrs=%i\n",nattrs);
		*/
	
	}
	status=SDend(sd_id);

/*   printf("the length of result = %ld\n", strlen(result)); */
   return(result);
}

/*
---------------------------------------------------------------
	list_vdata
---------------------------------------------------------------
*/
char    *list_vdata(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	char            dummy_names[4096];
	IES_RECORD_TYPE ies_record; 
	int             continue_flag;
	int32           current_vid;
	int32           current_vref;
	int32           dimsizes[8];
	int32           file_id;
	int             i,j;
	int             check_meta;
	int32           nattrs;
	int32           ndatasets;
	int32           nglobal_attr;
	int32           number_type;
	int32           number_of_vdatas;
	int32           previous_vref;
	int32           rank;
	/* char         result[8196]; */
	char            result[15400];
	int32           sd_id;
	int32           sds_id;
	char            *loc;
	char            sds_name[256];
	int32           size_of_data;
	int             size_of_single_record;
	int32           status;
	char            class_name[80];
	char            vdata_name[256];
	STRING          *data_name;     
	int             *field_quantity;
	STRING          *filename;      /* Pointer to IDL string structure      */
	char            *the_fields[100];


	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			number of fields
	*/
	if(argc != 2)
	{
		fprintf(stderr,"list_vdata: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	field_quantity=(int *)argv[1];

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C list_vdata");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "Filename: %s\r\n", filename->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/


	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id!=-1)
	{
		Vstart(file_id);
		continue_flag=1;
		strcpy(result,"");
		previous_vref=-1;
		number_of_vdatas=0;
		while (continue_flag) 
		{
			current_vref=VSgetid(file_id,previous_vref);
			if (current_vref!=-1)
			{
				/* Loop through the vdatas in this file */

				current_vid=VSattach(file_id,current_vref,"r");
				if (current_vid!=-1)
				{
					previous_vref=current_vref;
					VSgetname(current_vid,vdata_name);
					/* fprintf(stdout,"vdata name=%s\n",vdata_name); */

					/* Omit the fakennn vdata sets */
					if (strncmp(vdata_name,"fake",4)!=0)
					{
						check_meta = 0;
						if (strcmp(vdata_name, "coremetadata") == 0) check_meta = 1;
						if (strcmp(vdata_name, "archivemetadata") == 0) check_meta = 1;
						if ((loc = strstr(vdata_name, "CoreMetadata")) != NULL) check_meta = 1;
						if ((loc = strstr(vdata_name, "ArchiveMetadata")) != NULL) check_meta = 1;
						if(check_meta || VSisattr(current_vid) == FALSE)
						{
							VSgetclass(current_vid, class_name);
							if (strncmp(class_name,"DimVal",6)!=0)
							{
								if(strlen(result) + strlen(vdata_name) > 15399) 
								{
									VSdetach(current_vid);
									break;
								}
								strcat(result,vdata_name);
								strcat(result,"|");
								number_of_vdatas++;
							}
						}
					}
					VSdetach(current_vid);
				}
				else
				{
					fprintf(stdout,"\tCurrent Vid is bad!\n");
					continue_flag=0;
				}
			}
			else
			{
				/* No more Vdatas in the file */
				/* fprintf(stdout,"\tCurrent Vref is bad!\n"); */
				continue_flag=0;
			}
		}       /* End while */
		Vend(file_id);
		Hclose(file_id);
	}
	else
	{
		/* Bad filename */
		fprintf(stdout,"\tBad Filename!\n");
		strcpy(result,"BAD");
	}
	*field_quantity=number_of_vdatas;

   return(result);
}

/*
---------------------------------------------------------------
	list_vdata_attr
---------------------------------------------------------------
*/
char    *list_vdata_attr(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	char            dummy_names[4096];
	IES_RECORD_TYPE ies_record; 
	int             continue_flag;
	int32           current_vid;
	int32           current_vref;
	int32           dimsizes[8];
	int32           file_id;
	int             i,j;
	int32           nattrs;
	int32           ndatasets;
	int32           nglobal_attr;
	int32           number_type;
	int32           number_of_vdatas;
	int32           previous_vref;
	int32           rank;
	char            result[8196];
	int32           sd_id;
	int32           sds_id;
	char            sds_name[256];
	int32           size_of_data;
	int             size_of_single_record;
	int32           status;
	char            vdata_name[256];
	STRING          *data_name;     
	int             *field_quantity;
	STRING          *filename;      /* Pointer to IDL string structure      */
	char            *the_fields[100];


	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			number of fields
	*/
	if(argc != 2)
	{
		fprintf(stderr,"list_vdata: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	field_quantity=(int *)argv[1];

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C list_vdata");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "Filename: %s\r\n", filename->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/


	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id!=-1)
	{
		Vstart(file_id);
		continue_flag=1;
		strcpy(result,"");
		previous_vref=-1;
		number_of_vdatas=0;
		while (continue_flag) 
		{
			current_vref=VSgetid(file_id,previous_vref);
			if (current_vref!=-1)
			{
				/* Loop through the vdatas in this file */

				current_vid=VSattach(file_id,current_vref,"r");
				if (current_vid!=-1)
				{
					previous_vref=current_vref;
					VSgetname(current_vid,vdata_name);
					/* fprintf(stdout,"vdata name=%s\n",vdata_name); */

					/* Omit the fakennn vdata sets */
					if (strncmp(vdata_name,"fake",4)!=0)
					{
						strcat(result,vdata_name);
						strcat(result,"|");
						number_of_vdatas++;
					}
					VSdetach(current_vid);
				}
				else
				{
					fprintf(stdout,"\tCurrent Vid is bad!\n");
					continue_flag=0;
				}
			}
			else
			{
				/* No more Vdatas in the file */
				/* fprintf(stdout,"\tCurrent Vref is bad!\n"); */
				continue_flag=0;
			}
		}       /* End while */
		Vend(file_id);
		Hclose(file_id);
	}
	else
	{
		/* Bad filename */
		fprintf(stdout,"\tBad Filename!\n");
		strcpy(result,"BAD");
	}
	*field_quantity=number_of_vdatas;

   return(result);
}

/*
---------------------------------------------------------------
	get_vdata_fields
---------------------------------------------------------------
*/
char    *get_vdata_fields(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	char            dummy_names[4096];
	int32           current_vid;
	int32           current_vref;
	char            field_names[4096];
	int32           file_id;
	int             i;
	int32           number_of_fields;
	int32             *field_quantity;
	char            result[4096];
	int32           size_of_data;
	int             size_of_single_record;
	int32           status;
	STRING          *data_name;     
	STRING          *filename;      /* Pointer to IDL string structure      */
	char            *the_fields[100];


	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			data_name,
			ending_sample,
			bds_data
	*/
	if(argc != 3)
	{
		fprintf(stderr,"get_vdata_fields: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name= (STRING *) argv[1];
	field_quantity=(int32 *)argv[2];

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_vdata_fields");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	fprintf(stdout, "\tData Name: \t%s\r\n", data_name->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/
	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	/* fprintf(stdout,"\tThe file id is: \t%i\n"); */

	if (file_id!=-1)
	{
		Vstart(file_id);
		/* current_vref=VSgetid(file_id, -1); */

		current_vref=VSfind(file_id, data_name->s);
/*              if (current_vref!=-1)  */
		if (current_vref)
		{
		   current_vid=VSattach(file_id,current_vref,"r");
		   if (current_vid!=-1)
		   {
			number_of_fields=VSgetfields(current_vid, 
					dummy_names);
			*field_quantity=number_of_fields;
			fprintf(stdout,"\tNumber of Fields = \t%i\n",number_of_fields);
			set_field_names(field_names,number_of_fields,current_vid); 
			/*
			for (i=0; i<number_of_fields; i++) 
			{
				the_fields[i]=VFfieldname(current_vid,i);
				fprintf(stdout,"\tFields[%i]=%s\n",i,the_fields[i]);
			}
			*/

			VSdetach(current_vid);
			Vend(file_id);
			Hclose(file_id);
			strcpy(result,field_names);     
		   }
		   else
		   {
			fprintf(stdout,"\tCurrent Vid is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			strcpy(result,"BAD");
		   }
		}
		else
		{
			fprintf(stdout,"\tCurrent Vref is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			strcpy(result,"BAD");
		}
	}
	else
	{
		strcpy(result,"BAD");
	}

   return(result);
}

		   
/*
---------------------------------------------------------------
	read_v_bds
---------------------------------------------------------------
*/
long read_bds(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	unsigned char   *buffer;
	unsigned char   *buffer_ptr;
	int32           buffer_size;
	char            dummy_names[4096];
	IES_RECORD_TYPE ies_record; 
	int32           current_vid;
	int32           current_vref;
	int             *ending_sample;
	char            field_names[4096];
	int32           file_id;
	int             len;
	uint16          temp16;
	float32         temp32;
	float64         temp64;
	int             i;
	int32           number_of_fields;
	long            result;
	int32           size_of_data;
	int             size_of_single_record;
	int32           status;
	STRING          *data_name;     
	STRING          *filename;      /* Pointer to IDL string structure      */
	int             *starting_sample;
	int             *this_stuff;
	BDS_ROW         *bds_data;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			data_name,
			ending_sample,
			bds_data
	*/
	if(argc != 4)
	{
		fprintf(stderr,"read_bds: Incorrect number of arguments\r\n");
		return(-1);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name= (STRING *) argv[1];
	ending_sample=(int *)argv[2];
	/* Changed casting to be more correct & to remove annoying compiler warnings.  Kevin McIntire, 2/13/97 */
	/*bds_data=(float *)argv[3];*/
	bds_data = (BDS_ROW *) argv[3];

	size_of_data=(int32)(*ending_sample);
	/* size_of_data=(int32)(*ending_sample)-(*starting_sample); */

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C read_bds");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	fprintf(stdout, "\tData Name: \t%s\r\n", data_name->s );
	fprintf(stdout, "\tEnding: \t%i\r\n", *ending_sample);

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/
	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	/* fprintf(stdout,"\tThe file id is: \t%i\n"); */

	if (file_id!=-1)
	{
		Vstart(file_id);
		/* current_vref=VSgetid(file_id, -1); */

		current_vref=VSfind(file_id, data_name->s);
/*              if (current_vref!=-1)  */
		if (current_vref)
		{
		   current_vid=VSattach(file_id,current_vref,"r");
		   if (current_vid!=-1)
		   {
			number_of_fields=VSgetfields(current_vid, 
					dummy_names);
			fprintf(stdout,"\tNumber of Fields = \t%i\n",number_of_fields);
			set_field_names(field_names,number_of_fields,current_vid);
			fprintf(stdout,"\tFields=%s\n",field_names);
			
			/* get the size_of_single_record from the HDF vsize field INFO */
			size_of_single_record=VSsizeof(current_vid,field_names);
			if (size_of_single_record==-1)
			{
				fprintf(stdout,"\tVSsize of failed!\n");
				exit(-1);
			}
			else
			{
				fprintf(stdout,"\tVSsize = \t%i\n",size_of_single_record);
			}
			
			if (number_of_fields!=-1)
			{
			   status=VSseek(current_vid,0);
			   if (status==-1)
			   {
				fprintf(stdout,"\tEven VSseek fails!\n");
				exit(-1);
			   }
			   status=VSsetfields(current_vid, field_names);
			   /* fprintf(stdout,"\tStatus of VSsetfields: \t%i\n",
						status); */
			   if (status!=-1)
			   {

				/* Get size */
				buffer_size=size_of_single_record*size_of_data; 

				/* if (buffer_size>0 && status!=-1) */
				if (buffer_size>0 )
				{
				      buffer=malloc(buffer_size);
				      status=VSread(current_vid,buffer,
						size_of_data,FULL_INTERLACE);
				      if (status!=-1)
				      {
					   /* Uncram/Unpack the buffer */ 
					   buffer_ptr=buffer;
					   /*
					   for (i=0; i<size_of_data; i++)
					   {
					   }
					   */
					   fprintf(stdout,"VS Read is ok");
					   /* Done processing */
					   free(buffer);
					   VSdetach(current_vid);
					   Vend(file_id);
					   Hclose(file_id);
					   result=0;

				      }
				      else
				      {
					   fprintf(stdout,"\tVSread is bad!\n");
					   free(buffer);
					   VSdetach(current_vid);
					   Vend(file_id);
					   Hclose(file_id);
					   result=-1;
				      }
				}
				else
				{
				   fprintf(stdout,"\tBuffer size is bad!\n");
				   VSdetach(current_vid);
				   Vend(file_id);
				   Hclose(file_id);
				   result=-1;
				}
			   }
			   else
			   {
				fprintf(stdout,"\tVSsetfields is bad!\n");
				Vend(file_id);
				Hclose(file_id);
				result=-1;
			   }
			}
			else
			{
				fprintf(stdout,"\tNumber of fields is bad!\n");
				Vend(file_id);
				Hclose(file_id);
				result=-1;
			}
		   }
		   else
		   {
			fprintf(stdout,"\tCurrent Vid is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			result=-1;
		   }            
		}
		else
		{
			fprintf(stdout,"\tCurrent Vref is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			result=-1;
		}
	}
	else
	{
		result=-1;
	}

   return(result);
}

/*
---------------------------------------------------------------
	read_vdata
---------------------------------------------------------------
*/
long read_vdata(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	/* Changed buffer & ptr defs to be consistent with HDF.  Kevin McIntire, 2/13/97*/
	uint8           *buffer;
	uint8           *buffer_ptr;
	int32           buffer_size;
	char            dummy_names[4096];
	int32           current_order;
	int32           current_vid;
	int32           current_vref;
	int32           *ending_record;
	int             *field_count;
	STRING          *field_list;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int32           file_id;
	char            find_name[64];
	int             *increment;
	int             len;
	/* Following dec was formerly int *output_data;.  I changed it to void *output_data. */
	/* IMPORTANT to cast it to correct data type on assignment.  An int * declaration would */
	/* not have allowed all data types to be inserted into it.  See the case statement below */
	/* which switches on the data type.  Kevin McIntire, 2/13/97 */
	void            *output_data;

	int32           number_of_records;
	int32           *number_order;
	int32           *number_types;
	int32           record_size;
	int32           size_of_single_record;
	/* Expanded number of data types covered.  Removed unneeded pointer delcarations.  Kevin McIntire, 2/13/97 */
	char8           tempC8;
	uchar8          tempUC8;
	int             temp_char;
	int8            tempI8;
	uint8           tempU8;
	int16           tempI16;
	uint16          tempU16;
	int32           tempI32;
	uint32          tempU32;
	float32         tempF32;
	float64         tempF64;
	int             i,j,k;
	int32           number_of_fields;
	long            result;
	int32           status;
	int32           *starting_record;
	int             this_index;
	int             *this_stuff;
	int32           type_byte_size;
	STRING          *vdata_name;    /* Pointer to IDL string structure      */

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
	-------------------------------------------------------------------------
		Parameters:
			filename,
			vdata_name,
			field_count,
			starting_record,
			ending_record,
			increment,
			field_list,
			number_types,
			number_order,
			output_data     

		** Insure that the correct number of arguments were passed in 
	-------------------------------------------------------------------------
	*/
	if(argc != 10)
	{
		fprintf(stderr,"read_vdata: Incorrect number of arguments\r\n");
		return(-1);  /* Signal an error */
	}

	/*
	----------------------------------------------------
		Transform the parameters into C variables       
	----------------------------------------------------
	*/
	filename        =(STRING *)argv[0];
	vdata_name      =(STRING *)argv[1];
	field_count     =(int *)argv[2];
	starting_record =(int32 *)argv[3];
	ending_record   =(int32 *)argv[4];
	increment       =(int *)argv[5];
	field_list      =(STRING *)argv[6];     
	number_types    =(int32 *)argv[7];      /* Array */
	number_order    =(int32 *)argv[8];      /* Array */
	output_data     =(void *) argv[9];   

	/*
	----------------------------------------
		Print a header with the
		parameters.
	----------------------------------------
	*/
	
	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C read_vdata");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: %s\r\n", filename->s );
	fprintf(stdout, "\tVdata Name: %s\r\n", vdata_name->s );
	fprintf(stdout, "\tField Count: %i\r\n", *field_count);
	fprintf(stdout, "\tStart: %i\r\n", *starting_record);
	fprintf(stdout, "\tEnd: %i\r\n", *ending_record);
	fprintf(stdout, "\tIncrement: %i\r\n", *increment);
	fprintf(stdout, "\tFields: %s\r\n", field_list->s );
	for (i=0;i<*field_count; i++)
	{
		fprintf(stdout, "\tTypes: %i\r\n", *number_types);
		fprintf(stdout, "\tOrder: %i\r\n", *number_order);
	}
	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");

	/*
	----------------------------------------
		read_vdata Processing
	----------------------------------------
	*/

	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id!=-1)
	{
		Vstart(file_id);
		/* current_vref=VSgetid(file_id, -1); */

		current_vref=VSfind(file_id, vdata_name->s);
/*              if (current_vref!=-1)  */
		if (current_vref)
		{
		   current_vid=VSattach(file_id,current_vref,"r");
		   if (current_vid!=-1)
		   {
			if (number_of_fields!=-1)
			{
			   status=VSseek(current_vid,0);
			   if (status==-1)
			   {
				fprintf(stdout,"\tEven VSseek fails!\n");
				exit(-1);
			   }
			   status=VSsetfields(current_vid, field_list->s);
			   if (status!=-1)
			   {
				/* get the size_of_single_record from the HDF vsize field INFO */
				size_of_single_record=VSsizeof(current_vid,field_list->s);
				/*
				number_of_records=((*ending_record-(*starting_record)+1) / 
							(*increment));
				*/
				number_of_records=*ending_record + 1;
				buffer_size=number_of_records * size_of_single_record;
				if ((buffer_size>0) && (size_of_single_record>0))
				{
				      /* Added check on malloc return.  Cast result to HDF type.  Kevin McIntire, 2/13/97 */
				      if ( (buffer=(uint8 *) malloc(buffer_size)) == NULL)
				      {
					   printf("OUT OF MEMORY FOR VDATA BUFFER, %d BYTES\n",buffer_size);
					   return(-1);
				      }
				      /* Cast pointer to buffer to HDF type.  The vdata must have FULL_INTERLACE to be */
				      /* read field by field, so it is hard-coded here.  Kevin McIntire, 2/13/97 */
				      status=VSread(current_vid,(uint8 *) buffer,
						number_of_records,FULL_INTERLACE); 
				      if (status!=-1)
				      {
					   buffer_ptr=buffer;
					   for (i=0; i<number_of_records; i++)
					   {
						for (k=0; k<*field_count; k++)  
						{       
						   current_order=number_order[k];

						   /* Set this way for IDL structure */
						   for (j=0; j<current_order; j++)
						   {
						      /* Account for the 4 bytes from nfields */
/*                                                    this_index=i * (current_order + 1) + j + 1;*/
						      this_index=i * (current_order) + j;

						      /* fprintf(stdout, "i=%i j=%i index=%i ",
								i,j,this_index);    */
						      /* Added more data type to make this routine more generic. */
						      /* Note that output_data is cast to the appropriate type each */
						      /* time data is assigned to it.  */
						      /* The data transer routines were changed for each type to make */
						      /* the code a little clearer.  Kevin McIntire, 2/13/97 */
						      switch (number_types[k]) 
						      {
							case DFNT_CHAR8:
								memcpy(&tempC8, buffer_ptr, sizeof(char8));
								((int8 *)output_data)[this_index] = (int8)tempC8;
								buffer_ptr += sizeof(char8);
								break;
							case DFNT_UCHAR8:
								memcpy(&tempUC8, buffer_ptr, sizeof(uchar8));
								((int8 *)output_data)[this_index] = (int8)tempUC8;
								buffer_ptr += sizeof(uchar8);
								break;
							case DFNT_INT8:
								memcpy(&tempI8, buffer_ptr, sizeof(int8));
								((int8 *) output_data)[this_index] = tempI8;
								buffer_ptr += sizeof(int8);
								break;
							case DFNT_UINT8:
								memcpy(&tempU8, buffer_ptr, sizeof(uint8));
								/* ((uint8 *) output_data)[this_index] = tempU8; */
								((int16 *) output_data)[this_index] = (int16)tempU8;
								buffer_ptr += sizeof(uint8);
								break;
							case DFNT_INT16:
								memcpy(&tempI16, buffer_ptr, sizeof(int16));
								((int16 *) output_data)[this_index] = tempI16;
								buffer_ptr += sizeof(int16);
								break;
							case DFNT_UINT16:
								memcpy(&tempU16, buffer_ptr, sizeof(uint16));
								/* ((uint16 *) output_data)[this_index] = tempU16; */
								((int32 *) output_data)[this_index] = (int32)tempU16;
								buffer_ptr += sizeof(uint16);
								break;
							case DFNT_INT32:
								memcpy(&tempI32, buffer_ptr, sizeof(int32));
								((int32 *) output_data)[this_index] = tempI32;
								buffer_ptr += sizeof(int32);
								break;
							case DFNT_UINT32:
								memcpy(&tempU32, buffer_ptr, sizeof(uint32));
								/* ((uint32 *) output_data)[this_index] = tempU32; */
								((float64 *) output_data)[this_index] = (float64) tempU32;
								buffer_ptr += sizeof(uint32);
								break;
							case DFNT_FLOAT32:
								memcpy(&tempF32, buffer_ptr, sizeof(float32));
								((float32 *) output_data)[this_index] = tempF32;
								buffer_ptr += sizeof(float32);
								break;
							case DFNT_FLOAT64:
								memcpy(&tempF64, buffer_ptr,sizeof(float64));
								((float64 *) output_data)[this_index]=tempF64;
								buffer_ptr += sizeof(float64);
								break;
							}
						 }
					      } 
					   }

					   /* Done processing */
					   /* Check pointer before freeing memory.  Kevin McIntire, 2/13/97 */
					   if (buffer != NULL) free(buffer);
					   VSdetach(current_vid);
					   Vend(file_id);
					   Hclose(file_id);
					   result=0;

				      }
				      else
				      {
					   fprintf(stdout,"\tVSread is bad!\n");
					   /* Check pointer before freeing memory.  Kevin McIntire, 2/13/97 */
					   if (buffer != NULL) free(buffer);
					   VSdetach(current_vid);
					   Vend(file_id);
					   Hclose(file_id);
					   result=-1;
				      }
				}
				else
				{
				   fprintf(stdout,"\tBuffer size is bad!\n");
				   VSdetach(current_vid);
				   Vend(file_id);
				   Hclose(file_id);
				   result=-1;
				}
			   }
			   else
			   {
				fprintf(stdout,"\tVSsetfields is bad!\n");
				Vend(file_id);
				Hclose(file_id);
				result=-1;
			   }
			}
			else
			{
				fprintf(stdout,"\tNumber of fields is bad!\n");
				Vend(file_id);
				Hclose(file_id);
				result=-1;
			}
		   }
		   else
		   {
			fprintf(stdout,"\tCurrent Vid is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			result=-1;
		   }            
		}
		else
		{
			fprintf(stdout,"\tCurrent Vref is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			result=-1;
		}
	}
	else
	{
		result=-1;
	}

/*
** Now print a line to seperate the output from this function and others
*/
   /* 
   fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
   fprintf(stdout,"Leaving C routine with result=%i\n",result);
   */

   return(result);
}

/*
---------------------------------------------------------------
	read_sds
---------------------------------------------------------------
*/
int32   read_sds(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           start[8];
	int32           edge[8];
	int32           stride[8];
	int             i,j,k;
	int32           index;
	int32           result;
	int32           sd_id;
	int32           sds_id;
	char            sds_name[256];
	int32           status;
	uint8           tempuint8;
	int8            tempint8;
	uint16          tempuint16;
	int16           tempint16;
	uint32          tempuint32;
	int32           tempint32;
	float32         tempfloat32;
	float64         tempfloat64;
	int             data_size;
	int32           number_data;
	int32           this_index;

	uint8           *data_buffer;
	uint8           *ptr_buffer;

	STRING          *data_name;     
	STRING          *filename;      /* Pointer to IDL string structure      */
	int32           *rank;
	int32           *number_type;
	int32           *dimsizes;
	int32           *starting_record;
	int32           *ending_record;
	int32           *increment;
	int32           *the_count;
	void            *sds_data;


	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			data_name,
			rank,
			dimensions
	*/
	if(argc != 10)
	{
		fprintf(stderr,"read_sds: Incorrect number of arguments\r\n");
		result = -1;
		return result;  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name= (STRING *) argv[1];
	rank = (int32 *)argv[2];
	number_type = (int32 *)argv[3];
	dimsizes = (int32 *)argv[4];
	starting_record= (int32 *) argv[5];
	ending_record=(int32 *)argv[6];
	increment= (int32 *)argv[7];
	the_count= (int32 *)argv[8];
	sds_data= (void *)argv[9];

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C read_sds\n");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	fprintf(stdout, "\tData_name: \t%s\r\n", data_name->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/

	/* First Look at the SDS */
	sd_id=SDstart(filename->s,DFACC_RDONLY);
	if(sd_id == -1)
	{
	   fprintf(stderr,"Unable to open BDS file for SDS.\n");
	   result = -1;
	   return result;
	}

	index=SDnametoindex(sd_id,data_name->s);
	/* printf("in read_sds, index = %d\n", index); */
	if(index == -1)
	{
	   fprintf(stderr,"Could not find %s SDS\n", data_name->s);
	   status = SDend(sd_id);
	   result = -1;
	   return result;
	}

	sds_id=SDselect(sd_id,index);
	if(sds_id == -1)
	{
	   fprintf(stderr,"Could not get sds_id for %s\n", data_name->s);
	   status = SDend(sd_id);
	   result = -1;
	   return result;
	}
/*
	status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);
	if(status == -1)
	{
	   fprintf(stderr,"Could not get information for %s\n", data_name->s);
	   status = SDendaccess(sds_id);
	   status = SDend(sd_id);
	   result = -1;
	   return result;
	}
*/

	number_data = 1;
	for (i=0; i<*rank; i++)
	{
		start[i] = starting_record[i];
		edge[i] = the_count[i];
		stride[i] = increment[i];
		number_data *= the_count[i];
	}

	switch(*number_type)
	{
	   case 5:
	   case 6:
	   case 20:
	   case 22:
	   case 24:
		{
		   status = SDreaddata(sds_id, start, stride, edge, (void *) sds_data);
		   if(status == -1)
		   {
		      fprintf(stderr, "ERROR for reading %s SDS.\n", data_name->s);
		      result = -1;
		   }
		   else
		   {
		      result = 0;
		   }
		   status = SDendaccess(sds_id);
		   status = SDend(sd_id);
		   return result;
		}
	   case 21:
		{
		   data_size = sizeof(uint8);
		   break;
		}
	   case 23:
		{
		   data_size = sizeof(uint16);
		   break;
		}
	  case 25:
		{
		   data_size = sizeof(uint32);
		   break;
		}
	   default:
		{
		   fprintf(stderr, "Unknow type for SDS: %d.\n", *number_type);
		   status = SDendaccess(sds_id);
		   status = SDend(sd_id);
		   result = -1;
		   return result;
		}
	}


	data_buffer = (uint8 *)malloc(number_data * data_size);
	if(data_buffer == NULL)
	{
	   fprintf(stderr, "Unable to allocate memory.\n");
	   status = SDendaccess(sds_id);
	   status = SDend(sd_id);
	   result = -1;
	   return result;
	}

	status = SDreaddata(sds_id, start, stride, edge, (void *) data_buffer);
	if(status == -1)
	{
	   fprintf(stderr, "ERROR for reading %s SDS.\n", data_name->s);
	   if(data_buffer != NULL) free(data_buffer);
	   status = SDendaccess(sds_id);
	   status = SDend(sd_id);
	   result = -1;
	   return result;
	}

	 ptr_buffer = data_buffer;

	 for(i = 0 ; i < number_data; i++ )
	 {
		switch(*number_type)
		{
		   case 21:
		   {
			memcpy(&tempuint8, ptr_buffer, data_size);
			((int16 *) sds_data)[i] = (int16)tempuint8;
			break;
		   }
		   case 23:
		   {
			memcpy(&tempuint16, ptr_buffer, data_size);
			((int32 *) sds_data)[i] = (int32)tempuint16;
			break;
		   }
		   case 25:
		   {
			memcpy(&tempuint32, ptr_buffer, data_size);
			((float64 *) sds_data)[i] = (float64)tempuint32;
			break;
		   }
		   default :
		   {
		      fprintf(stderr, "Unknow type for SDS: %d.\n", *number_type);
		      if(data_buffer != NULL) 
			free(data_buffer);
		      status = SDendaccess(sds_id);
		      status = SDend(sd_id);
		      result = -1;
		      return result;
		   }
		}
	    ptr_buffer += data_size;
	 }

	fprintf(stdout, "Completed reading SDS %s in read_sds.\n", data_name->s);
	if(data_buffer != NULL) 
	   free(data_buffer);

	status = SDendaccess(sds_id);
	status = SDend(sd_id);

	result = 0;

   return(result);
}

/*
---------------------------------------------------------------
	get_printer_name
---------------------------------------------------------------
*/
char    *get_printer_name(argc, argv)
	int argc;
	void *argv[];
{

    FILE *pname;
    int i, j;
    char *ptr_name, printer_name[1000];
    char buf[120];
    STRING *filename;

/*
	Input Parameters:
		filename
*/
if(argc != 1)
{
	fprintf(stderr,"get_printer_name: Incorrect number of arguments\r\n");
	strcpy(printer_name,"BAD");
	ptr_name = printer_name;
	return(ptr_name);  /* Signal an error */
}

/*
** Cast the pointer in argv to the pointer variables
*/
filename= (STRING *) argv[0];

/*    if((pname = fopen("/etc/printcap", "r")) == NULL) */
/*    if((pname = fopen("printer_name", "r")) == NULL) */
    if((pname = fopen(filename->s, "r")) == NULL)
    {
/*      fprintf(stderr, "/etc/printcap file not found.\n"); */
/*      fprintf(stderr, "printer_name file not found.\n"); */
      fprintf(stderr, "%s file not found.\n",filename->s);
      strcpy(printer_name, "BAD");
      ptr_name = printer_name;
      return (ptr_name);
   }

   i = 0;
   while( !feof(pname) )
   {
      fgets(buf, 119, pname);
      if( !isalnum(buf[0]) ) continue;
      for( j = 0; j < 120; j++)
      {
	if( !isalnum(buf[j]) ) break;
	printer_name[i++] = buf[j];
      }
      printer_name[i++] = ',';
      buf[0] = '\0';
   }

   printer_name[i-1] = '\0';

   ptr_name = printer_name;

  fclose(pname);

   return (ptr_name);
 
}

/*
---------------------------------------------------------------
	get_sds_dims_eos
---------------------------------------------------------------
*/
char    *get_sds_dims_eos(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	char            dummy_names[4096];
	IES_RECORD_TYPE ies_record; 
	int32           dimsizes[8];
	int32           file_id;
	int             i,j;
	int32           index;
	int32           first_index;
	int             num_match;
	int32           nattrs;
	int32           ndatasets;
	int32           nglobal_attr;
	int32           number_type;
	int32           number_of_fields;
	int             *field_quantity;
	int32           *output_dimensions;
	int32           *output_rank;
	int32           *output_type;
	int32           rank;
	char            result[8196];
	int32           sd_id;
	int32           sds_id;
	char            sds_name[256];
	int32           size_of_data;
	int             size_of_single_record;
	int32           status;
/*	int32		dim_id;
	char		dim_name[256];
	int32		dim_count;
	int32		dim_type;
	int32		dim_nattrs; */
	int32           *sds_ref;
	int32           *num_sds;
	STRING          *data_name;     
	STRING          *filename;      /* Pointer to IDL string structure      */

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			index,
			rank,
			dimensions
	*/
	if(argc != 7)
	{
		fprintf(stderr,"get_sds_dims_eos: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name= (STRING *) argv[1];
	num_sds=(int32 *)argv[2];
	sds_ref=(int32 *)argv[3];
	output_rank=(int32 *) argv[4];
	output_dimensions=(int32 *)argv[5];
	output_type=(int32 *)argv[6];

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_sds_dims_eos\n");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	fprintf(stdout, "\tSDS name: \t%s\r\n", data_name->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/

	/* First Look at the SDS */
	sd_id=SDstart(filename->s,DFACC_RDONLY);
	sds_id=SDselect(sd_id,*sds_ref);
	status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);
	if(strcmp(sds_name, data_name->s) != 0)
	{
	   printf("sds_name: %s is not equal to data_name->s: %s\n",sds_name,data_name->s);
	   index=SDnametoindex(sd_id,data_name->s);
	   if(index == -1)
	   {
		strcpy(result,"BAD");
		return(result);
	   }
	   if( *num_sds > 1)
	   {
		first_index = index;
		num_match = 1;
		status = SDfileinfo(sd_id,&ndatasets,&nglobal_attr);
		i = first_index + 1;
		while (num_match < *num_sds && i < ndatasets)
		{
			sds_id = SDselect(sd_id, i);
			status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);
			if( strcmp(sds_name, data_name->s) == 0) num_match++;
			if(num_match == *num_sds)
			{
				index = i;
				status = SDendaccess(sds_id);
				break;
			}
			else
			{
				status = SDendaccess(sds_id);
			}
			i++;
		}
		if( i == ndatasets ) index = first_index;
	   }
	   *sds_ref = index;
	   printf("in get_sds_dims_eos, index = %d\n", index);
	   sds_id=SDselect(sd_id,index);
	   status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);
	}
	/* fprintf(stdout,"sds_name = %s\n", sds_name); */
	*output_rank=rank;
	*output_type=number_type;
	fprintf(stdout, "\tRank: \t%i\r\n", *output_rank );
	fprintf(stdout, "\tType: \t%i\r\n", *output_type );
	fprintf(stdout, "\tAttribute: \t%i\r\n", nattrs);
	for (i=0; i<rank; i++)
	{
		output_dimensions[i]=dimsizes[i];
		fprintf(stdout, "\tDimensions: \t%i\r\n", output_dimensions[i] );
	}
/*
	for (i=0; i < rank; i++)
	{
		dim_id = SDgetdimid(sds_id, i);
		if(dim_id != -1)
		{
			status=SDdiminfo(dim_id, dim_name, &dim_count, &dim_type, &dim_nattrs);
			if(status != -1)
			{
				printf("dim_id= %d, dim_name = %s, dim_count = %d, dim_type = %d, dim_nattrs = %d\n", \
					dim_id, dim_name, dim_count, dim_type, dim_nattrs);
			}
		}
	}
*/
	status=SDendaccess(sds_id);
	status=SDend(sd_id);
	strcpy(result,"GOOD");

   return(result);
}

/*
---------------------------------------------------------------
	get_sds_attr_value_eos
---------------------------------------------------------------
*/
char    *get_sds_attr_value_eos(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           dimsizes[8];
	int             i,j;
	int32           *index;
	int32           nattrs;
	int32           number_type;
	int32           rank;
	char            result[8196];
	int32           sd_id;
	int32           sds_id;
	char            sds_name[256];
	char            attr_name[256];
	int32           data_type;
	int32           count;
	int32           status;
	char            *buffer;
	char            *loc;
	STRING          *data_name;     
	STRING          *attr_type;     
	STRING          *filename;      /* Pointer to IDL string structure      */

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			index,
			attr_type
	*/
	if(argc != 4)
	{
		fprintf(stderr,"get_sds_attr_value_eos: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name=(STRING *) argv[1];
	index= (int32 *) argv[2];
	attr_type = (STRING *) argv[3];

	result[0] = '\0';
	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_sds_attr_value_eos\n");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	fprintf(stdout, "\tSDS name: \t%s\r\n", data_name->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/

	/* First Look at the SDS */
	sd_id=SDstart(filename->s,DFACC_RDONLY);
	/* index=SDnametoindex(sd_id,data_name->s); */
	printf("in get_sds_attr_value_eos, index = %d\n", *index);
	sds_id=SDselect(sd_id,*index);
	status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);
	if (status == -1)
	{
	   strcpy(result, "BAD");
	   return(result);
	}

/*      printf("sds_name = %s\n", sds_name); */
/*      printf("attribute type = %s\n", attr_type->s); */
/*      printf("nattrs = %d\n", nattrs); */
	j = 0;
	for (i=0; i<nattrs; i++)
	{
	   status = SDattrinfo(sds_id, i, attr_name, &data_type, &count);
	   if(status == -1)
	   {
		strcpy(result, "BAD");
		return(result);
	   }
	   else
	   {
		/* printf("attr_name = %s.\n", attr_name); */
		loc = strstr(attr_name, attr_type->s);
		if(loc == NULL) continue;
		buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
		if(buffer == NULL)
		{
			strcpy(result, "BAD");
			free(buffer);
			return(result);
		}
		*buffer = NULL;
		status = SDreadattr(sds_id, i, buffer);
		if(status == -1)
		{
			fprintf(stderr, "Error to read long_name\n");
			free(buffer);
			continue;
		}
		*(buffer+(count*DFKNTsize(data_type))) = NULL;
		if (j == 0)
		   strcpy(result, buffer);
		else
		{
		   strcat(result, "|");
		   strcat(result, buffer);
		}
/*              printf("buffer = %s\n", buffer); */
		j++;
		free(buffer);
	   }
	}
	status=SDendaccess(sds_id);
	status=SDend(sd_id);

   return(result);
}

/*
---------------------------------------------------------------
	read_sds_eos
---------------------------------------------------------------
*/
int32   read_sds_eos(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           start[8];
	int32           edge[8];
	int32           stride[8];
	int             i,j,k;
	int32           *index;
	int32           result;
	int32           sd_id;
	int32           sds_id;
	char            sds_name[256];
	int32           status;
	uint8           tempuint8;
	int8            tempint8;
	uint16          tempuint16;
	int16           tempint16;
	uint32          tempuint32;
	int32           tempint32;
	float32         tempfloat32;
	float64         tempfloat64;
	int             data_size;
	int32           number_data;
	int32           this_index;

	uint8           *data_buffer;
	uint8           *ptr_buffer;

	STRING          *data_name;     
	STRING          *filename;      /* Pointer to IDL string structure      */
	int32           *rank;
	int32           *number_type;
	int32           *dimsizes;
	int32           *starting_record;
	int32           *ending_record;
	int32           *increment;
	int32           *the_count;
	void            *sds_data;


	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			data_name,
			rank,
			dimensions
	*/
	if(argc != 11)
	{
		fprintf(stderr,"read_sds_eos: Incorrect number of arguments\r\n");
		result = -1;
		return result;  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name = (STRING *)argv[1];
	index= (int32 *) argv[2];
	rank = (int32 *)argv[3];
	number_type = (int32 *)argv[4];
	dimsizes = (int32 *)argv[5];
	starting_record= (int32 *) argv[6];
	ending_record=(int32 *)argv[7];
	increment= (int32 *)argv[8];
	the_count= (int32 *)argv[9];
	sds_data= (void *)argv[10];

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C read_sds_eos\n");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	fprintf(stdout, "\tData_name: \t%s\r\n", data_name->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/

	/* First Look at the SDS */
	sd_id=SDstart(filename->s,DFACC_RDONLY);
	if(sd_id == -1)
	{
	   fprintf(stderr,"Unable to open BDS file for SDS.\n");
	   result = -1;
	   return result;
	}

	printf("in read_sds_eos, index = %d\n", *index);
	/* index=SDnametoindex(sd_id,data_name->s);
	if(index == -1)
	{
	   fprintf(stderr,"Could not find %s SDS\n", data_name->s);
	   status = SDend(sd_id);
	   result = -1;
	   return result;
	} */

	sds_id=SDselect(sd_id,*index);
	if(sds_id == -1)
	{
	   fprintf(stderr,"Could not get sds_id for %s\n", data_name->s);
	   status = SDend(sd_id);
	   result = -1;
	   return result;
	}
/*
	status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);
	if(status == -1)
	{
	   fprintf(stderr,"Could not get information for %s\n", data_name->s);
	   status = SDendaccess(sds_id);
	   status = SDend(sd_id);
	   result = -1;
	   return result;
	}
	printf("sds_name = %s\n", sds_name);
*/

	number_data = 1;
	for (i=0; i<*rank; i++)
	{
		start[i] = starting_record[i];
		edge[i] = the_count[i];
		stride[i] = increment[i];
		number_data *= the_count[i];
	}

	switch(*number_type)
	{
	   case 5:
	   case 6:
	   case 20:
	   case 22:
	   case 24:
		{
		   status = SDreaddata(sds_id, start, stride, edge, (void *) sds_data);
		   if(status == -1)
		   {
		      fprintf(stderr, "ERROR for reading %s SDS.\n", data_name->s);
		      result = -1;
		   }
		   else
		   {
		      result = 0;
		   }
		   status = SDendaccess(sds_id);
		   status = SDend(sd_id);
		   return result;
		}
	   case 21:
		{
		   data_size = sizeof(uint8);
		   break;
		}
	   case 23:
		{
		   data_size = sizeof(uint16);
		   break;
		}
	  case 25:
		{
		   data_size = sizeof(uint32);
		   break;
		}
	   default:
		{
		   fprintf(stderr, "Unknow type for SDS: %d.\n", *number_type);
		   status = SDendaccess(sds_id);
		   status = SDend(sd_id);
		   result = -1;
		   return result;
		}
	}


	data_buffer = (uint8 *)malloc(number_data * data_size);
	if(data_buffer == NULL)
	{
	   fprintf(stderr, "Unable to allocate memory.\n");
	   status = SDendaccess(sds_id);
	   status = SDend(sd_id);
	   result = -1;
	   return result;
	}

	status = SDreaddata(sds_id, start, stride, edge, (void *) data_buffer);
	if(status == -1)
	{
	   fprintf(stderr, "ERROR for reading %s SDS.\n", data_name->s);
	   if(data_buffer != NULL) free(data_buffer);
	   status = SDendaccess(sds_id);
	   status = SDend(sd_id);
	   result = -1;
	   return result;
	}

	 ptr_buffer = data_buffer;

	 for(i = 0 ; i < number_data; i++ )
	 {
		switch(*number_type)
		{
		   case 21:
		   {
			memcpy(&tempuint8, ptr_buffer, data_size);
			((int16 *) sds_data)[i] = (int16)tempuint8;
			break;
		   }
		   case 23:
		   {
			memcpy(&tempuint16, ptr_buffer, data_size);
			((int32 *) sds_data)[i] = (int32)tempuint16;
			break;
		   }
		   case 25:
		   {
			memcpy(&tempuint32, ptr_buffer, data_size);
			((float64 *) sds_data)[i] = (float64)tempuint32;
			break;
		   }
		   default :
		   {
		      fprintf(stderr, "Unknow type for SDS: %d.\n", *number_type);
		      if(data_buffer != NULL) 
			free(data_buffer);
		      status = SDendaccess(sds_id);
		      status = SDend(sd_id);
		      result = -1;
		      return result;
		   }
		}
	    ptr_buffer += data_size;
	 }

	fprintf(stdout, "Completed reading SDS %s in read_sds.\n", data_name->s);
	if(data_buffer != NULL) 
	   free(data_buffer);

	status = SDendaccess(sds_id);
	status = SDend(sd_id);

	result = 0;

   return(result);
}
/*
---------------------------------------------------------------
	get_vdata_fields_eos
---------------------------------------------------------------
*/
char    *get_vdata_fields_eos(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	char            dummy_names[4096];
	int32           current_vid;
	int32           current_vref;
	int32           previous_vref;
	int32           first_vref;
	char            field_names[4096];
	int32           file_id;
	int             i;
	int32           number_of_fields;
	int             *field_quantity;
	char            result[4096];
	int32           size_of_data;
	int             size_of_single_record;
	int32           status;
	char            vdata_name[256];
	int             num_match;
	int32           *vdata_ref;
	int32           *num_vdata;
	STRING          *data_name;     
	STRING          *filename;      /* Pointer to IDL string structure      */
	char            *the_fields[100];


	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			data_name,
			ending_sample,
			bds_data
	*/
	if(argc != 5)
	{
		fprintf(stderr,"get_vdata_fields_eos: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name= (STRING *) argv[1];
	field_quantity=(int *)argv[2];
	num_vdata=(int32 *)argv[3];
	vdata_ref=(int32 *)argv[4];

	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_vdata_fields_eos");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	fprintf(stdout, "\tData Name: \t%s\r\n", data_name->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/
	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	/* fprintf(stdout,"\tThe file id is: \t%i\n"); */

	if (file_id!=-1)
	{
		Vstart(file_id);
		/* current_vref=VSgetid(file_id, -1); */

		current_vref=VSfind(file_id, data_name->s);
/*              if (current_vref!=-1)  */
		if (current_vref)
		{
		   first_vref=current_vref;
		   num_match = 1;
		   while (num_match != *num_vdata )
		   {
			previous_vref=current_vref;
			current_vref=VSgetid(file_id, previous_vref);
			if(current_vref == -1)
			{
			   current_vref = first_vref;
			   break;
			}
			current_vid=VSattach(file_id,current_vref,"r");
			if(current_vid == -1)
			{
			   current_vref = first_vref;
			   break;
			}
			VSgetname(current_vid,vdata_name);
			if(strcmp(vdata_name, data_name->s) == 0) num_match++;
			VSdetach(current_vid);
		   } 
		   *vdata_ref = current_vref;
		   current_vid=VSattach(file_id,current_vref,"r");
		   if (current_vid!=-1)
		   {
			number_of_fields=VSgetfields(current_vid, 
					dummy_names);
			*field_quantity=number_of_fields;
			fprintf(stdout,"\tNumber of Fields = \t%i\n",number_of_fields);
			set_field_names(field_names,number_of_fields,current_vid); 
			/*
			for (i=0; i<number_of_fields; i++) 
			{
				the_fields[i]=VFfieldname(current_vid,i);
				fprintf(stdout,"\tFields[%i]=%s\n",i,the_fields[i]);
			}
			*/

			VSdetach(current_vid);
			Vend(file_id);
			Hclose(file_id);
			strcpy(result,field_names);     
		   }
		   else
		   {
			fprintf(stdout,"\tCurrent Vid is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			strcpy(result,"BAD");
		   }
		}
		else
		{
			fprintf(stdout,"\tCurrent Vref is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			strcpy(result,"BAD");
		}
	}
	else
	{
		strcpy(result,"BAD");
	}

   return(result);
}

/*
---------------------------------------------------------------
	get_vdata_n_rec_eos
---------------------------------------------------------------
*/

long get_vdata_n_rec_eos(argc, argv)
	int argc;
	void *argv[];
{
	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           current_vid;
	int32           current_vref;
	STRING          *data_name;
	int32           file_id;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int32           *vdata_ref;
	int32           n_records;
	long            result;
	int32           status;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/* printf("get_vdata_n_rec was called...\n"); */
	if(argc != 3)
	{
		fprintf(stderr,"simple: Incorrect number of arguments\r\n");
		return(-1);  /* Signal an error */
	}
	/*
	** Cast the pointer in argv to the pointer variables
	*/

	filename= (STRING *) argv[0];
	data_name=(STRING *) argv[1];
	vdata_ref=(int32 *) argv[2];

	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id!=-1)
	{
		Vstart(file_id);
		current_vref=*vdata_ref;
/*              current_vref=VSfind(file_id, data_name->s); */
/*                if (current_vref!=-1)  */
		if (current_vref)
		{
		   current_vid=VSattach(file_id,current_vref,"r");
		   if (current_vid!=-1)
		   {

			status=VSinquire(current_vid,&n_records,
				NULL,NULL,NULL,NULL);
			if (status!=-1)
			{
				VSdetach(current_vid);
				Vend(file_id);
				Hclose(file_id);
				result=(long)n_records;
			}
			else
			{
				VSdetach(current_vid);
				Vend(file_id);
				Hclose(file_id);
				result=-1;
			}
		   }
		   else
		   {
			fprintf(stdout,"\tCurrent Vid is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			result=-1;
		   }            
		}
		else
		{
			fprintf(stdout,"\tCurrent Vref is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			result=-1;
		}
	}
	else
	{
		result=-1;
	}
	return(result);
}

/*
---------------------------------------------------------------
	get_vfield_types_eos
---------------------------------------------------------------
*/

char *get_vfield_types_eos(argc, argv)
	int argc;
	void *argv[];
{
	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           current_vfield_index;
	int32           current_vid;
	int32           current_vref;
	STRING          *data_name;
	int32           file_id;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int             i, j, k;
	int32           n_records;
	int32           *output_order;
	int32           *output_types;
	char            result[256];
	int32           status;
	int32           nattrs;
	char            attr_name[256];
	int32           data_type;
	int32           count;
	int32           attr_size;
	size_t          offset;
	int32		loc;
	char            *loc_start;
	char            *loc_end;
	char            *buffer;
	int             *vfield_count;
	int32           *vfield_indices;
	int32           *vdata_ref;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	printf("get_vfield_types was called...\n"); 
	if(argc != 7)
	{
		fprintf(stderr,"get_vfield_types: Incorrect number of arguments\r\n");
		strcpy(result, "BAD");
		return(result);  /* Signal an error */
	}
	/*
	** Cast the pointer in argv to the pointer variables
	*/
	/* Changed pointer casts to HDF types.  Kevin McIntire, 2/13/97 */
	filename= (STRING *) argv[0];
	data_name=(STRING *) argv[1];
	vdata_ref=(int32 *)argv[2];
	vfield_indices=(int32 *) argv[3];
	vfield_count=(int *) argv[4];
	output_types=(int32 *) argv[5];
	output_order=(int32 *) argv[6];

	/*
	for (i=0; i<*vfield_count; i++)
	{
		output_order[i]=i*100;
		output_types[i]=i*200;
	}
	*/
	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_vfield_types");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: %s\r\n", filename->s );
	fprintf(stdout, "\tDataname: %s\r\n", data_name->s );
	for (i=0; i<*vfield_count; i++)
	{
		fprintf(stdout, "\tFieldindices: \t%i\r\n", vfield_indices[i] );
	}

	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");

	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id!=-1)
	{
		Vstart(file_id);
		current_vref=*vdata_ref;
/*              current_vref=VSfind(file_id, data_name->s); */
/*                if (current_vref!=-1)  */
		if (current_vref)
		{
		   current_vid=VSattach(file_id,current_vref,"r");
		   if (current_vid!=-1)
		   {
			for (i=0; i<*vfield_count; i++)
			{
				/* Convert name to field_index */
				current_vfield_index=vfield_indices[i];
				status=VFfieldtype(current_vid,current_vfield_index);
				if (status!=-1)
				{
					output_types[i]=status;
					/* printf("The field type is %d\n",output_types[i]); */
					/* result=0; */
				}
				else
				{
					fprintf(stdout,"\tVFfieldtype is bad!\n");
					strcpy(result, "BAD");
					break;
				}
				status=VFfieldorder(current_vid,current_vfield_index);
				if (status!=-1)
				{
					output_order[i]=status;
					/* printf("The field order is %d\n",output_order[i]); */
					/* result=0; */
				}
				else
				{
					fprintf(stdout,"\tVFfieldorder is bad!\n");
					strcpy(result, "BAD");
					break;
				}
				strcpy(result, "BAD");
				nattrs = VSfnattrs(current_vid, current_vfield_index);
				/* printf("nattrs = %d\n", nattrs); */
				if(nattrs != 0)
				{
				    k = 0;
				    for(j = 0; j < nattrs; j++)
				    {
					status = VSattrinfo(current_vid, current_vfield_index, j, attr_name, &data_type, &count, &attr_size);
					if(status == -1) continue;
					/* printf("%d: attr_name = %s  data_type = %d   count = %d   attr_size = %d\n", \
						j, attr_name, data_type, count, attr_size); */
					buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
					if(buffer != NULL)
					{
						status = VSgetattr(current_vid, current_vfield_index, j, buffer);
						if(status != -1)
						{
							*(buffer+count*DFKNTsize(data_type)) = NULL;
							/* printf("buffer = %s\n", buffer); */
							/* loc_start = strstr(attr_name, "units");
							if(loc_start != NULL) */
							loc = strcmp(attr_name, "units");
							if(loc == 0)
							{
								strcpy(result, buffer);
								k = 1;
							}
							else
							{
								loc_start = strstr(buffer, "unit is");
								if(loc_start != NULL)
								{
									loc_start = loc_start + 8;
									offset = strspn(loc_start, " ");
									loc_start = loc_start + offset - 1;
									loc_end = strchr(loc_start, ';');
									if(loc_end != NULL)
									{
										*loc_end = NULL;
										strcpy(result, loc_start);
										k = 1;
									}
								}
								else
								{
									loc_start = strstr(buffer, "units is");
									if(loc_start != NULL)
									{
										loc_start = loc_start + 9;
										offset = strspn(loc_start, " ");
										loc_start = loc_start + offset - 1;
										loc_end = strchr(loc_start, ';');
										if(loc_end != NULL)
										{
											*loc_end = NULL;
											strcpy(result, loc_start);
											k = 1;
										}
							   		}
								}
							}
						}
						free(buffer);
						if(k == 1) break;
					}
				    }
				}
				/* printf("result = %s\n", result); */
				if( ! strcmp(result, "BAD"))
				{
				    nattrs = VSnattrs(current_vid);
				    /* printf("nattrs for vdata = %d\n", nattrs); */
				    if(nattrs != 0)
				    {
					k = 0;
					for(j = 0; j < nattrs; j++)
					{
					   status = VSattrinfo(current_vid, _HDF_VDATA, j, attr_name, &data_type, &count, &attr_size);
					   if(status == -1) continue;
					   /* printf("%d: attr_name = %s  data_type = %d   count = %d   attr_size = %d\n", \
						j, attr_name, data_type, count, attr_size); */
					   buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
					   if(buffer != NULL)
					   {
						status = VSgetattr(current_vid,  _HDF_VDATA, j, buffer);
						if(status != -1)
						{
							*(buffer+count*DFKNTsize(data_type)) = NULL;
							/* printf("buffer = %s\n", buffer); */
							/* loc_start = strstr(attr_name, "units");
							if(loc_start != NULL) */
							loc = strcmp(attr_name, "units");
							if(loc == 0)
							{
								strcpy(result, buffer);
								k = 1;
							}
							else
							{
								loc_start = strstr(buffer, "unit is");
								if(loc_start != NULL)
								{
									loc_start = loc_start + 8;
									offset = strspn(loc_start, " ");
									loc_start = loc_start + offset - 1;
									loc_end = strchr(loc_start, ';');
									if(loc_end != NULL)
									{
										*loc_end = NULL;
										strcpy(result, loc_start);
										k = 1;
									}
								}
								else
								{
									loc_start = strstr(buffer, "units is");
							   		if(loc_start != NULL)
							   		{
										loc_start = loc_start + 9;
										offset = strspn(loc_start, " ");
										loc_start = loc_start + offset - 1;
										loc_end = strchr(loc_start, ';');
										if(loc_end != NULL)
										{
											*loc_end = NULL;
											strcpy(result, loc_start);
											k = 1;
										}
							   		}
								}
							}
						}
						free(buffer);
						if(k == 1) break;
					   }
					}
				    }
				}
			}
			VSdetach(current_vid);
			Vend(file_id);
			Hclose(file_id);
		   }
		   else
		   {
			fprintf(stdout,"\tCurrent Vid is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			strcpy(result, "BAD");
		   }            
		}
		else
		{
			fprintf(stdout,"\tCurrent Vref is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			strcpy(result, "BAD");
		}
	}
	else
	{
		strcpy(result, "BAD");
	}
	return(result);
}

/*
---------------------------------------------------------------
	read_vdata_eos
---------------------------------------------------------------
*/
long read_vdata_eos(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	/* Changed buffer & ptr defs to be consistent with HDF.  Kevin McIntire, 2/13/97*/
	uint8           *buffer;
	uint8           *buffer_ptr;
	int32           buffer_size;
	char            dummy_names[4096];
	int32           current_order;
	int32           current_vid;
	int32           current_vref;
	int32           *ending_record;
	int             *field_count;
	STRING          *field_list;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int32           file_id;
	int32           *vdata_ref;
	char            find_name[64];
	int             *increment;
	int             len;
	/* Following dec was formerly int *output_data;.  I changed it to void *output_data. */
	/* IMPORTANT to cast it to correct data type on assignment.  An int * declaration would */
	/* not have allowed all data types to be inserted into it.  See the case statement below */
	/* which switches on the data type.  Kevin McIntire, 2/13/97 */
	void            *output_data;

	int32           number_of_records;
	int32           *number_order;
	int32           *number_types;
	int32           record_size;
	int32           size_of_single_record;
	/* Expanded number of data types covered.  Removed unneeded pointer delcarations.  Kevin McIntire, 2/13/97 */
	char8           tempC8;
	uchar8          tempUC8;
	int             temp_char;
	int8            tempI8;
	uint8           tempU8;
	int16           tempI16;
	uint16          tempU16;
	int32           tempI32;
	uint32          tempU32;
	float32         tempF32;
	float64         tempF64;
	int             i,j,k;
	int32           number_of_fields;
	long            result;
	int32           status;
	int32           *starting_record;
	int             this_index;
	int             *this_stuff;
	int32           type_byte_size;
	STRING          *vdata_name;    /* Pointer to IDL string structure      */

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
	-------------------------------------------------------------------------
		Parameters:
			filename,
			vdata_name,
			field_count,
			starting_record,
			ending_record,
			increment,
			field_list,
			number_types,
			number_order,
			output_data     

		** Insure that the correct number of arguments were passed in 
	-------------------------------------------------------------------------
	*/
	if(argc != 11)
	{
		fprintf(stderr,"read_vdata: Incorrect number of arguments\r\n");
		return(-1);  /* Signal an error */
	}

	/*
	----------------------------------------------------
		Transform the parameters into C variables       
	----------------------------------------------------
	*/
	filename        =(STRING *)argv[0];
	vdata_name      =(STRING *)argv[1];
	vdata_ref       =(int32 *)argv[2];
	field_count     =(int *)argv[3];
	starting_record =(int32 *)argv[4];
	ending_record   =(int32 *)argv[5];
	increment       =(int *)argv[6];
	field_list      =(STRING *)argv[7];     
	number_types    =(int32 *)argv[8];      /* Array */
	number_order    =(int32 *)argv[9];      /* Array */
	output_data     =(void *) argv[10];   

	/*
	----------------------------------------
		Print a header with the
		parameters.
	----------------------------------------
	*/
	
	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C read_vdata");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: %s\r\n", filename->s );
	fprintf(stdout, "\tVdata Name: %s\r\n", vdata_name->s );
	fprintf(stdout, "\tField Count: %i\r\n", *field_count);
	fprintf(stdout, "\tStart: %i\r\n", *starting_record);
	fprintf(stdout, "\tEnd: %i\r\n", *ending_record);
	fprintf(stdout, "\tIncrement: %i\r\n", *increment);
	fprintf(stdout, "\tFields: %s\r\n", field_list->s );
	for (i=0;i<*field_count; i++)
	{
		fprintf(stdout, "\tTypes: %i\r\n", *number_types);
		fprintf(stdout, "\tOrder: %i\r\n", *number_order);
	}
	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");

	/*
	----------------------------------------
		read_vdata Processing
	----------------------------------------
	*/

	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id!=-1)
	{
		Vstart(file_id);
		/* current_vref=VSgetid(file_id, -1); */

		current_vref=*vdata_ref;
/*              current_vref=VSfind(file_id, vdata_name->s); */
/*              if (current_vref!=-1)  */
		if (current_vref)
		{
		   current_vid=VSattach(file_id,current_vref,"r");
		   if (current_vid!=-1)
		   {
			if (number_of_fields!=-1)
			{
			   status=VSseek(current_vid,0);
			   if (status==-1)
			   {
				fprintf(stdout,"\tEven VSseek fails!\n");
				exit(-1);
			   }
			   status=VSsetfields(current_vid, field_list->s);
			   if (status!=-1)
			   {
				/* get the size_of_single_record from the HDF vsize field INFO */
				size_of_single_record=VSsizeof(current_vid,field_list->s);
				/*
				number_of_records=((*ending_record-(*starting_record)+1) / 
							(*increment));
				*/
				number_of_records=*ending_record + 1;
				buffer_size=number_of_records * size_of_single_record;
				if ((buffer_size>0) && (size_of_single_record>0))
				{
				      /* Added check on malloc return.  Cast result to HDF type.  Kevin McIntire, 2/13/97 */
				      if ( (buffer=(uint8 *) malloc(buffer_size)) == NULL)
				      {
					   printf("OUT OF MEMORY FOR VDATA BUFFER, %d BYTES\n",buffer_size);
					   return(-1);
				      }
				      /* Cast pointer to buffer to HDF type.  The vdata must have FULL_INTERLACE to be */
				      /* read field by field, so it is hard-coded here.  Kevin McIntire, 2/13/97 */
				      status=VSread(current_vid,(uint8 *) buffer,
						number_of_records,FULL_INTERLACE); 
				      if (status!=-1)
				      {
					   buffer_ptr=buffer;
					   for (i=0; i<number_of_records; i++)
					   {
						for (k=0; k<*field_count; k++)  
						{       
						   current_order=number_order[k];

						   /* Set this way for IDL structure */
						   for (j=0; j<current_order; j++)
						   {
						      /* Account for the 4 bytes from nfields */
/*                                                    this_index=i * (current_order + 1) + j + 1;*/
						      this_index=i * (current_order) + j;

						      /* fprintf(stdout, "i=%i j=%i index=%i ",
								i,j,this_index);    */
						      /* Added more data type to make this routine more generic. */
						      /* Note that output_data is cast to the appropriate type each */
						      /* time data is assigned to it.  */
						      /* The data transer routines were changed for each type to make */
						      /* the code a little clearer.  Kevin McIntire, 2/13/97 */
						      switch (number_types[k]) 
						      {
							case DFNT_CHAR8:
								memcpy(&tempC8, buffer_ptr, sizeof(char8));
								((int8 *)output_data)[this_index] = (int8)tempC8;
								buffer_ptr += sizeof(char8);
								break;
							case DFNT_UCHAR8:
								memcpy(&tempUC8, buffer_ptr, sizeof(uchar8));
								((int8 *)output_data)[this_index] = (int8)tempUC8;
								buffer_ptr += sizeof(uchar8);
								break;
							case DFNT_INT8:
								memcpy(&tempI8, buffer_ptr, sizeof(int8));
								((int8 *) output_data)[this_index] = tempI8;
								buffer_ptr += sizeof(int8);
								break;
							case DFNT_UINT8:
								memcpy(&tempU8, buffer_ptr, sizeof(uint8));
								/* ((uint8 *) output_data)[this_index] = tempU8; */
								((int16 *) output_data)[this_index] = (int16)tempU8;
								buffer_ptr += sizeof(uint8);
								break;
							case DFNT_INT16:
								memcpy(&tempI16, buffer_ptr, sizeof(int16));
								((int16 *) output_data)[this_index] = tempI16;
								buffer_ptr += sizeof(int16);
								break;
							case DFNT_UINT16:
								memcpy(&tempU16, buffer_ptr, sizeof(uint16));
								/* ((uint16 *) output_data)[this_index] = tempU16; */
								((int32 *) output_data)[this_index] = (int32)tempU16;
								buffer_ptr += sizeof(uint16);
								break;
							case DFNT_INT32:
								memcpy(&tempI32, buffer_ptr, sizeof(int32));
								((int32 *) output_data)[this_index] = tempI32;
								buffer_ptr += sizeof(int32);
								break;
							case DFNT_UINT32:
								memcpy(&tempU32, buffer_ptr, sizeof(uint32));
								/* ((uint32 *) output_data)[this_index] = tempU32; */
								((float64 *) output_data)[this_index] = (float64) tempU32;
								buffer_ptr += sizeof(uint32);
								break;
							case DFNT_FLOAT32:
								memcpy(&tempF32, buffer_ptr, sizeof(float32));
								((float32 *) output_data)[this_index] = tempF32;
								buffer_ptr += sizeof(float32);
								break;
							case DFNT_FLOAT64:
								memcpy(&tempF64, buffer_ptr,sizeof(float64));
								((float64 *) output_data)[this_index]=tempF64;
								buffer_ptr += sizeof(float64);
								break;
							}
						 }
					      } 
					   }

					   /* Done processing */
					   /* Check pointer before freeing memory.  Kevin McIntire, 2/13/97 */
					   if (buffer != NULL) free(buffer);
					   VSdetach(current_vid);
					   Vend(file_id);
					   Hclose(file_id);
					   result=0;

				      }
				      else
				      {
					   fprintf(stdout,"\tVSread is bad!\n");
					   /* Check pointer before freeing memory.  Kevin McIntire, 2/13/97 */
					   if (buffer != NULL) free(buffer);
					   VSdetach(current_vid);
					   Vend(file_id);
					   Hclose(file_id);
					   result=-1;
				      }
				}
				else
				{
				   fprintf(stdout,"\tBuffer size is bad!\n");
				   VSdetach(current_vid);
				   Vend(file_id);
				   Hclose(file_id);
				   result=-1;
				}
			   }
			   else
			   {
				fprintf(stdout,"\tVSsetfields is bad!\n");
				Vend(file_id);
				Hclose(file_id);
				result=-1;
			   }
			}
			else
			{
				fprintf(stdout,"\tNumber of fields is bad!\n");
				Vend(file_id);
				Hclose(file_id);
				result=-1;
			}
		   }
		   else
		   {
			fprintf(stdout,"\tCurrent Vid is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			result=-1;
		   }            
		}
		else
		{
			fprintf(stdout,"\tCurrent Vref is bad!\n");
			Vend(file_id);
			Hclose(file_id);
			result=-1;
		}
	}
	else
	{
		result=-1;
	}

/*
** Now print a line to seperate the output from this function and others
*/
   /* 
   fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
   fprintf(stdout,"Leaving C routine with result=%i\n",result);
   */

   return(result);
}

/*
---------------------------------------------------------------
	get_sds_attr
---------------------------------------------------------------
*/
char    *get_sds_attr(argc, argv)
	int argc;
	void *argv[];
{

	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           dimsizes[8];
	int             i,j;
	int32           *index;
	int32           nattrs;
	int32           number_type;
	int32           rank;
	char            result[17000];
	char            *ptr_dummy;
	int32           sd_id;
	int32           sds_id;
	char            sds_name[256];
	char            attr_name[256];
	int32           data_type;
	int32           count;
	int32           status;
	char            *buffer;
	char            *loc;
	STRING          *data_name;     
	STRING          *attr_type;     
	STRING          *filename;      /* Pointer to IDL string structure      */

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	/*
		Input Parameters:
			filename,
			index,
			attr_type
	*/
	if(argc != 3)
	{
		fprintf(stderr,"get_sds_attr: Incorrect number of arguments\r\n");
		strcpy(result,"BAD");
		return(result);  /* Signal an error */
	}

	/*
		** Cast the pointer in argv to the pointer variables
	*/
	filename= (STRING *) argv[0];
	data_name=(STRING *) argv[1];
	index= (int32 *) argv[2];

	result[0] = '\0';
	/*
	** Now print a header indicating that we are in the called C function
	** The "\r" was added to insure correct operation on Solaris systems
	*/   
	
	fprintf(stdout,
	"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_sds_attr\n");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: \t%s\r\n", filename->s );
	fprintf(stdout, "\tSDS name: \t%s\r\n", data_name->s );

	/*
	----------------------------------------
		Process the Data        
	----------------------------------------
	*/

	/* First Look at the SDS */
	sd_id=SDstart(filename->s,DFACC_RDONLY);
	if(sd_id == -1)
	{
	   fprintf(stderr,"Problem to start HDF file: %s\n",filename->s);
	   strcpy(result, "BAD");
	   return(result);
	}
	/* index=SDnametoindex(sd_id,data_name->s); */
	/* printf("in get_sds_attr, index = %d\n", *index); */
	sds_id=SDselect(sd_id,*index);
	if(sds_id == -1)
	{
	   fprintf(stderr,"Problem to access SDS: %s\n",data_name->s);
	   strcpy(result, "BAD");
	   status=SDend(sd_id);
	   return(result);
	}
	status=SDgetinfo(sds_id,sds_name,&rank,dimsizes,&number_type,&nattrs);
	if (status == -1)
	{
	   strcpy(result, "BAD");
	   status=SDendaccess(sds_id);
	   status=SDend(sds_id);
	   return(result);
	}

	if (nattrs == 0)
	{
	   strcpy(result, "No Data");
	   status=SDendaccess(sds_id);
	   status=SDend(sds_id);
	   return(result);
	}

/*      printf("sds_name = %s\n", sds_name); */
/*      printf("attribute type = %s\n", attr_type->s); */
/*      printf("nattrs = %d\n", nattrs); */
	j = 0;
	for (i=0; i<nattrs; i++)
	{
	   status = SDattrinfo(sds_id, i, attr_name, &data_type, &count);
	   if(status == -1)
	   {
		strcpy(result, "BAD");
		status=SDendaccess(sds_id);
		status=SDend(sds_id);
		return(result);
	   }
	   else
	   {
/*
		printf("attr_name = %s.\n", attr_name);
		printf("count = %d.\n", count);
		printf("data_type = %d.\n", data_type);
*/
/*
		loc = strstr(attr_name, attr_type->s);
		if(loc == NULL) continue;
*/
		buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
		if(buffer == NULL)
		{
			strcpy(result, "BAD");
			status=SDendaccess(sds_id);
			status=SDend(sds_id);
			return(result);
		}
		*buffer = NULL;
		status = SDreadattr(sds_id, i, buffer);
		if(status == -1)
		{
			fprintf(stderr, "Error to read attribute: %s\n",attr_name);
			free(buffer);
			continue;
		}
		ptr_dummy=get_attr_string(buffer, data_type, count);
/*              printf("ptr_dummy = %s\n", ptr_dummy); */
		if(j == 0)
		{
		   strcpy(result, attr_name);
		   strcat(result, ":  ");
		   strcat(result, ptr_dummy);
		}
		else
		{
		   strcat(result, "|");
		   strcat(result, attr_name);
		   strcat(result, ":  ");
		   strcat(result, ptr_dummy);
		}
/*              printf("result = %s\n", result); */
		j++;
		free(buffer);
		if (strlen(result) > 16950) break;
	   }
	}
	status=SDendaccess(sds_id);
	status=SDend(sd_id);

   return(result);
}

/*
---------------------------------------------------------------
	get_vfield_attr
---------------------------------------------------------------
*/

char *get_vfield_attr(argc, argv)
	int argc;
	void *argv[];
{
	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           current_vfield_index;
	int32           current_vid;
	int32           current_vref;
	STRING          *data_name;
	int32           file_id;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int             i, j, k;
	char            result[8196];
	char            *ptr_dummy;
	int32           status;
	int32           nattrs;
	char            attr_name[256];
	char            vdata_name[256];
	int32           data_type;
	int32           count;
	int32           attr_size;
	size_t          offset;
	char            *loc_start;
	char            *loc_end;
	char            *buffer;
	int32           *vdata_ref;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	printf("get_vfield_attr was called...\n"); 
	if(argc != 3)
	{
		fprintf(stderr,"get_vfield_attr: Incorrect number of arguments\r\n");
		strcpy(result, "BAD");
		return(result);  /* Signal an error */
	}
	/*
	** Cast the pointer in argv to the pointer variables
	*/
	/* Changed pointer casts to HDF types.  Kevin McIntire, 2/13/97 */
	filename= (STRING *) argv[0];
	data_name=(STRING *) argv[1];
	vdata_ref=(int32 *) argv[2];

	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_vfield_attr");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: %s\r\n", filename->s );
	fprintf(stdout, "\tDataname: %s\r\n", data_name->s );
	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");

	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id==-1)
	{
	   fprintf(stderr,"Problem to open HDF file!\n");
	   strcpy(result, "BAD");
	   return(result);
	}
	Vstart(file_id);
	current_vref = *vdata_ref;
/*         current_vref=VSfind(file_id, data_name->s); */
/*         if (current_vref!=-1)  */
	if (current_vref==0)
	{
	   fprintf(stderr,"current_vref is bad!\n");
	   strcpy(result,"BAD");
	   Vend(file_id);
	   Hclose(file_id);
	   return(result);
	}
	current_vid=VSattach(file_id,current_vref,"r");
	if (current_vid==-1)
	{
	   fprintf(stderr,"current_vid is bad!\n");
	   strcpy(result,"BAD");
	   Vend(file_id);
	   Hclose(file_id);
	   return(result);
	}
	j = 0;
	result[0]=NULL;
	status = VSfindex(current_vid,data_name->s,&current_vfield_index);
	/* printf("current_vfield_index = %d\n", current_vfield_index); */
	if(status != -1)
	{
	   nattrs = VSfnattrs(current_vid, current_vfield_index);
	   /* printf("nattrs = %d\n", nattrs); */
	   if(nattrs != 0)
	   {
		/* j = 0; */
		for(i = 0; i < nattrs; i++)
		{
		   status = VSattrinfo(current_vid, current_vfield_index, i, attr_name, &data_type, &count, &attr_size);
		   if(status == -1) continue;
		   /* printf("%d: attr_name = %s  data_type = %d   count = %d   attr_size = %d\n", \
			i, attr_name, data_type, count, attr_size); */
		   buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
		   if(buffer == NULL)
		   {
			fprintf(stderr,"Unable to allocate memory for attribute!\n");
			strcpy(result,"BAD");
			VSdetach(current_vid);
			Vend(file_id);
			Hclose(file_id);
			return(result);
		   }
		   status = VSgetattr(current_vid, current_vfield_index, i, buffer);
		   if(status == -1) continue;
		   ptr_dummy=get_attr_string(buffer, data_type, count);
		   /* printf("the value = %s\n",ptr_dummy); */
		   /* printf("the length of value = %d\n", strlen(ptr_dummy)); */
		   if(strlen(ptr_dummy) == 0) continue;
		   if(j == 0)
		   {
			strcpy(result, attr_name);
			strcat(result, ":  ");
			strcat(result, ptr_dummy);
		   }
		   else
		   {
			strcat(result, "|");
			strcat(result, attr_name);
			strcat(result, ":  ");
			strcat(result, ptr_dummy);
		   }
		   j++;
		   free(buffer);
		}
/*
		if(strlen(result))
		{
			VSdetach(current_vid);
			Vend(file_id);
			Hclose(file_id);
			return(result);
		}
*/
	   }
	}
/*      Look for attributes for Vdata  */
	nattrs = VSnattrs(current_vid);
	/* printf("nattrs for vdata = %d\n", nattrs); */
	if(nattrs == 0)
	{
	   if(j == 0) strcpy(result,"No Data");
		VSdetach(current_vid);
		Vend(file_id);
		Hclose(file_id);
		return(result);
	}
	k = 0;
/*
	j = 0;
	result[0]=NULL;
*/
	for(i = 0; i < nattrs; i++)
	{
	   status = VSattrinfo(current_vid, _HDF_VDATA, i, attr_name, &data_type, &count, &attr_size);
	   if(status == -1) continue;
	   /* printf("%d: attr_name = %s  data_type = %d   count = %d   attr_size = %d\n", \
		i, attr_name, data_type, count, attr_size); */
	   buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
	   if(buffer == NULL)
	   {
		strcpy(result,"BAD");
		VSdetach(current_vid);
		Vend(file_id);
		Hclose(file_id);
		return(result);
	   }
	   status = VSgetattr(current_vid,  _HDF_VDATA, i, buffer);
	   if(status == -1) continue;
	   ptr_dummy=get_attr_string(buffer, data_type, count);
	   /* printf("the length of value = %d\n", strlen(ptr_dummy)); */
	   /* printf("the value = %s\n",ptr_dummy); */
	   if(strlen(ptr_dummy) == 0) continue;
	   if(k == 0)
	   {
		status = VSgetname(current_vid, vdata_name);
		if(status != -1)
		{
		    if(j == 0)
		    {
			strcpy(result, vdata_name);
		    }
		    else
		    {
			strcat(result, "| |");
			strcat(result, vdata_name);
		    }
		    strcat(result, " Global attributes:");
		}
		else
		{
		    if(j == 0)
		    {
			strcpy(result, "Global attributes:");
		    }
		    else
		    {
			strcat(result, "| |Global attributes:");
		    }
		}
		k = 1;
	   }
/*		    
	   if(j == 0)
	   {
		strcpy(result, attr_name);
		strcat(result, ":  ");
		strcat(result, ptr_dummy);
	   }
	   else
	   {
*/
		strcat(result, "|");
		strcat(result, attr_name);
		strcat(result, ":  ");
		strcat(result, ptr_dummy);
/*	   } */
	   j++;
	   free(buffer);
	   if (strlen(result) > 16950) break;
	}
	if(strlen(result) == 0)
	{
	   strcpy(result,"No Data");
	}
	VSdetach(current_vid);
	Vend(file_id);
	Hclose(file_id);
	return(result);
}

/*
---------------------------------------------------------------
	get_vfield_unit
---------------------------------------------------------------
*/

char *get_vfield_unit(argc, argv)
	int argc;
	void *argv[];
{
	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           current_vfield_index;
	int32           current_vid;
	int32           current_vref;
	STRING          *data_name;
	int32           file_id;
	STRING          *filename;      /* Pointer to IDL string structure      */
	int             i, j, k;
	char            result[256];
	int32           status;
	int32           nattrs;
	char            attr_name[256];
	int32           data_type;
	int32           count;
	int32           attr_size;
	size_t          offset;
	int32		loc;
	char            *loc_start;
	char            *loc_end;
	char            *buffer;
	int32           *vfield_indices;
	int32           *vdata_ref;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	printf("get_vfield_unit was called...\n"); 
	if(argc != 4)
	{
	    fprintf(stderr,"get_vfield_types: Incorrect number of arguments\r\n");
	    strcpy(result, "BAD");
	    return(result);  /* Signal an error */
	}
	/*
	** Cast the pointer in argv to the pointer variables
	*/
	/* Changed pointer casts to HDF types.  Kevin McIntire, 2/13/97 */
	filename= (STRING *) argv[0];
	data_name=(STRING *) argv[1];
	vdata_ref=(int32 *)argv[2];
	vfield_indices=(int32 *) argv[3];

	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C get_vfield_unit");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\tFilename: %s\r\n", filename->s );
	fprintf(stdout, "\tDataname: %s\r\n", data_name->s );

	strcpy(result, "BAD");

	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id!=-1)
	{
	    Vstart(file_id);
	    current_vref=*vdata_ref;
/*	    current_vref=VSfind(file_id, data_name->s); */
/*	    if (current_vref!=-1)  */
	    if (current_vref)
	    {
		current_vid=VSattach(file_id,current_vref,"r");
		if (current_vid!=-1)
		{
		    /* Convert name to field_index */
		    current_vfield_index=vfield_indices[0];
		    nattrs = VSfnattrs(current_vid, current_vfield_index);
		    /* printf("nattrs = %d\n", nattrs); */
		    if(nattrs != 0)
		    {
			k = 0;
			for(j = 0; j < nattrs; j++)
			{
			    status = VSattrinfo(current_vid, current_vfield_index, j, attr_name, &data_type, &count, &attr_size);
			    if(status == -1) continue;
			    /* printf("%d: attr_name = %s  data_type = %d   count = %d   attr_size = %d\n", \
				j, attr_name, data_type, count, attr_size); */
			    buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
			    if(buffer != NULL)
			    {
				status = VSgetattr(current_vid, current_vfield_index, j, buffer);
				if(status != -1)
				{
				    *(buffer+count*DFKNTsize(data_type)) = NULL;
				    /* printf("buffer = %s\n", buffer); */
				    /* loc_start = strstr(attr_name, "units");
				    if(loc_start != NULL) */
				    loc = strcmp(attr_name, "units");
				    if(loc == 0)
				    {
					strcpy(result, buffer);
					k = 1;
				    }
				    else
				    {
					loc_start = strstr(buffer, "unit is");
					if(loc_start != NULL)
					{
					    loc_start = loc_start + 8;
					    offset = strspn(loc_start, " ");
					    loc_start = loc_start + offset - 1;
					    loc_end = strchr(loc_start, ';');
					    if(loc_end != NULL)
					    {
						*loc_end = NULL;
						strcpy(result, loc_start);
						k = 1;
					    }
					}
					else
					{
					    loc_start = strstr(buffer, "units is");
					    if(loc_start != NULL)
					    {
						loc_start = loc_start + 9;
						offset = strspn(loc_start, " ");
						loc_start = loc_start + offset - 1;
						loc_end = strchr(loc_start, ';');
						if(loc_end != NULL)
						{
						    *loc_end = NULL;
						    strcpy(result, loc_start);
						    k = 1;
						}
					    }
					}
				    }
				}
				free(buffer);
				if(k == 1) break;
			    }
			}
		    }
		    /* printf("result = %s\n", result); */
		    if( ! strcmp(result, "BAD"))
		    {
			nattrs = VSnattrs(current_vid);
			/* printf("nattrs for vdata = %d\n", nattrs); */
			if(nattrs != 0)
			{
			    k = 0;
			    for(j = 0; j < nattrs; j++)
			    {
				status = VSattrinfo(current_vid, _HDF_VDATA, j, attr_name, &data_type, &count, &attr_size);
				if(status == -1) continue;
				/* printf("%d: attr_name = %s  data_type = %d   count = %d   attr_size = %d\n", \
				    j, attr_name, data_type, count, attr_size); */
				buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
				if(buffer != NULL)
				{
				    status = VSgetattr(current_vid,  _HDF_VDATA, j, buffer);
				    if(status != -1)
				    {
					*(buffer+count*DFKNTsize(data_type)) = NULL;
					/* printf("buffer = %s\n", buffer); */
					/* loc_start = strstr(attr_name, "units");
					if(loc_start != NULL) */
					loc = strcmp(attr_name, "units");
					if(loc == 0)
					{
					    strcpy(result, buffer);
					    k = 1;
					}
					else
					{
					    loc_start = strstr(buffer, "unit is");
					    if(loc_start != NULL)
					    {
						loc_start = loc_start + 8;
						offset = strspn(loc_start, " ");
						loc_start = loc_start + offset - 1;
						loc_end = strchr(loc_start, ';');
						if(loc_end != NULL)
						{
						    *loc_end = NULL;
						    strcpy(result, loc_start);
						    k = 1;
						}
					    }
					    else
					    {
						loc_start = strstr(buffer, "units is");
						if(loc_start != NULL)
						{
						    loc_start = loc_start + 9;
						    offset = strspn(loc_start, " ");
						    loc_start = loc_start + offset - 1;
						    loc_end = strchr(loc_start, ';');
						    if(loc_end != NULL)
						    {
							*loc_end = NULL;
							strcpy(result, loc_start);
							k = 1;
						    }
						}
					    }
					}
				    }
				    free(buffer);
				    if(k == 1) break;
				}
			    }
			}
		    }
		    VSdetach(current_vid);
		    Vend(file_id);
		    Hclose(file_id);
		}
		else
		{
		    fprintf(stdout,"\tCurrent Vid is bad!\n");
		    Vend(file_id);
		    Hclose(file_id);
		    strcpy(result, "BAD");
		}            
	    }
	    else
	    {
		fprintf(stdout,"\tCurrent Vref is bad!\n");
		Vend(file_id);
		Hclose(file_id);
		strcpy(result, "BAD");
	    }
	}
	else
	{
		strcpy(result, "BAD");
	}
	/* printf("result = %s\n", result); */
	return(result);
}

/*
---------------------------------------------------------------
	write_vfield_attr
---------------------------------------------------------------
*/

char *write_vfield_attr(argc, argv)
	int argc;
	void *argv[];
{
	/*
	----------------------------------------
		Local variables
	----------------------------------------
	*/
	int32           current_vfield_index;
	int32           current_vid;
	int32           current_vref;
	int32           new_vfield_index;
	int32           new_vid;
	int32           new_vref;
	STRING          *data_name;
	STRING          *new_data_name;
	int32           file_id;
	int32           new_file_id;
	STRING          *filename;      /* Pointer to IDL string structure      */
	STRING          *new_file;      /* Pointer to IDL string structure      */
	int             i, j;
	char            result[8196];
	char            *ptr_dummy;
	int32           status;
	int32           nattrs;
	char            attr_name[256];
	int32           data_type;
	int32           count;
	int32           attr_size;
	size_t          offset;
	char            *loc_start;
	char            *loc_end;
	char            *buffer;
	int32           *vdata_ref;
	int32           *new_field_index;

	/*
	----------------------------------------
		Start of Code   
	----------------------------------------
	*/
	printf("write_vfield_attr was called...\n"); 
	if(argc != 6)
	{
		fprintf(stderr,"write_vfield_attr: Incorrect number of arguments\r\n");
		strcpy(result, "BAD");
		return(result);  /* Signal an error */
	}
	/*
	** Cast the pointer in argv to the pointer variables
	*/
	/* Changed pointer casts to HDF types.  Kevin McIntire, 2/13/97 */
	filename= (STRING *) argv[0];
	data_name=(STRING *) argv[1];
	vdata_ref=(int32 *) argv[2];
	new_file= (STRING *) argv[3];
	new_data_name=(STRING *) argv[4];
	new_field_index=(int32 *) argv[5];

	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");
	fprintf(stdout,"Inside C write_vfield_attr");
	fprintf(stdout,"(Called from IDL using CALL_EXTERNAL)\r\n\r\n");
	fprintf(stdout, "\told Filename: %s\r\n", filename->s );
	fprintf(stdout, "\tDataname: %s\r\n", data_name->s );
	fprintf(stdout, "\tnew Filename: %s\r\n", new_file->s );
	fprintf(stdout, "\tnew Dataname: %s\r\n", new_data_name->s );
	fprintf(stdout,"\r\n-----------------------------------------------------\r\n");

	file_id=Hopen(filename->s,DFACC_RDONLY,0);
	if (file_id==-1)
	{
	   fprintf(stderr,"Problem to open HDF file!\n");
	   strcpy(result, "BAD");
	   return(result);
	}
	Vstart(file_id);
	current_vref = *vdata_ref;
/*         current_vref=VSfind(file_id, data_name->s); */
/*         if (current_vref!=-1)  */
	if (current_vref==0)
	{
	   fprintf(stderr,"current_vref is bad!\n");
	   strcpy(result,"BAD");
	   Vend(file_id);
	   Hclose(file_id);
	   return(result);
	}
	current_vid=VSattach(file_id,current_vref,"r");
	if (current_vid==-1)
	{
	   fprintf(stderr,"current_vid is bad!\n");
	   strcpy(result,"BAD");
	   Vend(file_id);
	   Hclose(file_id);
	   return(result);
	}

	new_file_id=Hopen(new_file->s,DFACC_RDWR,0);
	if (new_file_id==-1)
	{
	    fprintf(stderr,"Problem to open new HDF file!\n");
	    VSdetach(current_vid);
	    Vend(file_id);
	    Hclose(file_id);
	    strcpy(result, "BAD");
	    return(result);
	}
	Vstart(new_file_id);
	new_vref=VSfind(new_file_id, new_data_name->s);
	if (new_vref==0)
	{
	    fprintf(stderr,"new_vref is bad!\n");
	    strcpy(result,"BAD");
	    VSdetach(current_vid);
	    Vend(file_id);
	    Hclose(file_id);
	    Hclose(new_file_id);
	    return(result);
	}
	new_vid=VSattach(new_file_id,new_vref,"w");
	if (new_vid==-1)
	{
	    fprintf(stderr,"new_vid is bad!\n");
	    strcpy(result,"BAD");
	    VSdetach(current_vid);
	    Vend(file_id);
	    Hclose(file_id);
	    Hclose(new_file_id);
	    return(result);
	}

	j = 0;
	result[0]=NULL;
	status = VSfindex(current_vid,data_name->s,&current_vfield_index);
	/* printf("current_vfield_index = %d\n", current_vfield_index); */
	if(status != -1)
	{
	   nattrs = VSfnattrs(current_vid, current_vfield_index);
	   /* printf("nattrs for field = %d\n", nattrs); */
	   if(nattrs != 0)
	   {
		for(i = 0; i < nattrs; i++)
		{
		   status = VSattrinfo(current_vid, current_vfield_index, i, attr_name, &data_type, &count, &attr_size);
		   if(status == -1) continue;
		   /* printf("%d: attr_name = %s  data_type = %d   count = %d   attr_size = %d\n", \
			i, attr_name, data_type, count, attr_size); */
		   buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
		   if(buffer == NULL)
		   {
			fprintf(stderr,"Unable to allocate memory for attribute!\n");
			strcpy(result,"BAD");
			VSdetach(current_vid);
			Vend(file_id);
			Hclose(file_id);
			VSdetach(new_vid);
			Vend(new_file_id);
			Hclose(new_file_id);
			return(result);
		   }
		   status = VSgetattr(current_vid, current_vfield_index, i, buffer);
		   if(status == -1) continue;
		   /* ptr_dummy=get_attr_string(buffer, data_type, count);
		   printf("attr_name = %s, value = %s\n", attr_name, ptr_dummy); */
		   status = VSsetattr(new_vid,*new_field_index,attr_name, data_type,count, buffer);
		   j++;
		   free(buffer);
		}
/*
		if(j != 0)
		{
		   strcpy(result,"GOOD");
		   VSdetach(current_vid);
		   Vend(file_id);
		   Hclose(file_id);
		   VSdetach(new_vid);
		   Vend(new_file_id);
		   Hclose(new_file_id);
		   return(result);
		}
*/
	   }
	}
/*      Look for attributes for Vdata  */
	nattrs = VSnattrs(current_vid);
	/* printf("nattrs for vdata = %d\n", nattrs); */
	if(nattrs == 0)
	{
	   if(j == 0 )
		 strcpy(result,"No Data");
	   else
		strcpy(result,"GOOD");
	   VSdetach(current_vid);
	   Vend(file_id);
	   Hclose(file_id);
	   VSdetach(new_vid);
	   Vend(new_file_id);
	   Hclose(new_file_id);
	   return(result);
	}
/*	result[0]=NULL; */
	for(i = 0; i < nattrs; i++)
	{
	   status = VSattrinfo(current_vid, _HDF_VDATA, i, attr_name, &data_type, &count, &attr_size);
	   if(status == -1) continue;
	   /* printf("%d: attr_name = %s  data_type = %d   count = %d   attr_size = %d\n", \
		i, attr_name, data_type, count, attr_size); */
	   buffer = (char *)malloc(count*DFKNTsize(data_type)+1);
	   if(buffer == NULL)
	   {
		strcpy(result,"BAD");
		VSdetach(current_vid);
		Vend(file_id);
		Hclose(file_id);
		VSdetach(new_vid);
		Vend(new_file_id);
		Hclose(new_file_id);
		return(result);
	   }
	   status = VSgetattr(current_vid,  _HDF_VDATA, i, buffer);
	   if(status == -1) continue;
	   /* ptr_dummy=get_attr_string(buffer, data_type, count);
	   printf("attr_name = %s, value = %s\n", attr_name, ptr_dummy); */
	   status = VSsetattr(new_vid,*new_field_index,attr_name, data_type,count, buffer);
	   j++;
	   free(buffer);
	}
/*	if(strlen(result) == 0) */
	if(j == 0)
	{
	   strcpy(result,"No Data");
	}
	else
	{
	    strcpy(result,"GOOD");
	}
	VSdetach(current_vid);
	Vend(file_id);
	Hclose(file_id);
	VSdetach(new_vid);
	Vend(new_file_id);
	Hclose(new_file_id);
	return(result);
}
